;*** SATNC - S53MV 27.03.2024 ***

;*** Tacke LPC23xx (58.9824MHz@kristal=14.7456MHz) ***
;P0.0 - digitalni izhod TXD UART3
;P0.1 - digitalni vhod RXD UART3
;P0.2 - digitalni izhod TXD UART0
;P0.3 - digitalni vhod RXD UART0
;P0.4 - digitalni vhod vezan na P0.6=SSEL (SSP1)
;P0.5 - dgitalni izhod /CS=CSN (SSP1)
;P0.6=SSEL - digitalni izhod vezan na P0.4
;P0.7=SCK1 - digitalni izhod (S)CLK (SSP1)
;P0.8=MISO1 - digitalni vhod (MI)SO (SSP1)
;P0.9=MOSI1 - digitalni izhod (MO)SI (SSP1)
;P0.10 - digitalni izhod TXD UART2
;P0.11 - digitalni vhod RXD UART2
;P0.15 - digitalni izhod TXD UART1
;P0.16 - digitalni vhod RXD UART1
;P0.17-22 - neuporabljeni digitalni vhodi
;P0.23-25=AD0.0-2 - analogni vhod (TLM#1)
;P0.26=AD0.3 - analogni vhod (TLM#2)
;P0.27-30 - neuporabljeni digitalni vhodi
;P1.0,1,4,8,9,10,14,15 - Ethernet RMII (digitalni)
;P1.16 - digitalni izhod /PTT
;P1.17 - digitalni vhod /DCD
;P1.18-29 - neuporabljeni digitalni vhodi
;P1.30 - digitalni izhod (masa za AD0.4)
;P1.31 - digitalni izhod (telekomanda)
;P2.0-9 - neuporabljeni digitalni vhodi (TRACE)
;P2.10 - digitalni vhod /bootloader, pullup na +3.3V 
;P2.11-13 - neuporabljeni digitalni vhodi
;P3.25-26 - neuporabljena digitalna vhoda
;P4.28 - digitalni izhod /RESET (SSP1)
;P4.29 - neuporabljen digitalni izhod

;*** Ukazi za delovanje zbirnika ARM ***
	AREA	RESET, CODE, READONLY, ALIGN=9
	ENTRY
	CODE32

;*** Dodelitev registrov in pomnilnika RAM ***
;R10 - osnovni naslov SSP1
;R11 - osnovni naslov RMII registrov
;R12 - osnovni naslov UART1
SKLAD	EQU	0x400000B0	;glavni sklad (navzdol ???)
FSKLAD	EQU	0x400000C0	;FIQ sklad (navzdol -8)

SLTXBLO	EQU	0x400000C0	;SLIP TX blok (32bit)

SLTXKAZ	EQU	0x400000CC	;SLIP TX kazalec (16bit)
SLTXSTA	EQU	0x400000CE	;SLIP TX stanje (8bit)
SPREKLP	EQU	0x400000CF	;PREKLOP stanje (8bit)
TPREKLP	EQU	0x400000D0	;PREKLOP cas (32bit)

ZPONOVI	EQU	0x400000D8	;PREKLOP zadnje ponavljanje (32bit)
POLINOM	EQU	0x400000DC	;PSEVNAK vsebina LFSR (32bit)
DVOJKAZ	EQU	0x400000E0	;kazalec v tabeli dvojnikov (16bit)
NODZIV	EQU	0x400000E2	;stevilo byte odziva, 0=izklop (16bit)
MIKRSEK	EQU	0x400000E4	;mikrosekundni stevec (16bit)
MILISEK	EQU	0x400000E6	;milisekundni stevec (16bit)
SEKUNDA	EQU	0x400000E8	;sekundni stevec (32bit)
NZANK	EQU	0x400000EC	;izmerjeno stevilo zank (32bit)
SZANK	EQU	0x400000F0	;stevec zank (32bit)
NDCD	EQU	0x400000F4	;izmerjeni promil DCD (16bit)
SDCD	EQU	0x400000F6	;stevec DCD (16bit)
NPTT	EQU	0x400000F8	;izmerjeni promil PTT (16bit)
SPTT	EQU	0x400000FA	;stevec PTT (16bit)
MPONOVI	EQU	0x400000FC	;minimum ponavljanja (8bit)
DPONOVI	EQU	0x400000FD	;neizvedena ponavljanja (8bit)
NICBLOK	EQU	0x400000FE	;zastavica nic blokov (8bit)
ZNACIN	EQU	0x400000FF	;zastavice (8bit):
;bit0(1)=(0)ukaz/(1)pogovor s tipkovnice
;bit1(2)=(0)uporabni/(1)vsi okvirji na zaslon
;bit2(4)=(0)izklop/(1)vklop izpisa stanja vsako sekundo
;bit3(8)=(0)zaslon/(1)okvir izpis znakov
;bit4(16)=(0)izklop/(1)vklop izpisa PHY vsako sekundo
;bit5(32)=(0)naslov36/(1)naslovHEX
;bit6(64)=(0)izklop/(1)vklop izpisa ADC vsako sekundo
;bit7(128)=(0)vklop/(1)izklop oddajnika

MCACHE	EQU	25		;velikost cacheja (max 25 vpisov)
ECACHE	EQU	0x40000100	;EthernetARP cache (MCACHE*16byte)
;Ethernet/ARP cache: max 25 vpisov po 16byte v 0x40000100-0x4000028F
;odmik 0 - IP stevilka (32bit)
;odmik 4 - MAC stevilka (48bit)
;odmik 10 - preostalo trajanje vpisa (16bit)
;odmik 12 - zakasnitev ARP odgovora (32bit)

MACMAC	EQU	0x40000294	;zadnji sogovornikov MAC/izracunani MAC (48bit)

ARMTAKT	EQU	0x4000029C	;takt ARM procesorja (32bit)
ARPCAS	EQU	0x400002A0	;cas oddaje ARP (32bit)
ETXBLOK	EQU	0x400002A4	;cakajoci blok na ARP (32bit)
ZASKAZ	EQU	0x400002A8	;kazalca zaslona (2*16bit)
ZASTRI	EQU	0x400002AC	;vmesnik za 1-3 znake (32bit)
MAXZAS	EQU	0x800		;dolzina vmesnika 2kbyte BATRAM

KRXBLOK	EQU	0x400002E0	;blok sprejemnika KSZ8851SNL
KTXBLOK	EQU	0x400002E4	;blok oddajnika KSZ8851SNL
PHYSTAT	EQU	0x400002E8	;stanje PHY (16bit)

KSZID	EQU	0x400002EC	;chip ID KSZ8851SNL (32bit)
PARTID	EQU	0x400002F0	;chip ID mikrokrmilnika (32bit)

SMETER	EQU	0x400002F8	;S-METER sprejetega okvirja iz AD0.1 (32bit)

SLISAL	EQU	0x40000300	;seznam slisanih radijskih oddajnikov (MAXSLIS*20byte)
;odmik 0 - slisani NBP naslov (32bit)
;odmik 4 - zadnji slisani SMETER (32bit)
;odmik 8 - skupno stevilo okvirjev (32bit)
;odmik 12 - skupno stevilo bajtov (32bit)
;odmik 16 - sekunda sprejema zadnjega okvirja (32bit)
MAXSLIS	EQU	20		;dolzina seznama slisanih

;parametri iz FLASH 0x40000500-0x400008FF (1024byte)
RAMFPAR	EQU	0x40000500	;zacetek parametrov iz FLASH (1024byte)
MNASLOV	EQU	0x40000500	;moj naslov (32bit)
TSLOT	EQU	0x40000504	;trajanje slota (us) (32bit)
TGLAVA	EQU	0x40000508	;dolzina glave (us) (32bit)
TREP	EQU	0x4000050C	;dolzina repa (us) (32bit)
TECEN1	EQU	0x40000510	;tecnoba z DCD /65536 (16bit)
TECEN2	EQU	0x40000512	;tecnoba brez DCD /65536 (16bit)
GESLO	EQU	0x40000514	;geslo za telekomando (32bit)
BAUDMOD	EQU	0x40000518	;baudrate modulo UART1 (16bit)
MINBLOK	EQU	0x4000051A	;najmanj prostih blokov (8bit)
NPONOVI	EQU	0x4000051B	;stevilo ponavljanj (8bit)
QUARTZ	EQU	0x4000051C	;mnogokratnik (4bit) in frekvenca (28bit) kristala
TPONOVI	EQU	0x40000520	;zakasnitev ponavljanj (32bit)
TIMEOUT	EQU	0x40000524	;Hayes/EthernetARP timeout (32bit)

NENOTA	EQU	0x40000528	;skala in enote ADC TLM (MENOTA*32BIT)
MENOTA	EQU	6
NASLOV	EQU	0x40000540	;naslov poti (MAXNAS*32bit)
INASLOV	EQU	0x40000580	;naslov IP poti (MAXNAS*32bit)
MAXNAS	EQU	16
JAVITXT	EQU	0x400005C0	;besedilo javljanja (MAXJAVI byte)
MAXJAVI	EQU	64		;najvecja dolzina besedila
ANASLOV	EQU	0x40000600	;naslov ARP tabele (MAXARP*(MAXNAS+1)*32bit)
AKORAK	EQU	68		;korak v tabeli ARP (1+MAXNAS)*4
MAXARP	EQU	10
ANACIN	EQU	0x400008A8	;nacin delovanja ARP (8bit)

LNASLOV	EQU	0x400008B0	;naslovi lokalne zanke (MAXLOK*32bit)
MAXLOK	EQU	8

DOMENA	EQU	0x400008E0	;IP domena (12byte)
TCACHE	EQU	0x400008EC	;EthernetARP cache trajanje (16bit)
ARPMODE	EQU	0x400008EE	;ARP mode: bit0(1)=vprasanje, bit1(2)=nepoklicani/zamujeni
GATEWAY	EQU	0x400008F0	;Privzeti prehod/maska (8byte)
ESPEED	EQU	0x400008F9	;Ethernet hitrost in MDIX (8bit)
ETHMAC	EQU	0x400008FA	;Ethernet MAC naslov (6byte)

ZNASLOV	EQU	0x40000900	;zacasni naslov za vnos ARP ((MAXNAS+1)*32bit)

TIPVMES	EQU	0x40000950	;vmesnik tipk ((MAXUKAZ+2)*8bit)
MAXUKAZ	EQU	170
KSTEVEC	EQU	0x400009FE	;perioda/stevec kokodakanja (2byte)

MAXBLOK	EQU	44		;RAMBLOK dolzine (MAXBLOK+1)*32bit
RAMBLOK	EQU	0x40000A00	;naslov blokov RAM

MAXVMES	EQU	47		;nnXVMES dolzine 2*16bit+MAXVMES*32bit
KTXVMES	EQU	0x40000AC0	;naslov vmesnika KTX ETH
HTPVMES	EQU	0x40000B80	;naslov potrdilnega vmesnika TX RMII
HTXVMES	EQU	0x40000C40	;naslov vmesnika TX RMII
HTZVMES	EQU	0x40000D00	;naslov zacasnega vmesnika TX RMII
SRXVMES	EQU	0x40000DC0	;naslov vmesnika RX SLIP
STXVMES	EQU	0x40000E80	;naslov vmesnika TX SLIP
ETXVMES	EQU	0x40000F40	;naslov vmesnika TX ETH

DVOJTAB	EQU	0x40001000	;tabela dvojnikov (4kbyte) lihi 0x1000!!!

;bloki pomnilnika (RAM, USBRAM), 20-45 blokov dolzine 1632=0x660
;odmik 0 - dolzina vsebine (16bit)
;odmik 2 - zacetek vsebine (16bit)
;odmik 4 - stevilo ponavljanj (8bit)
;odmik 5 - zakasnitev ponavljanja (8bit)
;odmik 4 - ARP IP4 cilj (32bit) samo v ETXBLOK

;IAP ukazi za programiranje FLASH 0x40007FA0-0x40007FFF
IAPCMD	EQU	0x40007FA0	;ukaz IAP
IAPRES	EQU	0x40007FA0	;rezultat IAP
POTBLOK	EQU	0x40007FA0	;kratki potrditveni blok (max 96byte)
POTMAX	EQU	16		;MAX stevilo potrditev v okvirju

;RMII RAM (16kB 0x7FE00000-0x7FE03FFF)
RXDESC	EQU	0x7FE00000	;RxDescriptor
RXSTAT	EQU	0x7FE00100	;RxStatus
RXDESCN	EQU	5		;RxDescriptorNumber-1
RXCONT	EQU	0x8000069F	;RxDescriptorControlWord=Int,1696-1
TXDESC	EQU	0x7FE00200	;TxDescriptor
TXSTAT	EQU	0x7FE00300	;TxStatus
TXDESCN	EQU	5		;TxDescriptorNumber-1 (TXDESCN+1=2*TXBLOKN)
TXBLOKN	EQU	3		;stevilo razlicnih oddajnih blokov
ZACERX	EQU	0x7FE003F2	;zacetni naslov blokov sprejemnika (-14)
ZACETX	EQU	0x7FE02BB2	;zacetni naslov blokov oddajnika (-14)
RMIIBLK	EQU	0x6A0		;dolzina bloka 1696

;baterijsko napajani RAM (2kbyte 0xE0084000-0xE00847FF, 32-bitni dostop)
BATRAM	EQU	0xE0084000	;vmesnik 2kbyte za ZNAK,ZASLON

;*** Tabela izjem na zacetnem naslovu 0 ***
IZJEME	B	ZACNI
	B	IZJEME	;Undef
	B	IZJEME	;SWI
	B	IZJEME	;PAbt
	B	IZJEME	;DAbt
	NOP		;Reserved Vector
	B	IZJEME	;IRQ
;*** Prekinitev FIQ - SLIP sprejem UART1 *** (R8-12.FIQ,sklad -8)+WINSRX
SLIPZAC	EQU	8	;zacetek podatkov lokalni/SLIP
WINSZAC	EQU	100
SLIPMIN	EQU	16	;min kazalec podatkov RX (8byte) lokalni/SLIP
WINSMIN	EQU	108
SLIPMAX	EQU	1616	;max kazalec podatkov RX (16byte rezerve)
;R8=blok, R9=ULSR/racun, R10=URBR/racun, R11=kazalec+zastavice, R12=UART1
	LDRB	R9,[R12,#ULSR]	;U1LSR - precitaj stanje v R9
	TST	R9,#0x01	;U1LSR[0]=RDR=0 napacni FIQ?
	SUBSEQ	PC,LR,#4	;izhod iz prekinitve FIQ?
SLIPRX1	LDRB	R10,[R12,#URBR]	;U1RBR - precitaj znak v R10
	CMP	R10,#SLFEND	;znak FEND - konec SLIP okvirja?
	BEQ	SLIPRX4
	MOVS	R8,R8		;ni sprejemnega bloka?
	SUBSEQ	PC,LR,#4	;izhod iz prekinitve FIQ?
	MOVS	R11,R11		;preveri zastavico v R11?
	BMI	SLIPRX3
	CMP	R10,#SLFESC	;znak FESC?
	BEQ	SLIPRX2
	CMP	R11,#SLIPMAX	;prostor v vmesniku?
	STRBCC	R10,[R8,R11]	;shrani byte in povecaj kazalec
	ADDCC	R11,R11,#1
	LDRB	R9,[R12,#ULSR]	;U1LSR - ReceiveDataReady?
	TST	R9,#0x01
	BNE	SLIPRX1
	SUBS	PC,LR,#4	;izhod iz prekinitve FIQ
	ALIGN	16
SLIPRX2	ORR	R11,R11,#0x80000000	;postavi zastavico v R11
	LDRB	R9,[R12,#ULSR]	;U1LSR - ReceiveDataReady?
	TST	R9,#0x01
	BNE	SLIPRX1
	SUBS	PC,LR,#4	;izhod iz prekinitve FIQ
	ALIGN	16
SLIPRX3	CMP	R10,#SLTFESC	;TFESC?
	MOV	R10,#SLFEND	;zamenjaj TFEND>FEND
	MOVEQ	R10,#SLFESC	;zamenjaj TFESC>FESC?
	BIC	R11,R11,#0x80000000	;brisi zastavico v R11
	CMP	R11,#SLIPMAX	;prostor v vmesniku?
	STRBCC	R10,[R8,R11]	;shrani byte in povecaj kazalec
	ADDCC	R11,R11,#1
	LDRB	R9,[R12,#ULSR]	;U1LSR - ReceiveDataReady?
	TST	R9,#0x01
	BNE	SLIPRX1
	SUBS	PC,LR,#4	;izhod iz prekinitve FIQ
	ALIGN	16
SLIPRX4	MOVS	R8,R8		;dodeli nov blok, ce ni sprejemnega bloka?
	BEQ	SLIPRX5
	BIC	R11,R11,#0xF0000000	;odstrani zastavice v R11
	LDRH	R9,[R8]		;sprejeti blok premajhen?
	CMP	R11,R9
	BCC	SLIPRX6
	LDRH	R9,[R8,#2]	;odstej zacetek za dolzino okvirja
	SUB	R11,R11,R9
	STRH	R11,[R8]
	LDR	R9,=SRXVMES	;sprejeti okvir v SRXVMES
	LDRH	R10,[R9]
	ADD	R10,R10,#1
	STR	R8,[R9,R10,LSL#2]
	CMP	R10,#MAXVMES
	MOVCS	R10,#0
	STRH	R10,[R9]
	MOV	R8,#0
	LDRB	R9,[R12,#ULSR]	;U1LSR - ReceiveDataReady?
	TST	R9,#0x01
	BNE	SLIPRX1
	SUBS	PC,LR,#4	;izhod iz prekinitve FIQ
	ALIGN	16
SLIPRX5	STMDB	SP!,{R0,LR}	;resi R0,LR
	BL	DODELI		;dodeli nov sprejemni blok
	LDMIA	SP!,{R0,LR}	;povrni R0,LR
	MOVS	R8,R8		;izprazni FIFO, ce ni bloka?
	BEQ	SLIPRX8
SLIPRX6	LDR	R9,=MNASLOV	;moj naslov in lokalno vkljucena?
	LDR	R10,[R9]
	LDR	R9,=LNASLOV
	LDR	R9,[R9]
	MOVS	R10,R10
	MOVEQ	R9,#0
	MOVS	R9,R9
	MOV	R10,#SLIPMIN	;postavi minimalno dolzino okvirja
	MOVEQ	R10,#WINSMIN
	STRH	R10,[R8]
	MOV	R11,#SLIPZAC	;kazalec R11 brisi napake in zacetek podatkov
	MOVEQ	R11,#WINSZAC
	STRH	R11,[R8,#2]
	LDRB	R9,[R12,#ULSR]	;U1LSR - ReceiveDataReady?
	TST	R9,#0x01
	BNE	SLIPRX1
	SUBS	PC,LR,#4	;izhod iz prekinitve FIQ
SLIPRX7	LDRB	R10,[R12,#URBR]	;U1RBR - izprazni FIFO, ko ni bloka
SLIPRX8	LDRB	R9,[R12,#ULSR]	;U1LSR - ReceiveDataReady?
	TST	R9,#0x01
	BNE	SLIPRX7
	SUBS	PC,LR,#4	;izhod iz prekinitve FIQ
	LTORG

;*** Oddaja SLIP okvirja na UART1 *** (R0-8,R12)
SLFEND	EQU	0xC0	;FEND
SLFESC	EQU	0xDB	;FESC
SLTFEND	EQU	0xDC	;TFEND
SLTFESC	EQU	0xDD	;TFESC
MAXFIFO	EQU	16	;dolzina TX FIFO
	ALIGN	16
SLIPTX	LDRB	R0,[R12,#ULSR]	;U1LSR bit5=TxHoldingRegisterEmpty?
	TST	R0,#0x20
	MOVEQ	PC,LR
	STR	LR,[SP,#-4]!
	MOV	R4,#MAXFIFO
	LDR	R0,=SLTXSTA	;SLTXSTA v R6
	LDRB	R6,[R0]
	LDR	R0,=SLTXKAZ	;SLTXKAZ v R7
	LDRH	R7,[R0]
	LDR	R0,=SLTXBLO	;SLTXBLO v R8
	LDR	R8,[R0]
	BL	SLIPTX1
	LDR	R0,=SLTXBLO	;R8 v SLTXBLO
	STR	R8,[R0]
	LDR	R0,=SLTXKAZ	;R7 v SLTXKAZ
	STRH	R7,[R0]
	LDR	R0,=SLTXSTA	;R6 v SLTXSTA
	STRB	R6,[R0]
	LDR	PC,[SP],#4
	LTORG
	ALIGN	16
SLIPTX1	SUBS	R4,R4,#1	;stej znake v FIFO?
	MOVMI	PC,LR
	CMP	R6,#1		;stanje=1 - oddaja vsebine?
	BNE	SLIPTX5
	LDRH	R0,[R8]		;konec podatkov?
	CMP	R7,R0
	BCS	SLIPTX2
	LDRH	R0,[R8,#2]	;precitaj znak v R1
	ADD	R0,R0,R7
	LDRB	R1,[R8,R0]
	ADD	R7,R7,#1	;povecaj kazalec
	CMP	R1,#SLFESC	;znak FESC?
	BEQ	SLIPTX3
	CMP	R1,#SLFEND	;znak FEND?
	BEQ	SLIPTX4
	STRB	R1,[R12,#UTHR]	;oddaj znak v U1THR
	B	SLIPTX1
SLIPTX2	MOV	R0,#SLFEND	;oddaj FEND na koncu v U1THR
	STRB	R0,[R12,#UTHR]
	STR	LR,[SP,#-4]!
	LDR	R0,=MNASLOV	;moj naslov in lokalno vkljucena?
	LDR	R1,[R0]
	LDR	R0,=LNASLOV
	LDR	R0,[R0]
	MOVS	R1,R1
	MOVEQ	R0,#0
	MOVS	R0,R0
	BLNE	SIZPOKV		;prikazi vse lokalno oddane okvirje?
	BL	SPROSTI		;sprosti oddani blok
	LDR	LR,[SP],#4
	MOV	R6,#0		;postavi stanje=0 cakanje
	MOV	PC,LR
SLIPTX3	MOV	R0,#SLFESC	;oddaj FESC v U1THR
	STRB	R0,[R12,#UTHR]
	MOV	R6,#2		;postavi stanje=2 oddaja TFESC
	B	SLIPTX1
SLIPTX4	MOV	R0,#SLFESC	;oddaj FESC v U1THR
	STRB	R0,[R12,#UTHR]
	MOV	R6,#3		;postavi stanje=3 oddaja TFEND
	B	SLIPTX1
SLIPTX5	CMP	R6,#2		;stanje=2 - oddaja TFESC?
	BNE	SLIPTX6
	MOV	R0,#SLTFESC	;oddaj TFESC v U1THR
	STRB	R0,[R12,#UTHR]
	MOV	R6,#1		;postavi stanje=1 oddaja vsebine
	B	SLIPTX1
SLIPTX6	CMP	R6,#3		;stanje=3 - oddaja TFEND?
	BNE	SLIPTX7
	MOV	R0,#SLTFEND	;oddaj TFEND v U1THR
	STRB	R0,[R12,#UTHR]
	MOV	R6,#1		;postavi stanje=1 oddaja vsebine
	B	SLIPTX1
SLIPTX7	STR	LR,[SP,#-4]!	;nov blok - stanje=0 - cakanje?
	LDR	R0,=STXVMES
	BL	IZVMES
	LDR	LR,[SP],#4
	MOVS	R8,R8		;bloka ni?
	MOVEQ	PC,LR
	MOV	R7,#0		;kazalec na 0
	MOV	R0,#SLFEND	;oddaj FEND na zacetku v U1THR
	STRB	R0,[R12,#UTHR]
	MOV	R6,#1		;postavi stanje=1 oddaja vsebine
	B	SLIPTX1
	LTORG

;*** Inicializiraj UART1 *** (R0-1,R12)
UDLL	EQU	0x00
UTHR	EQU	0x00
URBR	EQU	0x00
UDLM	EQU	0x04
UIER	EQU	0x04
UFCR	EQU	0x08
ULCR	EQU	0x0C
ULSR	EQU	0x14
UART1	LDR	R12,=0xE0010000 ;osnovni naslov UART1 v R12
	MOV	R0,#0x83	;U1LCR=0x83 DLAB=1,no-parity,1stop,8bit
	STRB	R0,[R12,#ULCR]
	LDR	R0,=BAUDMOD	;baudrate modulo (16bit)
	LDRH	R0,[R0]
	STRB	R0,[R12,#UDLL]	;U1DLL
	MOV	R0,R0,LSR#8	;U1DLM
	STRB	R0,[R12,#UDLM]
	MOV	R0,#0x81	;U1FCR=0x81 RXtrig=8bit,FIFOenable
	STRB	R0,[R12,#UFCR]
	MOV	R0,#0x03	;U1LCR=0x03 DLAB=0,no-parity,1stop,8bit
	STRB	R0,[R12,#ULCR]
	MOV	R0,#0x01	;U1IER=0x01 ReceiveDataAvailable
	STRB	R0,[R12,#UIER]
	LDR	R0,=SLTXSTA	;zacetno stanje SLIP TX
	MOV	R1,#0
	STRB	R1,[R0]
	MOV	PC,LR
	LTORG

;*** Nastavi prekinitve v VIC v LPC23xx *** (R0-1)
VIC	LDR	R0,=0xFFFFF00C	;VICIntSelect
	LDR	R1,=0x00000080	;UART1=FIQ
	STR	R1,[R0]
	LDR	R0,=0xFFFFF010	;VICIntEnable
	LDR	R1,=0x00000080	;vklop UART1
	STR	R1,[R0]
	MOV	PC,LR
	LTORG

;*** Vkljuci prekinitve FIQ v ARM *** (R0-1)
IRQON	MRS	R0,CPSR		;preklopi v nacin FIQ
	BIC	R0,R0,#0x0000001F
	ORR	R0,R0,#0x00000011
	MSR	CPSR_c,R0	;popravi CPSR
	LDR	SP,=FSKLAD	;inicializiraj FIQ SP
	LDR	R12,=0xE0010000 ;osnovni naslov UART1 v R12
	MOV	R8,#0		;neobstojec zacetni blok za SLIPRX
	MRS	R0,CPSR		;nacin supervisor in vklop FIQ
	BIC	R0,R0,#0x0000005F
	ORR	R0,R0,#0x00000093
	MSR	CPSR_c,R0	;popravi CPSR
	MOV	PC,LR
	LTORG

;*** Preklop sprejem/oddaja v SATNC *** (R0-3,R8)
	ALIGN	16
PREKLOP	STR	LR,[SP,#-4]!
	LDR	R0,=0x3FFFC036	;FIO1PIN2 P1.16 sprejem/oddaja?
	LDRB	R0,[R0]
	TST	R0,#0x01
	BEQ	PREKLO1
	LDR	R0,=0xE0004008	;T0TC - Timer0
	LDR	R2,[R0]
	LDR	R0,=ZPONOVI	;cas ponavljanja se je iztekel?
	LDR	R1,[R0]
	SUB	R1,R2,R1
	LDR	R3,=TPONOVI
	LDR	R3,[R3]
	CMP	R1,R3
	LDRCC	PC,[SP],#4
	LDR	R0,=TPREKLP	;cas slota se je iztekel?
	LDR	R1,[R0]
	SUB	R1,R2,R1
	LDR	R3,=TSLOT	
	LDR	R3,[R3]
	CMP	R1,R3
	LDRCC	PC,[SP],#4
	STR	R2,[R0]		;postavi novo casovno mejo
	LDR	R0,=HTPVMES	;cakajoce stevilke potrditev na oddajo?
	LDRH	R1,[R0]
	LDRH	R2,[R0,#2]
	CMP	R1,R2
	BNE	VKLOP
	LDR	R0,=HTXVMES	;cakajoci podatkovni okvirji na oddajo?
	LDRH	R1,[R0]
	LDRH	R2,[R0,#2]
	CMP	R1,R2
	BNE	VKLOP
	LDR	PC,[SP],#4
	LTORG
	ALIGN	16
PREKLO1	LDR	R0,=SPREKLP	;stanje oddaje v R0
	LDRB	R0,[R0]
	CMP	R0,#1		;stanje 1 - oddaja sinhronizacijske glave?
	BEQ	GLAVA
	CMP	R0,#2		;stanje 2 - poslji okvir v oddajo?
	BEQ	OKVIR
	CMP	R0,#3		;stanje 3 - cakanje konca okvirja?
	BEQ	KONEC
	B	REP		;stanje 4 (ali drugo) - oddaja repa na koncu
	LTORG

;*** Vklop oddajnika v SATNC *** (R0-2,SP+4)
	ALIGN	16
VKLOP	BL	PSEVNAK		;psevdonakljucno(16bit)
	MOV	R0,R0,LSR#16
	LDR	R1,=TECEN2	;pn(16bit)<>TECEN2 brez DCD?
	LDRH	R1,[R1]
	CMP	R0,R1
	BCC	VKLOP1
	LDR	R1,=0x3FFFC036	;FIO1PIN2 P1.17=/DCD?
	LDRB	R1,[R1]
	TST	R1,#0x02
	LDREQ	PC,[SP],#4
	LDR	R1,=TECEN1	;pn(16bit)<>TECEN1 z DCD?
	LDRH	R1,[R1]
	CMP	R0,R1
	LDRCS	PC,[SP],#4
VKLOP1	LDR	R0,=MPONOVI	;minimum ponavljanja?
	LDRB	R1,[R0]
	SUBS	R1,R1,#1
	STRB	R1,[R0]
	BMI	VKLOP2
	LDR	R0,=DPONOVI	;povecaj neizvedena ponavljanja
	LDRB	R1,[R0]
	ADD	R1,R1,#1
	STRB	R1,[R0]
	LDR	R0,=0xE0004008	;T0TC - Timer0
	LDR	R1,[R0]
	LDR	R0,=ZPONOVI	;zadnje ponavljanje
	STR	R1,[R0]
	LDR	PC,[SP],#4
VKLOP2	LDR	R0,=ZNACIN	;vklop/izklop TX?
	LDRB	R1,[R0]
	TST	R1,#128
	BEQ	VKLOP5
	LDR	R0,=HTPVMES	;brisi stevilke potrditve!
	MOV	R1,#0
	STR	R1,[R0]
	BL	IZVMES
VKLOP3	LDR	R0,=HTXVMES	;sprosti cakajoce podatkovne okvirje?
	BL	IZVMES
	MOVS	R8,R8
	LDREQ	PC,[SP],#4
	LDR	R0,=MNASLOV	;KISS brez naslova?
	LDR	R0,[R0]
	MOVS	R0,R0
	BEQ	VKLOP4
	LDR	R0,=LNASLOV	;lokalni naslovi vkljuceni?
	LDR	R0,[R0]
	MOVS	R0,R0
	BEQ	VKLOP4
	LDRH	R0,[R8,#2]	;okvir VSEM na SLIP TX?
	ADD	R0,R0,#4
	LDR	R1,[R8,R0]
	MVNS	R0,R1
	BNE	VKLOP4
	LDR	R0,=STXVMES	;okvir v SLIP oddajni vmesnik
	BL	VVMES
	B	VKLOP3
VKLOP4	BL	SPROSTI		;KISS, izkljuceni lokalni, ni VSEM
	B	VKLOP3
VKLOP5	LDR	R0,=0x3FFFC036	;FIO1PIN2 vklopi oddajnik
	MOV	R1,#0x00
	STRB	R1,[R0]
	LDR	R0,=SPREKLP	;postavi stanje 1
	MOV	R1,#1
	STRB	R1,[R0]
	LDR	PC,[SP],#4
	LTORG

;*** Oddaja sinhronizacijske glave (stanje 1) *** (R0-2,R8,SP+4)+RMIITX,TIZPOKV
	ALIGN	16
GLAVA	LDR	R0,=0xE0004008	;T0TC - Timer0
	LDR	R2,[R0]
	LDR	R0,=TPREKLP	;cas glave se je iztekel?
	LDR	R1,[R0]
	SUB	R1,R2,R1
	LDR	R2,=TGLAVA
	LDR	R2,[R2]
	CMP	R1,R2
	LDRCC	PC,[SP],#4
	LDR	R8,=POTBLOK	;sestavi slepi okvir za reset sprejemnika
	MOV	R0,#4		;dolzina 4
	STRH	R0,[R8]
	MOV	R0,#8		;zacetek 8
	STRH	R0,[R8,#2]
	MOV	R0,#-1		;slepa vsebina
	STR	R0,[R8,#8]
	BL	RMIITX		;oddaj okvir (R8) na RMII
	LDR	R0,=SPREKLP	;postavi stanje 3
	MOV	R1,#3
	STRB	R1,[R0]
	LDR	PC,[SP],#4
	LTORG

;*** Poslji okvir v oddajo (stanje 2) *** (R0-4,R8,SP+4)+RMIITX,TIZPOKV
	ALIGN	16
OKVIR	LDR	R0,=HTPVMES	;stevilka potrditve iz vmesnika?
	BL	IZVMES
	MOVS	R8,R8
	BEQ	OKVIR1
	LDR	R3,=POTBLOK	;sestavi potrdilni okvir
	MOV	R0,#8		;dolzina 8
	STRH	R0,[R3]
	MOV	R0,#8		;zacetek 8
	STRH	R0,[R3,#2]
	MOV	R0,#0		;znacka nicla
	STR	R0,[R3,#8]
	STR	R8,[R3,#12]	;stevilka prve potrditve
	MOV	R4,#1
OKVIR6	LDR	R0,=HTPVMES	;dodatna potrditev?	
	BL	IZVMES
	MOVS	R8,R8
	BEQ	OKVIR7
	LDRH	R0,[R3]		;dodatna stevilka v okvir
	ADD	R0,R0,#8
	STR	R8,[R3,R0]
	SUB	R0,R0,#4
	STRH	R0,[R3]
	ADD	R4,R4,#1	;MAX stevilk v okvirju?
	CMP	R4,#POTMAX
	BCC	OKVIR6
OKVIR7	MOV	R8,R3
	BL	RMIITX		;oddaj okvir (R8) na RMII
	BL	TIZPOKV		;prikazi vse oddane okvirje?
	LDR	R0,=SPREKLP	;postavi stanje 3
	MOV	R1,#3
	STRB	R1,[R0]
	LDR	PC,[SP],#4
OKVIR1	LDR	R0,=HTXVMES	;podatkovni okvir iz vmesnika?
	BL	IZVMES
	MOVS	R8,R8
	BEQ	OKVIR5
	LDRB	R0,[R8,#5]	;izvajanje ponavljanja?
	LDR	R1,=DPONOVI
	LDRB	R1,[R1]
	SUB	R0,R0,R1	;odstej neizvedena ponavljanja
	SUBS	R0,R0,#1
	STRB	R0,[R8,#5]
	BMI	OKVIR2
	LDR	R0,=HTZVMES	;shrani v zacasni pomnilnik
	BL	VVMES
	LDR	PC,[SP],#4
OKVIR2	BL	RMIITX		;oddaj okvir (R8) na RMII
	BL	TIZPOKV		;prikazi vse oddane okvirje?
	LDR	R0,=SPREKLP	;postavi stanje 3
	MOV	R1,#3
	STRB	R1,[R0]
	LDRB	R3,[R8,#4]	;odstej ponavljanja
	SUBS	R3,R3,#1
	STRB	R3,[R8,#4]
	BMI	OKVIR3		;sprosti/SLIP, ko se stevec iztece?
	BL	PSEVNAK		;psevdonakljucno(20bit)
	MOV	R0,R0,LSR#12
	LDR	R1,=NPONOVI	;izracunaj zakasnitev ponavljanja
	LDRB	R1,[R1]
	SUBS	R1,R1,R3
	MOVMI	R1,#0
	MUL	R2,R1,R0
	MOV	R0,R2,LSR#19
	STRB	R0,[R8,#5]
	LDR	R0,=HTZVMES	;shrani v zacasni pomnilnik
	BL	VVMES
	LDR	PC,[SP],#4
OKVIR3	LDR	R0,=MNASLOV	;KISS brez naslova?
	LDR	R0,[R0]
	MOVS	R0,R0
	BEQ	OKVIR4
	LDR	R0,=LNASLOV	;lokalni naslovi vkljuceni?
	LDR	R0,[R0]
	MOVS	R0,R0
	BEQ	OKVIR4
	LDRH	R0,[R8,#2]	;okvir VSEM na SLIP TX?
	ADD	R0,R0,#4
	LDR	R1,[R8,R0]
	MVNS	R0,R1
	BNE	OKVIR4
	LDR	R0,=STXVMES	;okvir v SLIP oddajni vmesnik
	BL	VVMES
	LDR	PC,[SP],#4
OKVIR4	BL	SPROSTI		;KISS, izkljuceni lokalni, ni VSEM
	LDR	PC,[SP],#4
OKVIR5	LDR	R0,=SPREKLP	;postavi stanje 4
	MOV	R1,#4
	STRB	R1,[R0]
	LDR	PC,[SP],#4
	LTORG

;*** Cakanje konca okvirja (stanje 3) *** (R0-3,R8,R11,SP+4)
	ALIGN	16
KONEC	LDR	R1,[R11,#0x128]	;TxProduceIndex v R1
	LDR	R0,[R11,#0x12C]	;TxConsumeIndex v R0
	SUBS	R1,R1,R0	;preskok stevcev na koncu?
	BPL	KONEC1
	ADD	R1,R1,#TXDESCN
	ADD	R1,R1,#1
KONEC1	CMP	R1,#TXBLOKN	;RMII oddajnik poln?
	LDREQ	PC,[SP],#4
	LDR	R0,=SPREKLP	;postavi stanje 2
	MOV	R1,#2
	STRB	R1,[R0]
	LDR	PC,[SP],#4
	LTORG

;*** Oddaja repa na koncu (stanje 4) *** (R0-2,R11,SP+4)
	ALIGN	16
REP	LDR	R1,[R11,#0x128]	;TxProduceIndex v R1
	LDR	R0,[R11,#0x12C]	;TxConsumeIndex v R0
	CMP	R1,R0		;oddajnik prazen?
	BEQ	REP1
	LDR	R0,=0xE0004008	;T0TC - Timer0
	LDR	R1,[R0]
	LDR	R0,=TPREKLP	;casovna meja za oddajo repa
	STR	R1,[R0]
	LDR	PC,[SP],#4
REP1	LDR	R0,=0xE0004008	;T0TC - Timer0
	LDR	R2,[R0]
	LDR	R0,=TPREKLP	;cas repa se je iztekel?
	LDR	R1,[R0]
	SUB	R1,R2,R1
	LDR	R2,=TREP
	LDR	R2,[R2]
	CMP	R1,R2
	LDRCC	PC,[SP],#4
	B	IZKLOP
	LTORG

;*** Izklop oddajnika v SATNC *** (R0-3,R8,SP+4)
	ALIGN	16
IZKLOP	BL	PROSTO		;izracunaj primanjkljaj v R3
	LDR	R1,=MINBLOK
	LDRB	R3,[R1]
	SUB	R3,R0,R3
	SUB	R3,R3,#1
IZKLOP1	LDR	R0,=HTZVMES	;iz zacasnega vmesnika
	BL	IZVMES
	MOVS	R8,R8		;izklop, ko ni vec bloka
	BEQ	IZKLOP3
	ADDS	R3,R3,#1	;primanjkljaj se obstaja?
	BMI	IZKLOP2
	LDRB	R0,[R8,#5]	;poisci minimum ponavljanja
	LDR	R1,=MPONOVI
	LDRB	R2,[R1]
	CMP	R2,R0
	STRBCS	R0,[R1]
	LDR	R0,=HTXVMES	;v oddajni vmesnik
	BL	VVMES
	B	IZKLOP1
IZKLOP2	BL	SPROSTI		;sprosti odvecni blok
	B	IZKLOP1
IZKLOP3	LDR	R0,=0x3FFFC036	;FIO1PIN2 izklopi oddajnik
	MOV	R1,#0xFF
	STRB	R1,[R0]
	LDR	R0,=DPONOVI	;resetiraj neizvedena ponavljanja
	MOV	R1,#0
	STRB	R1,[R0]
	LDR	R0,=0xE0004008	;T0TC - Timer0
	LDR	R1,[R0]
	LDR	R0,=ZPONOVI	;zadnje ponavljanje
	STR	R1,[R0]
	LDR	PC,[SP],#4
	LTORG

;*** Oddaj okvir (R8) na RMII *** (R0-8,R11)
NBPMIN	EQU	8		;min dolzina NBP vsebine
	ALIGN	16
	SPACE	4
RMIITX	LDR	R1,[R11,#0x128]	;TxProduceIndex v R1
	LDR	R0,=TXDESC	;TxDescriptor
	ADD	R0,R0,R1,LSL#3
	LDR	R1,[R0],#4	;oddajni blok (R1), control word (R0)
	LDRH	R4,[R8]		;preverjena dolzina v R4
	CMP	R4,#RMIIMAX
	MOVCS	R4,#RMIIMAX
	ADD	R3,R4,#13	;dodaj glavo in Int,Last,CRC,Pad,Override
	ORR	R3,R3,#0xF4000000
	STR	R3,[R0]
	CMP	R4,#NBPMIN	;veljaven NBP okvir?
	BCC	RMIITX1
	MOV	R0,#0x58	;veljaven prejemnikov MAC 58-86-FB-E8-9B-1C
	STRB	R0,[R1],#1
	MOV	R0,#0x86
	STRB	R0,[R1],#1
	MOV	R0,#0xFB
	STRB	R0,[R1],#1
	MOV	R0,#0xE8
	STRB	R0,[R1],#1
	MOV	R0,#0x9B
	STRB	R0,[R1],#1
	MOV	R0,#0x1C
	STRB	R0,[R1],#1
	B	RMIITX2
RMIITX1	MOV	R0,#0		;neveljaven prejemnikov MAC 00-00-00-00-00-00
	STRH	R0,[R1],#2
	STRH	R0,[R1],#2
	STRH	R0,[R1],#2
RMIITX2 MOV	R0,#0		;posiljateljev MAC 00-00-00-00-00-00
	STRH	R0,[R1],#2
	STRH	R0,[R1],#2
	STRH	R0,[R1],#2
	MOV	R0,R4,LSR#8	;dolzina okvirja
	STRB	R0,[R1],#1
	STRB	R4,[R1],#1
	SUB	R7,R4,#1	;dolzina-1
	LDRH	R0,[R8,#2]	;zacetni naslov
	ADD	R0,R0,R8
RMIITX3	LDMIA	R0!,{R2-R6}	;prepisi vsebino max 81*20byte=1620byte
	STMIA	R1!,{R2-R6}
	SUBS	R7,R7,#20
	BPL	RMIITX3
	LDR	R1,[R11,#0x128]	;TxProduceIndex povecaj za 1
	ADD	R1,R1,#1
	RSBS	R2,R1,#TXDESCN	;preveri gornjo mejo
	MOVCC	R1,#0
	STR	R1,[R11,#0x128]
	MOV	PC,LR
	LTORG

;*** Sprejem RMII okvirjev *** (R0-7,R11)
RMIIZAC	EQU	8		;zacetek podatkov RX
RMIIMIN	EQU	63		;min dolzina (s CRC) 64-1
RMIIMAX	EQU	1616		;max dolzina (s CRC) 1616
	ALIGN	16
RMIIRX	LDR	R0,[R11,#0xFE0]	;IntStatus RxDoneInt=?
	TST	R0,#0x0008
	MOVEQ	PC,LR
	MOV	R1,#0x0008	;IntClear RxDoneInt
	STR	R1,[R11,#0xFE8]
RMIIRX1	LDR	R1,[R11,#0x114]	;RxProduceIndex v R1 primerjaj z
	LDR	R2,[R11,#0x118]	;RxConsumeIndex v R2
	CMP	R2,R1		;nov RMII okvir?
	MOVEQ	PC,LR
	STR	LR,[SP,#-4]!	;resi link
	BL	RMIIRX2		;obdelaj okvir
	LDR	R1,[R11,#0x118]	;RxConsumeIndex povecaj za 1
	ADD	R1,R1,#1
	RSBS	R2,R1,#RXDESCN	;preveri gornjo mejo
	MOVCC	R1,#0
	STR	R1,[R11,#0x118]
	LDR	LR,[SP],#4
	B	RMIIRX1		;takoj naslednji okvir?
	LTORG
	ALIGN	16
RMIIRX2	STR	LR,[SP,#-4]!	;resi link
	LDR	R0,=RXSTAT	;RX status v R6
	LDR	R6,[R0,R2,LSL#3]
	MOV	R0,#0x10800000	;Overrun,CRC
	TST	R6,R0
	LDRNE	PC,[SP],#4
	MOV	R6,R6,LSL#21	;ociscena dolzina-1 v R6
	MOV	R6,R6,LSR#21
	CMP	R6,#RMIIMAX	;preveri max dolzino
	LDRCS	PC,[SP],#4
	CMP	R6,#RMIIMIN	;preveri min dolzino
	LDRCC	PC,[SP],#4
	LDR	R0,=RXDESC	;zacetek sprejetega bloka v R5
	LDR	R5,[R0,R2,LSL#3]
	LDRB	R1,[R5,#12]	;navedena NBP dolzina v R7
	LDRB	R7,[R5,#13]
	ORR	R7,R7,R1,LSL#8
	CMP	R7,#NBPMIN	;preveri NBP MIN dolzino
	LDRCC	PC,[SP],#4
	SUB	R1,R6,#17	;sprejeta dolzina v R1 brez MAC,navedene,CRC
	CMP	R1,R7
	LDRCC	PC,[SP],#4	;navedena>sprejeta?
	BEQ	RMIIRX3		;navedena=sprejeta?
	CMP	R6,#RMIIMIN
	LDRNE	PC,[SP],#4	;ni kratek dopolnjen okvir?
RMIIRX3	BL	VAD012		;povprecje AD0.0,1,2 v R3
	LDR	R0,=SMETER	;S-METER sprejetega okvirja
	STR	R3,[R0]
	BL	DODELI		;sprejemni blok obstaja?
	MOVS	R8,R8
	LDREQ	PC,[SP],#4
	STRH	R7,[R8]		;NBP dolzina v sprejemni blok
	MOV	R0,#RMIIZAC	;zacetek ponora v blok in R1
	STRH	R0,[R8,#2]
	ADD	R1,R8,R0
	ADD	R0,R5,#14	;odrezi glavo
	SUB	R7,R6,#18	;max vsebina 1620byte=81*20byte
RMIIRX4	LDMIA	R0!,{R2-R6}
	STMIA	R1!,{R2-R6}
	SUBS	R7,R7,#20
	BPL	RMIIRX4
	B	HOBDELA		;obdelava okvirja (R8) iz RMII sprejemnika
	LTORG

;*** Inicializiraj RMII v LPC23xx *** (R0-3,R11)+RMIIRAM
INIRMII	STR	LR,[SP,#-4]!	;resi link
	LDR	R11,=0xFFE00000	;osnovni naslov RMII registrov v R11
;Remove the soft reset condition from the MAC
	MOV	R1,#0		;MAC1=0 remove soft reset
	STR	R1,[R11]
	MOV	R0,#0x10000	;64k cakalnih zank za REFCLK!
INIRMI1	SUBS	R0,R0,#1
	BPL	INIRMI1
;Select RMII mode
	LDR	R1,=0x0200	;Command=RMII
	STR	R1,[R11,#0x100]
;Configure the transmit and receive DMA engines, including the descriptor arrays
	BL	RMIIRAM		;razrezi RMII RAM
	LDR	R1,=RXDESC	;RxDescriptor
	STR	R1,[R11,#0x108]
	LDR	R1,=RXSTAT	;RxStatus
	STR	R1,[R11,#0x10C]
	MOV	R1,#RXDESCN	;RxDescriptorNumer
	STR	R1,[R11,#0x110]
	LDR	R1,=TXDESC	;TxDescriptor
	STR	R1,[R11,#0x11C]
	LDR	R1,=TXSTAT	;TxStatus
	STR	R1,[R11,#0x120]
	MOV	R1,#TXDESCN	;TxDescriptorNumber
	STR	R1,[R11,#0x124]
;Configure the host registers (MAC1,MAC2 etc.) in the MAC, MAC=58-86-FB-E8-9B-1C
	MOV	R2,#0x58	;Station Address 2 Register
	MOV	R3,#0x86
	ORR	R2,R2,R3,LSL#8
	STR	R2,[R11,#0x048]
	MOV	R2,#0xFB	;Station Address 1 Register
	MOV	R3,#0xE8
	ORR	R2,R2,R3,LSL#8
	STR	R2,[R11,#0x044]
	MOV	R2,#0x9B	;Station Address 0 Register
	MOV	R3,#0x1C
	ORR	R2,R2,R3,LSL#8
	STR	R2,[R11,#0x040]
	MOV	R1,#0x20	;RxFilterCtrl=AcceptPerfect
	STR	R1,[R11,#0x200]
	MOV	R1,#0x31	;MAC2=Pad,CRCEnable,FullDuplex
	STR	R1,[R11,#0x004]
	MOV	R1,#0x15	;IPGT=0x15
	STR	R1,[R11,#0x008]
	LDR	R1,=0x0C12	;IPGR=0x0C12
	STR	R1,[R11,#0x00C]
	MOV	R1,#0x6A0	;MAXF=0x6A0
	STR	R1,[R11,#0x014]
	MOV	R1,#0x100	;SUPP=0x100
	STR	R1,[R11,#0x018]
;Enable the receive and transmit datapaths
	LDR	R1,=0x0633	;Command=DUP,RMII,RxReset,TxReset,TxEnable,RxEnable
	STR	R1,[R11,#0x100]
	MOV	R1,#0x0001	;MAC1=RxEnable
	STR	R1,[R11]
	LDR	PC,[SP],#4
	LTORG

;*** Razrezi RMII RAM *** (R0-4)
RMIIRAM	LDR	R0,=ZACERX	;razrezi sprejemne bloke, naslov+RMIIBLK
	LDR	R1,=RXDESC
	LDR	R2,=RXCONT
	MOV	R3,#RXDESCN
RMIIRA1	STR	R0,[R1],#4	;Packet
	ADD	R0,R0,#RMIIBLK
	STR	R2,[R1],#4	;Control
	SUBS	R3,R3,#1	;zanka
	BPL	RMIIRA1
	LDR	R1,=TXDESC	;razrezi oddajne bloke, naslov+RMIIBLK, ponavljaj po TXBLOKN
	MOV	R3,#TXDESCN
	MOV	R4,#0
	LDR	R0,=ZACETX
RMIIRA2	STR	R0,[R1],#4	;Packet
	ADD	R0,R0,#RMIIBLK
	MOV	R2,#0		;Control=0
	STR	R2,[R1],#4
	ADD	R4,R4,#1	;ponavljanje po TXBLOKN?
	CMP	R4,#TXBLOKN
	BNE	RMIIRA3
	MOV	R4,#0
	LDR	R0,=ZACETX
RMIIRA3	SUBS	R3,R3,#1	;zanka
	BPL	RMIIRA2
	MOV	PC,LR
	LTORG

;*** Inicializiraj SSP1 v LPC23xx *** (R0,R10)
S8BIT	EQU	0x0007		;SSP1CR0: SCR=0,CPHA=0,CPOL=0,SPI,8bit
S16BIT	EQU	0x000F		;SSP1CR0: SCR=0,CPHA=0,CPOL=0,SPI,16bit
INISSP1	LDR	R10,=0xE0030000	;osnovni naslov SSP1 v R10
	LDR	R0,=S16BIT	;SSP1CR0: 16bit
	STRH	R0,[R10]
	MOV	R0,#4		;SSP1CPSR: prescaler=4
	STRB	R0,[R10,#0x10]
	MOV	R0,#2		;SSP1CR1: master, SSE=1
	STRB	R0,[R10,#0x04]
	MOV	PC,LR
	LTORG

;*** Preberi 16bitov (R0) iz KSZ8851SNL v R3 *** (R0-3,R10)
	ALIGN	16
KSZ16R	LDR	R2,=0x3FFFC014	;FIO0PIN0 v R2
	MOV	R1,#0x00	;vklopi CSN
	STRB	R1,[R2]
	TST	R0,#0x02	;SSP1DR: ukaz in naslov
	MOVEQ	R1,#0x0C00
	MOVNE	R1,#0x3000
	ORR	R1,R1,R0,LSL#2
	BIC	R1,R1,#0x0F
	STRH	R1,[R10,#0x08]
	MOV	R1,#-1		;SSP1DR: oddaj dva prazna bajta
	STRH	R1,[R10,#0x08]
KSZ16R1	LDRB	R1,[R10,#0x0C]	;SSP1SR: pocakaj SPI prenos?
	TST	R1,#0x10	;BSY=?
	BNE	KSZ16R1
	MOV	R1,#0xFF	;izklopi CSN
	STRB	R1,[R2]
KSZ16R2	LDRH	R3,[R10,#0x08]	;SSP1DR: precitaj dva bajta
	LDRB	R1,[R10,#0x0C]	;SSP1SR: prazen sprejemnik?
	TST	R1,#0x04	;RNE=?
	BNE	KSZ16R2
	MOV	R1,R3,LSR#8	;zamenjaj nizji/visji bajt
	AND	R3,R3,#0x00FF
	ORR	R3,R1,R3,LSL#8
	MOV	PC,LR
	LTORG

;*** Vpisi 16bitov R3 v KSZ8851SNL (R0) *** (R0-3,R10)
	ALIGN	16
KSZ16W	LDR	R2,=0x3FFFC014	;FIO0PIN0 v R2
	MOV	R1,#0x00	;vklopi CSN
	STRB	R1,[R2]
	TST	R0,#0x02	;SSP1DR: ukaz in naslov
	MOVEQ	R1,#0x4C00
	MOVNE	R1,#0x7000
	ORR	R1,R1,R0,LSL#2
	BIC	R1,R1,#0x0F
	STRH	R1,[R10,#0x08]
	MOV	R1,R3,LSL#8	;SSP1DR: zamenjaj nizji/visji bajt
	ORR	R1,R1,R3,LSR#8
	STRH	R1,[R10,#0x08]
KSZ16W1	LDRB	R1,[R10,#0x0C]	;SSP1SR: pocakaj SPI prenos?
	TST	R1,#0x10	;BSY=?
	BNE	KSZ16W1
	MOV	R1,#0xFF	;izklopi CSN
	STRB	R1,[R2]
KSZ16W2	LDRH	R1,[R10,#0x08]	;SSP1DR: precitaj SPI v prazno
	LDRB	R1,[R10,#0x0C]	;SSP1SR: prazen sprejemnik?
	TST	R1,#0x04	;RNE=?
	BNE	KSZ16W2
	MOV	PC,LR
	LTORG

;*** Inicializiraj KSZ8851SNL *** (R0-4,R10)
	ALIGN	16
INIKSZ	STR	LR,[SP,#-4]!	;resi link
	LDR	R0,=0x3FFFC097	;FIO4PIN3 postavi /RESET
	MOV	R1,#0x00
	STRB	R1,[R0]
	MOV	R0,#0x80000	;zakasnitev 512k zank
INIKSZ1	SUBS	R0,R0,#1
	BPL	INIKSZ1
	LDR	R0,=0x3FFFC097	;FIO4PIN3 odpusti /RESET
	MOV	R1,#0xFF
	STRB	R1,[R0]
	MOV	R0,#0x80000	;zakasnitev 512k zank
INIKSZ2	SUBS	R0,R0,#1
	BPL	INIKSZ2
	LDR	R0,=ESPEED	;nastavi ETH AN oziroma hitrost in MDIX
	LDRB	R4,[R0]
	MOV	R0,#0xF6	;P1CR preberi privzeto
	BL	KSZ16R
	CMP	R4,#10		;10Mbps MDI?
	LDREQ	R3,=0x0632
	CMP	R4,#11		;10Mbps MDIX?
	LDREQ	R3,=0x0432
	CMP	R4,#100		;100Mbps MDI?
	LDREQ	R3,=0x0678
	CMP	R4,#101		;100Mbps MDIX?
	LDREQ	R3,=0x0478
	MOV	R0,#0xF6	;P1CR vpisi popravljeno
	BL	KSZ16W
	MOV	R0,#0x84	;TXFDPR
	MOV	R3,#0x4000	;Enable QMU Transmit Frame Data Pointer Auto Increment
	BL	KSZ16W
	MOV	R0,#0x70	;TXCR
	MOV	R3,#0x000E	;Transmit Flow Control Enable / Enable Transmit padding / Transmit CRC
	BL	KSZ16W
	MOV	R0,#0x86	;RXFDPR
	MOV	R3,#0x4000	;Enable QMU Receive Frame Data Pointer Auto Increment
	BL	KSZ16W
	MOV	R0,#0x9C	;RXFCTR
	MOV	R3,#0x0001	;Configure Receive Frame Threshold for one frame
	BL	KSZ16W
	MOV	R0,#0x74	;RXCR1
	MOV	R3,#0x0400	;Receive Flow Control Enable
	ORR	R3,R3,#0x0012	;RX all/RXINVF
	BL	KSZ16W
	MOV	R0,#0x76	;RXCR2
	MOV	R3,#0x0080	;Enable SPI single frame data burst 
	BL	KSZ16W
	MOV	R0,#0x82	;RXQCR
	MOV	R3,#0x0230	;Enable QMU Receive IP Header Two-Byte Offset / RXFCTE / ADRFE
	BL	KSZ16W
	MOV	R0,#0x92	;ISR
	MOV	R3,#-1		;Clear the interrupts status
	BL	KSZ16W
	MOV	R0,#0x70	;TXCR
	BL	KSZ16R
	ORR	R3,R3,#0x0001	;Enable QMU Transmit
	BL	KSZ16W
	MOV	R0,#0x74	;RXCR1
	BL	KSZ16R
	ORR	R3,R3,#0x0001	;Enable QMU Receive
	BL	KSZ16W
	LDR	R0,=KRXBLOK	;zacetni blok sprejemnika KSZ8851SNL
	MOV	R1,#0
	STR	R1,[R0]
	LDR	R0,=KTXBLOK	;zacetni blok oddajnika KSZ8851SNL
	MOV	R1,#0
	STR	R1,[R0]
	LDR	R0,=MACMAC	;zacetni MAC sogovornika
	MOV	R1,#-1
	STRH	R1,[R0],#2
	STRH	R1,[R0],#2
	STRH	R1,[R0],#2
	LDR	R0,=ETXBLOK	;zacetni ETXBLOK
	MOV	R1,#0
	STR	R1,[R0]
	LDR	R0,=ECACHE	;brisi celoten ECACHE
	MOV	R1,#MCACHE
	MOV	R2,#0
INIKSZ3	SUBS	R1,R1,#1	;konec seznama?
	LDRMI	PC,[SP],#4
	STR	R2,[R0],#4	;brisi 16byte
	STR	R2,[R0],#4
	STR	R2,[R0],#4
	STR	R2,[R0],#4
	B	INIKSZ3
	LTORG

;*** Oddaja ETH okvirja iz KTXVMES na KSZ8851SNL *** (R0-8,R10)
KTXMIN	EQU	1600		;MIN pomnilnik za oddajo
	ALIGN	16
KETHTX	LDR	R0,=KRXBLOK	;obstaja KRXBLOK?
	LDR	R8,[R0]
	MOVS	R8,R8
	BNE	KETHRX6		;zanka prepisovanja sprejemnika
	LDR	R0,=KTXBLOK	;obstaja KTXBLOK?
	LDR	R8,[R0]
	MOVS	R8,R8
	BNE	KETHTX2		;zanka prepisovanja oddajnika
	LDR	R0,=KTXVMES	;cakajoci okvirji na oddajo?
	LDRH	R1,[R0]
	LDRH	R2,[R0,#2]
	CMP	R1,R2
	MOVEQ	PC,LR
	STR	LR,[SP,#-4]!	;resi link
	LDR	R0,=KSZID	;veljaven KSZ8851SNL?
	LDR	R1,[R0]
	MOVS	R1,R1
	BMI	KETHTX7		;sprosti okvir, ko ni KSZ8851SNL
	MOV	R0,#0x78	;TXMIR
	BL	KSZ16R
	BIC	R3,R3,#0xE000
	CMP	R3,#KTXMIN	;prostor v pomnilniku KSZ8851SNL?
	LDRCC	PC,[SP],#4
	MOV	R0,#0x80	;TXQCR
	BL	KSZ16R
	TST	R3,#0x001	;presnji okvir zakljucen?
	LDRNE	PC,[SP],#4
	LDR	R0,=KTXVMES	;nov okvir v oddajo?
	BL	IZVMES
	MOVS	R8,R8
	LDREQ	PC,[SP],#4
	MOV	R0,#0x82	;RXQCR
	MOV	R3,#0x0238	;Start QMU DMA transfer operation
	BL	KSZ16W
	LDR	R0,=S8BIT	;SSP1CR0: 8bit
	STRH	R0,[R10]
	LDR	R2,=0x3FFFC014	;FIO0PIN0 vklopi CSN
	MOV	R1,#0x00
	STRB	R1,[R2]
	MOV	R1,#0xC0	;SSP1DR: Write TXQ command phase
	STRB	R1,[R10,#0x08]
	MOV	R1,#0x00	;SSP1DR: 0x8000 control word (2byte)
	STRB	R1,[R10,#0x08]
	MOV	R1,#0x80
	STRB	R1,[R10,#0x08]
	LDRH	R5,[R8]		;SSP1DR: dolzina okvirja (2byte)
	AND	R1,R5,#0xFF
	STRB	R1,[R10,#0x08]
	MOV	R1,R5,LSR#8
	STRB	R1,[R10,#0x08]
KETHTX1	LDRB	R1,[R10,#0x0C]	;SSP1SR: pocakaj SPI prenos?
	TST	R1,#0x10	;BSY=?
	BNE	KETHTX1
	LDRB	R1,[R10,#0x08]	;SSP1DR: precitaj 5 praznih bajtov
	LDRB	R1,[R10,#0x08]
	LDRB	R1,[R10,#0x08]
	LDRB	R1,[R10,#0x08]
	LDRB	R1,[R10,#0x08]
	LDR	R0,=S16BIT	;SSP1CR0: 16bit
	STRH	R0,[R10]
	ADD	R0,R5,#3	;izracunaj stevilo 16-bitnih prenosov
	MOV	R0,R0,LSR#2
	MOV	R0,R0,LSL#1
	STRH	R0,[R8,#4]
	LDRH	R0,[R8,#2]	;izracunaj kazalec v okvirju
	ADD	R0,R0,R8
	STR	R0,[R8,#8]
	LDR	R0,=KTXBLOK	;shrani oddajni blok v KRXBLOK
	STR	R8,[R0]
	LDR	PC,[SP],#4
	LTORG
	ALIGN	16
KETHTX2	STR	LR,[SP,#-4]!	;resi link - vstop zanke prepisovanja z R8
	LDRH	R6,[R8,#4]	;stevilo prenosov v R6
	LDR	R5,[R8,#8]	;kazalec v okvirju v R5
KETHTX3	LDRB	R1,[R10,#0x0C]	;SSP1SR: poln oddajnik?
	TST	R1,#0x02	;TNF=?
	BEQ	KETHTX6
	LDRH	R0,[R5],#2	;SSP1DR: prepisi 16 bitov okvirja
	MOV	R0,R0,LSL#8
	ORR	R0,R0,R0,LSR#16
	STRH	R0,[R10,#0x08]
	SUBS	R6,R6,#1	;stevec prenosov
	BNE	KETHTX3
KETHTX4	LDRB	R1,[R10,#0x0C]	;SSP1SR: pocakaj SPI prenos?
	TST	R1,#0x10	;BSY=?
	BNE	KETHTX4
KETHTX5	LDRB	R1,[R10,#0x0C]	;SSP1SR: prazen sprejemnik?
	TST	R1,#0x04	;RNE=?
	LDRHNE	R0,[R10,#0x08]	;precitaj dva prazna bajta
	BNE	KETHTX5
	LDR	R2,=0x3FFFC014	;FIO0PIN0 izklopi CSN
	MOV	R1,#0xFF
	STRB	R1,[R2]
	MOV	R0,#0x82	;RXQCR
	MOV	R3,#0x0230	;Stop QMU DMA transfer operation
	BL	KSZ16W
	MOV	R0,#0x80	;TXQCR
	MOV	R3,#0x0001	;TxQ Manual-Enqueue
	BL	KSZ16W
	LDR	R0,=KTXBLOK	;sprosti KTXBLOK
	MOV	R1,#0
	STR	R1,[R0]
	BL	SPROSTI
	LDR	PC,[SP],#4
	ALIGN	16
KETHTX6	STRH	R6,[R8,#4]	;shrani stevilo prenosov
	STR	R5,[R8,#8]	;shrani kazalec v okvirju
	LDR	PC,[SP],#4
	LTORG
KETHTX7	LDR	R0,=KTXVMES	;sprosti okvir, ko ni KSZ8851SNL
	BL	IZVMES
	BL	SPROSTI
	LDR	PC,[SP],#4
	LTORG

;*** Sprejem ETH okvirjev iz KSZ8851SNL *** (R0-8,R10)
KRXZAC	EQU	76		;zacetek podatkov ETH okvirja
KRXMIN	EQU	66		;min dolzina ETH okvirja +2
KRXMAX	EQU	1536		;max dolzina ETH okvirja +2
;dodatni podatki v glavi KRXBLOK/KTXBLOK (R8):
;odmik 0 - dolzina koristne vsebine (16bit)
;odmik 2 - zacetek koristne vsebine (16bit)
;odmik 4 - stevilo 16-bitnih prenosov (16bit)
;odmik 6 - stevilo ETH okvirjev (16bit)
;odmik 8 - kazalec prepisovanja v okvirju (32bit)
	ALIGN	16
KETHRX	LDR	R0,=KRXBLOK	;obstaja KRXBLOK?
	LDR	R8,[R0]
	MOVS	R8,R8
	BNE	KETHRX6
	LDR	R0,=KTXBLOK	;obstaja KTXBLOK?
	LDR	R8,[R0]
	MOVS	R8,R8
	MOVNE	PC,LR
	STR	LR,[SP,#-4]!	;resi link
	MOV	R0,#0xF8	;P1SR precitaj stanje PHY
	BL	KSZ16R
	LDR	R0,=PHYSTAT
	STRH	R3,[R0]
	LDR	R0,=KSZID	;veljaven KSZ8851SNL?
	LDR	R1,[R0]
	MOVS	R1,R1
	LDRMI	PC,[SP],#4	;izhod, ko ni KSZ8851SNL
	MOV	R0,#0x92	;ISR sprejem okvirja?
	BL	KSZ16R
	TST	R3,#0x2000
	LDREQ	PC,[SP],#4
	BL	PROSTO		;prestej proste bloke pomnilnika v R0
	LDR	R1,=MINBLOK
	LDRB	R2,[R1]
	CMP	R0,R2		;pomanjkanje blokov zaustavi ETH sprejem?
	LDRCC	PC,[SP],#4
	MOV	R0,#0x92	;ISR pocisti sprejem
	MOV	R3,#0x2000
	BL	KSZ16W
	MOV	R0,#0x9C	;RXFCTR stevilo sprejetih okvirjev v R7?
	BL	KSZ16R
	MOVS	R7,R3,LSR#8
	LDREQ	PC,[SP],#4
KETHRX1	MOV	R0,#0x7C	;RXFHSR received frame status v R6
	BL	KSZ16R
	MOV	R6,R3
	MOV	R0,#0x7E	;RXFHBCR received frame byte size v R5
	BL	KSZ16R
	BIC	R5,R3,#0xF000
	TST	R6,#0x8000	;RXFV Receive Frame Valid?
	BEQ	KETHRX3
	TST	R6,#0x0001	;RXCE Receive CRC Error?
	BNE	KETHRX3
	CMP	R5,#KRXMIN	;prekratek okvir?
	BCC	KETHRX3
	CMP	R5,#KRXMAX	;predolg okvir?
	BCS	KETHRX3
	BL	DODELI		;dodeli blok za nov okvir
	MOVS	R8,R8		;zavrzi okvir, ce ni bloka?
	BEQ	KETHRX3
	MOV	R0,#0x86	;RXFDPR
	MOV	R3,#0x4000	;Reset QMU RXQ frame pointer to zero
	BL	KSZ16W
	MOV	R0,#0x82	;RXQCR
	MOV	R3,#0x0238	;Start QMU DMA transfer operation
	BL	KSZ16W
	LDR	R0,=S8BIT	;SSP1CR0: 8bit
	STRH	R0,[R10]
	LDR	R2,=0x3FFFC014	;FIO0PIN0 vklopi CSN
	MOV	R1,#0x00
	STRB	R1,[R2]
	MOV	R1,#0x80	;SSP1DR: Read RXQ command phase
	STRB	R1,[R10,#0x08]
KETHRX2	LDRB	R1,[R10,#0x0C]	;SSP1SR: pocakaj SPI prenos?
	TST	R1,#0x10	;BSY=?
	BNE	KETHRX2
	LDRB	R1,[R10,#0x08]	;SSP1DR: precitaj prazen bajt
	LDR	R0,=S16BIT	;SSP1CR0: 16bit
	STRH	R0,[R10]
	SUB	R0,R5,#6	;izracunaj dolzino ETH okvirja
	STRH	R0,[R8]
	MOV	R0,#KRXZAC	;izracunaj zacetek ETH okvirja
	ADD	R0,R0,#10
	STRH	R0,[R8,#2]
	ADD	R0,R5,#11	;izracunaj stevilo 16-bitnih prenosov
	MOV	R0,R0,LSR#2
	MOV	R0,R0,LSL#1
	SUB	R0,R0,#7
	STRH	R0,[R8,#4]
	STRH	R7,[R8,#6]	;stevilo okvirjev
	ADD	R0,R8,#KRXZAC	;kazalec v okvirju
	STR	R0,[R8,#8]
	LDR	R0,=KRXBLOK	;shrani sprejemni blok v KRXBLOK
	STR	R8,[R0]
	MOV	R1,#-1		;SSP1DR: vpisi PRVIH 16 praznih bajtov
	STRH	R1,[R10,#0x08]
	STRH	R1,[R10,#0x08]
	STRH	R1,[R10,#0x08]
	STRH	R1,[R10,#0x08]
	STRH	R1,[R10,#0x08]
	STRH	R1,[R10,#0x08]
	STRH	R1,[R10,#0x08]
	STRH	R1,[R10,#0x08]
	LDR	PC,[SP],#4
KETHRX3	MOV	R0,#0x82	;RXQCR RELEASE error frame command
	BL	KSZ16R
	ORR	R3,R3,#0x0001
	BL	KSZ16W
KETHRX4	BL	KSZ16R		;cakaj konec sproscanja pomnilnika?
	TST	R3,#0x0001
	BNE	KETHRX4
KETHRX5	SUBS	R7,R7,#1	;zanka po okvirjih
	BNE	KETHRX1
	LDR	PC,[SP],#4
	LTORG
	ALIGN	16
KETHRX6	STR	LR,[SP,#-4]!	;resi link - vstop zanke prepisovanja z R8
	LDRH	R6,[R8,#4]	;stevilo prenosov v R6
	LDR	R5,[R8,#8]	;kazalec v okvirju v R5
	MOV	R2,#-1		;prazni bajti
KETHRX7	LDRB	R1,[R10,#0x0C]	;SSP1SR: prazen sprejemnik?
	TST	R1,#0x04	;RNE=?
	BEQ	KETHRX9
	LDRH	R0,[R10,#0x08]	;precitaj dva podatkovna bajta
	MOV	R0,R0,LSL#8
	ORR	R0,R0,R0,LSR#16
	STRH	R0,[R5],#2
	SUBS	R6,R6,#1	;konec zanke prepisovanja?
	STRHNE	R2,[R10,#0x08]	;SSP1DR: vpisi dva prazna bajta
	BNE	KETHRX7
KETHRX8	LDRB	R1,[R10,#0x0C]	;SSP1SR: pocakaj SPI prenos?
	TST	R1,#0x10	;BSY=?
	BNE	KETHRX8
	TST	R1,#0x04	;RNE=? prazen sprejemnik?
	BEQ	KETHR10
	LDRH	R0,[R10,#0x08]	;SSP1DR: precitaj ZADNJE podatkovne bajte
	MOV	R0,R0,LSL#8
	ORR	R0,R0,R0,LSR#16
	STRH	R0,[R5],#2
	B	KETHRX8
	ALIGN	16
KETHRX9	STRH	R6,[R8,#4]	;shrani stevilo prenosov
	STR	R5,[R8,#8]	;shrani kazalec v okvirju
	LDR	PC,[SP],#4
	ALIGN	16
KETHR10	LDR	R2,=0x3FFFC014	;FIO0PIN0 izklopi CSN
	MOV	R1,#0xFF
	STRB	R1,[R2]
	MOV	R0,#0x82	;RXQCR
	MOV	R3,#0x0230	;Stop QMU DMA transfer operation
	BL	KSZ16W
	LDR	R0,=KRXBLOK	;sprosti KRXBLOK
	MOV	R1,#0
	STR	R1,[R0]
	LDRH	R7,[R8,#6]	;stevilo okvirjev v R7
	LDR	R2,=MACMAC	;prepisi MAC posiljatelja v MACMAC
	LDRH	R1,[R8,#2]
	ADD	R1,R1,#6
	ADD	R1,R1,R8
	LDRH	R0,[R1],#2
	STRH	R0,[R2],#2
	LDRH	R0,[R1],#2
	STRH	R0,[R2],#2
	LDRH	R0,[R1],#2
	STRH	R0,[R2],#2
	BL	KETHR11		;POZOR! Ohrani R7!
	B	KETHRX5
KETHR11	STR	LR,[SP,#-4]!	;resi link
	LDR	R0,=LNASLOV	;Ethernet vkljucen (L<>0)?
	LDR	R0,[R0]
	MOVS	R0,R0
	BEQ	KETHR12
	LDRH	R5,[R8,#2]	;zacetek ETH okvirja v R5
	ADD	R5,R5,R8
	LDRB	R0,[R5,#12]	;preveri IP4 ali ARP?
	CMP	R0,#0x08
	BNE	KETHR12
	LDRB	R0,[R5,#13]	;preveri IP4?
	CMP	R0,#0x00
	BEQ	ETHIP
	LDRB	R0,[R5,#13]	;preveri ARP?
	CMP	R0,#0x06
	BEQ	ETHARP
KETHR12	BL	SPROSTI		;sprosti neznane okvirje
	LDR	PC,[SP],#4
	LTORG

;*** Obdelava Ethernet IP4 okvirjev *** (R0,R8,SP+4)+IPODDAJ,VVMES
ETHIP	LDRH	R0,[R8]		;odrezi ETH glavo
	SUB	R0,R0,#14
	STRH	R0,[R8]
	LDRH	R0,[R8,#2]	;zacetek IP vsebine
	ADD	R0,R0,#14
	STRH	R0,[R8,#2]
	LDR	R0,=MNASLOV	;brez naslova Ethernet >>> SLIP?
	LDR	R0,[R0]
	MOVS	R0,R0
	BNE	IPODDAJ		;oddaja IP okvirjev
	LDR	R0,=STXVMES	;okvir v SLIP oddajni vmesnik
	BL	VVMES
	LDR	PC,[SP],#4
	LTORG

;*** Obdelava Ethernet ARP okvirjev (R5,R8) *** (R0-6,R8-9,SP+4)+VCACHE,ETHTX
ARPOP	EQU	21	;Operation (byte) 1=vprasanje, 2=odgovor
ARPSHA	EQU	22	;Sender Hardware Address
ARPSPA	EQU	28	;Sender Protocol Address
ARPTHA	EQU	32	;Target Hardware Address
ARPTPA	EQU	38	;Target Protocol Address
ARPZAC	EQU	200	;zacetek ARP odgovora
ETHARP	LDRB	R0,[R5,#ARPOP]	;ARP vprasanje/odgovor?
	CMP	R0,#1
	BNE	ETHARP2
	MOV	R4,#0		;ARP vprasanje v ARP cache?
	LDR	R0,=ARPMODE
	LDRB	R1,[R0]
	TST	R1,#2
	BLNE	VCACHE		;ARP podatki (R5), odziv R4 v ARP cache
	ADD	R0,R5,#ARPSPA	;IP izprasevalca v R2
	LDRH	R2,[R0],#2
	LDRH	R1,[R0],#2
	ORR	R2,R2,R1,ROR#16
	ADD	R0,R5,#ARPTPA	;ciljni IP v R3
	LDRH	R3,[R0],#2
	LDRH	R1,[R0],#2
	ORR	R3,R3,R1,ROR#16
	CMP	R2,R3		;izloci enake IPje?
	BEQ	ETHARP1
	LDR	R0,=DOMENA	;primerjava z IP domeno
	LDR	R1,[R0],#4
	EOR	R1,R1,R3
	LDR	R2,[R0],#4	;maska sprejmi?
	TST	R1,R2
	BNE	ETHARP1
	LDR	R2,[R0],#4	;maska zavrni?
	TST	R1,R2
	BEQ	ETHARP1
	ADD	R2,R8,#ARPZAC	;sestavljanje ARP odgovora (R2)+
	ADD	R0,R5,#ARPSHA	;MAC izprasevalca
	LDRH	R3,[R0],#2
	STRH	R3,[R2],#2
	LDRH	R3,[R0],#2
	STRH	R3,[R2],#2
	LDRH	R3,[R0],#2
	STRH	R3,[R2],#2
	LDR	R0,=ETHMAC	;lastni MAC
	LDRH	R3,[R0],#2
	STRH	R3,[R2],#2
	LDRH	R3,[R0],#2
	STRH	R3,[R2],#2
	LDRH	R3,[R0],#2
	STRH	R3,[R2],#2
	MOV	R0,#0x08	;type=0x0806 ARP
	STRB	R0,[R2],#1
	MOV	R0,#0x06
	STRB	R0,[R2],#1
	MOV	R0,#0x00	;hardware=0x0001 Ethernet
	STRB	R0,[R2],#1
	MOV	R0,#0x01
	STRB	R0,[R2],#1
	MOV	R0,#0x08	;IP4=0x0800
	STRB	R0,[R2],#1
	MOV	R0,#0x00
	STRB	R0,[R2],#1
	MOV	R0,#0x06	;HLEN=0x06 Ethernet
	STRB	R0,[R2],#1
	MOV	R0,#0x04	;PLEN=0x04 IP
	STRB	R0,[R2],#1
	MOV	R0,#0x00	;OPER=0x0002 odgovor
	STRB	R0,[R2],#1
	MOV	R0,#0x02
	STRB	R0,[R2],#1
	LDR	R0,=ETHMAC	;SHA=lastni MAC
	LDRH	R3,[R0],#2
	STRH	R3,[R2],#2
	LDRH	R3,[R0],#2
	STRH	R3,[R2],#2
	LDRH	R3,[R0],#2
	STRH	R3,[R2],#2
	ADD	R0,R5,#ARPTPA	;SPA=ciljni IP
	LDRH	R3,[R0],#2
	STRH	R3,[R2],#2
	LDRH	R3,[R0],#2
	STRH	R3,[R2],#2
	ADD	R0,R5,#ARPSHA	;THA=MAC izprasevalca
	LDRH	R3,[R0],#2
	STRH	R3,[R2],#2
	LDRH	R3,[R0],#2
	STRH	R3,[R2],#2
	LDRH	R3,[R0],#2
	STRH	R3,[R2],#2
	ADD	R0,R5,#ARPSPA	;TPA=IP izprasevalca
	LDRH	R3,[R0],#2
	STRH	R3,[R2],#2
	LDRH	R3,[R0],#2
	STRH	R3,[R2],#2
	SUB	R0,R2,R8	;izracunaj dolzino sestavljenega odgovora
	SUB	R0,R0,#ARPZAC
	STRH	R0,[R8]
	MOV	R0,#ARPZAC	;zacetek odgovora
	STRH	R0,[R8,#2]
	LDR	R0,=KTXVMES	;APR odgovor v oddajo
	BL	VVMES
	LDR	PC,[SP],#4
ETHARP1	BL	SPROSTI		;sprosti blok zavrnjenega ARP vprasanja
	LDR	PC,[SP],#4
	LTORG
ETHARP2	LDR	R0,=ETXBLOK	;ARP odgovor >>> obstaja ETXBLOK?
	LDR	R6,[R0]
	MOVS	R6,R6
	BEQ	ETHARP3		;nepoklicani (zamujeni) ARP v cache
	LDR	R0,[R6,#4]	;ARP IP4 cilj iz ETXBLOK v R0
	ADD	R1,R5,#ARPSPA	;IP naslovnika ARPodgovor v R2
	LDRH	R2,[R1],#2
	LDRH	R3,[R1],#2
	ORR	R2,R2,R3,ROR#16
	CMP	R0,R2		;neenaka IPja?
	BNE	ETHARP3		;nepoklicani (zamujeni) ARP v cache
	LDR	R4,=ARPCAS	;izracunaj odzivni cas ARP v R4
	LDR	R4,[R4]
	LDR	R0,=0xE0004008	;T0TC - Timer0
	LDR	R0,[R0]
	SUB	R4,R0,R4
	BL	VCACHE		;ARP podatki (R5), odziv R4 v ARP cache
	BL	SPROSTI		;sprosti blok ARP odgovora
	LDR	R0,=ETXBLOK	;oddaj blok (R8) na ARP odgovor
	LDR	R8,[R0]
	MOV	R1,#0		;brisi obstojeci ETXBLOK
	STR	R1,[R0]
	LDR	R5,=ECACHE	;MAC(R5) iz ECACHE
	ADD	R5,R5,#4
	B	ETHTX
	LTORG
ETHARP3	LDR	R0,=TIMEOUT	;nepoklicani (zamujeni) ARP v cache?
	LDR	R4,[R0]
	LDR	R0,=ARPMODE
	LDRB	R1,[R0]
	TST	R1,#1
	BLNE	VCACHE		;ARP podatki (R5), odziv R4 v ARP cache
	BL	SPROSTI		;sprosti blok ARP odgovora
	LDR	PC,[SP],#4
	LTORG

;*** ARP podatki (R5), odzivni cas R4 v ARP cache *** (R0-6,R9)
VCACHE	ADD	R0,R5,#ARPSPA	;IP naslov (4byte) v R1
	LDRH	R1,[R0],#2
	LDRH	R6,[R0],#2
	ORR	R1,R1,R6,ROR#16
	ADD	R0,R5,#ARPSHA	;MAC naslov (prvi 4 od 6byte) v R2
	LDRH	R2,[R0],#2
	LDRH	R6,[R0],#2
	ORR	R2,R2,R6,ROR#16
	LDRH	R3,[R0],#2	;MAC naslov (zadnja 2 od 6byte) v spodnji R3
	LDR	R0,=TCACHE	;trajanje (2byte) v gornji R3
	LDRH	R6,[R0]
	ORR	R3,R3,R6,ROR#16
	MOVS	R6,R1		;IP naslov dodatno v R6
	LDR	R0,=ECACHE
	MOV	R9,#MCACHE
VCACHE1	SUBS	R9,R9,#1	;konec seznama?
	MOVMI	PC,LR
	SWP	R1,R1,[R0]	;zamenjaj R1-4 v seznamu
	ADD	R0,R0,#4
	SWP	R2,R2,[R0]
	ADD	R0,R0,#4
	SWP	R3,R3,[R0]
	ADD	R0,R0,#4
	SWP	R4,R4,[R0]
	ADD	R0,R0,#4
	CMP	R1,R6		;enaka IP naslova?
	MOVEQ	PC,LR
	TST	R3,#0xFF000000	;veljaven vnos?
	BNE	VCACHE1
	TST	R3,#0x00FF0000
	BNE	VCACHE1
	MOV	PC,LR
	LTORG

;*** Oddaj blok (R8) prejemniku (R5) v KTXVMES *** (R0-5,R8,SP+4)
KMINZAC	EQU	28		;minimalni zacetek okvirja
ETHTX	LDRH	R0,[R8,#2]	;nadzor zacetka okvirja?
	CMP	R0,#KMINZAC
	BCS	ETHTX1
	BL	SPROSTI		;sprosti neprimeren zacetek
	LDR	PC,[SP],#4
ETHTX1	SUB	R0,R0,#14	;zacetek ETH vsebine
	STRH	R0,[R8,#2]
	ADD	R2,R8,R0	;kazalec v ETH glavi R2
	LDRH	R0,[R8]		;dodaj dolzino ETH glave
	ADD	R0,R0,#14
	STRH	R0,[R8]
	LDRH	R1,[R5],#2	;naslovnikov MAC(R5)
	STRH	R1,[R2],#2
	LDRH	R1,[R5],#2
	STRH	R1,[R2],#2
	LDRH	R1,[R5],#2
	STRH	R1,[R2],#2
	LDR	R3,=ETHMAC	;lastni MAC v posiljatelja
	LDRH	R1,[R3],#2
	STRH	R1,[R2],#2
	LDRH	R1,[R3],#2
	STRH	R1,[R2],#2
	LDRH	R1,[R3],#2
	STRH	R1,[R2],#2
	MOV	R1,#8		;oznaka Ethernet IP4 0x08,0x00
	STRB	R1,[R2],#1
	MOV	R1,#0
	STRB	R1,[R2],#1
	LDR	R0,=KTXVMES	;IP4 okvir v oddajo
	BL	VVMES
	LDR	PC,[SP],#4
	LTORG

;*** Ethernet oddaja IP okvirjev z/brez ARP *** (R0-6,R8)
ETXMAX	EQU	1501		;max dolzina +1 (MTU+1)
ETXMIN	EQU	21		;min dolzina 21 (IP4glava+1)
	ALIGN	16
EODDAJ	STR	LR,[SP,#-4]!	;resi link
	LDR	R0,=ETXBLOK	;cakajoci ETXBLOK na ARP?
	LDR	R8,[R0]
	MOVS	R8,R8
	BNE	EODDAJ4
	LDR	R0,=ETXVMES	;ETH TX vmesnik prazen?
	BL	IZVMES
	MOVS	R8,R8
	LDREQ	PC,[SP],#4
	LDRH	R0,[R8] 	;min dolzina IP4 okvirja?
	CMP	R0,#ETXMIN
	BCC	EODDAJ3
	LDR	R1,=ETXMAX	;max dolzina IP4 okvirja?
	CMP	R0,R1
	BCS	EODDAJ3
	LDRH	R0,[R8,#2]	;prvi znak IP4 okvirja?
	LDRB	R0,[R8,R0]
	CMP	R0,#IP4MIN
	BCC	EODDAJ3
	CMP	R0,#IP4MAX
	BCS	EODDAJ3
	LDRH	R0,[R8,#2]	;iskani naslovnikov IP v R6
	ADD	R0,R0,R8
	LDR	R6,[R0,#IPCILJ]
	AND	R0,R6,#0xFF	;prvi byte prejemnika IP4
	CMP	R0,#0xE0	;Unicast?
	BCC	EODDA10
	CMP	R0,#0xF0	;Multicast?
	BCC	EODDA12
	LDR	R5,=EODDA11
	B	ETHTX		;takoj oddaj okvir (R8) na MAC (R5) brez ARP!
EODDA11	DCB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF	;Broadcast MAC
	ALIGN	4
EODDA12	LDR	R5,=MACMAC	;izracunaj class-D multicast MAC v MACMAC
	MOV	R0,#0x01
	STRB	R0,[R5]
	MOV	R0,#0x00
	STRB	R0,[R5,#1]
	MOV	R0,#0x5E
	STRB	R0,[R5,#2]
	MOV	R0,R6,LSR#8	;predpredzadnji byte multicast MAC
	AND	R0,R0,#0x7F
	STRB	R0,[R5,#3]
	MOV	R0,R6,LSR#16	;predzadnji byte multicast MAC
	AND	R0,R0,#0xFF
	STRB	R0,[R5,#4]
	MOV	R0,R6,LSR#24	;zadnji byte multicast MAC
	AND	R0,R0,#0xFF	
	STRB	R0,[R5,#5]
	B	ETHTX		;takoj oddaj okvir (R8) na MAC (R5) brez ARP!
EODDA10	LDR	R0,=GATEWAY	;privzeti prehod R1 in maska R2
	LDR	R1,[R0],#4
	LDR	R2,[R0],#4
	EOR	R0,R1,R6	;primerjaj s privzetim prehodom/maska
	TST	R0,R2
	MOVNE	R6,R1		;zamenjaj IP naslovnika s prehodom?
	STR	R6,[R8,#4]	;ARP IP4 cilj v ETXBLOK
	LDR	R5,=ECACHE	;poisci veljaven vpis v ECACHE
	MOV	R4,#MCACHE
EODDAJ1	SUBS	R4,R4,#1	;konec seznama?
	BMI	EODDAJ2
	LDRH	R0,[R5,#10]	;veljaven vpis?
	MOVS	R0,R0
	BEQ	EODDAJ2
	LDR	R0,[R5],#4	;enaki IPji?
	CMP	R0,R6	
	BEQ	ETHTX		;takoj oddaj okvir (R8) na MAC (R5) brez ARP!
	ADD	R5,R5,#12
	B	EODDAJ1
EODDAJ2	LDR	R0,=ETXBLOK	;okvir (R8) v ETXBLOK, sestavi ARP vprasanje
	STR	R8,[R0]
	MOV	R5,R8		;naslov okvirja zacasno v R5
	LDR	R0,=0xE0004008	;T0TC - Timer0 v cas ARP vprasanja
	LDR	R1,[R0]
	LDR	R0,=ARPCAS
	STR	R1,[R0]
	BL	DODELI		;dodeli blok za ARP vprasanje?
	MOVS	R8,R8
	LDREQ	PC,[SP],#4
	MOV	R0,#ARPZAC	;zacetek ARP vprasanja v blok in R2
	STRH	R0,[R8,#2]
	ADD	R2,R8,R0
	MOV	R0,#-1		;naslovnik VSEM
	STRH	R0,[R2],#2
	STRH	R0,[R2],#2
	STRH	R0,[R2],#2
	LDR	R0,=ETHMAC	;lastni MAC
	LDRH	R3,[R0],#2
	STRH	R3,[R2],#2
	LDRH	R3,[R0],#2
	STRH	R3,[R2],#2
	LDRH	R3,[R0],#2
	STRH	R3,[R2],#2
	MOV	R0,#0x08	;type=0x0806 ARP
	STRB	R0,[R2],#1
	MOV	R0,#0x06
	STRB	R0,[R2],#1
	MOV	R0,#0x00	;hardware=0x0001 Ethernet
	STRB	R0,[R2],#1
	MOV	R0,#0x01
	STRB	R0,[R2],#1
	MOV	R0,#0x08	;IP4=0x0800
	STRB	R0,[R2],#1
	MOV	R0,#0x00
	STRB	R0,[R2],#1
	MOV	R0,#0x06	;HLEN=0x06 Ethernet
	STRB	R0,[R2],#1
	MOV	R0,#0x04	;PLEN=0x04 IP
	STRB	R0,[R2],#1
	MOV	R0,#0x00	;OPER=0x0001 vprasanje
	STRB	R0,[R2],#1
	MOV	R0,#0x01
	STRB	R0,[R2],#1
	LDR	R0,=ETHMAC	;SHA=lastni MAC
	LDRH	R3,[R0],#2
	STRH	R3,[R2],#2
	LDRH	R3,[R0],#2
	STRH	R3,[R2],#2
	LDRH	R3,[R0],#2
	STRH	R3,[R2],#2
	LDRH	R0,[R5,#2]	;SPA=IP izprasevalca 
	ADD	R0,R0,R5
	ADD	R0,R0,#IPIZVOR
	LDRH	R3,[R0],#2
	STRH	R3,[R2],#2
	LDRH	R3,[R0],#2
	STRH	R3,[R2],#2
	MOV	R0,#0		;THA=0
	STRH	R0,[R2],#2
	STRH	R0,[R2],#2
	STRH	R0,[R2],#2
	STRH	R6,[R2],#2	;TPA=IP naslovnika/prehoda iz R6
	MOV	R6,R6,ROR#16
	STRH	R6,[R2],#2
	SUB	R0,R2,R8	;izracunaj dolzino ARP vprasanja
	SUB	R0,R0,#ARPZAC
	STRH	R0,[R8]
	LDR	R0,=KTXVMES	;APR vprasanje v oddajo
	BL	VVMES
	LDR	PC,[SP],#4
	LTORG
EODDAJ3	BL	SPROSTI		;sprosti okvir, ki ni IP4
	LDR	PC,[SP],#4
	ALIGN	16
EODDAJ4	LDR	R0,=ARPCAS	;brez ARP preveri cas oddaje
	LDR	R1,[R0]
	LDR	R0,=0xE0004008	;T0TC - Timer0
	LDR	R0,[R0]
	SUB	R1,R0,R1
	LDR	R0,=TIMEOUT	;primerjaj s casovno mejo?
	LDR	R0,[R0]
	CMP	R1,R0
	LDRCC	PC,[SP],#4
	LDR	R0,=ETXBLOK	;brisi obstojeci ETXBLOK
	MOV	R1,#0
	STR	R1,[R0]
	BL	SPROSTI		;sprosti okvir brez MAC naslova
	LDR	PC,[SP],#4
	LTORG

;*** Preveri takt ARM procesorja *** (R0-2)
MINTAKT	EQU	40000000	;40MHz
MAXTAKT	EQU	120000000	;120MHz
REFTAKT	EQU	58982400	;58.9824MHz=4*14.7456MHz
REFQ	EQU	14745600
REFM	EQU	4
INITAKT	LDR	R0,=QUARTZ	;izracunaj takt ARM procesorja v R1
	LDR	R1,[R0]
	BIC	R2,R1,#0xF0000000
	MOV	R0,R1,LSR#28
	MUL	R1,R0,R2
	LDR	R2,=MINTAKT	;preveri min.takt
	CMP	R1,R2
	BCC	INITAK1
	LDR	R2,=MAXTAKT	;preveri max.takt
	CMP	R1,R2
	BCS	INITAK1
	LDR	R0,=ARMTAKT	;shrani preverjeno
	STR	R1,[R0]
	MOV	PC,LR
INITAK1	LDR	R0,=ARMTAKT	;privzete vrednosti
	LDR	R1,=REFTAKT
	STR	R1,[R0]
	LDR	R1,=REFQ
	LDR	R0,=REFM
	ORR	R1,R1,R0,LSL#28
	LDR	R0,=QUARTZ
	STR	R1,[R0]
	MOV	PC,LR
	LTORG

;*** Vkljuci psa cuvaja v LPC23xx *** (R0-1)
PESCAKA	EQU	7000000		;7s@4MHzRCosc
PESJAN	LDR	R0,=0xE0000004	;WDTC
	LDR	R1,=PESCAKA
	STR	R1,[R0]
	LDR	R0,=0xE0000000	;WDMOD=3
	MOV	R1,#3
	STRB	R1,[R0]
	LDR	R0,=0xE0000008	;WDFEED pes vklop
	MOV	R1,#0xAA
	STRB	R1,[R0]
	MOV	R1,#0x55
	STRB	R1,[R0]
	MOV	PC,LR
	LTORG

;*** Pes cuva sprejem v LPC23xx *** (R0-1)
MAXTX	EQU	980		;MAX TX 98% (v promilih)
PESCUVA	LDR	R0,=NPTT
	LDR	R0,[R0]
	CMP	R0,#MAXTX
	MOVCS	PC,LR
	LDR	R0,=0xE0000008	;WDFEED pes cuva
	MOV	R1,#0xAA
	STRB	R1,[R0]
	MOV	R1,#0x55
	STRB	R1,[R0]
	MOV	PC,LR
	LTORG

;*** Inicializiraj parametre iz FLASH *** (R0-3)
INIPAR	LDR	R0,=FNASLOV	;source Flash address
	LDR	R1,=RAMFPAR	;destination RAM address
	LDR	R2,=256
INIPAR1	LDR	R3,[R0],#4	;prepisi 256*4=1024byte
	STR	R3,[R1],#4		
	SUBS	R2,R2,#1
	BNE	INIPAR1
	MOV	PC,LR
	LTORG

;*** Vpis parametrov v FLASH pomnilnik *** (R0-4,SP+4)+RIZVEDI
FSECTOR	EQU	25		;sektor 25: 0x0007B000-0x0007BFFF
FNASLOV	EQU	0x0007B000	;zacetni naslov sektorja 25
IAPADDR	EQU	0x7FFFFFF1 	;IAP naslov (THUMB)
FLASH	MRS	R0,CPSR		;izkljuci prekinitve IRQ in FIQ
	ORR	R0,R0,#0x000000C0
	MSR	CPSR_c,R0	;popravi CPSR
	LDR	R0,=IAPCMD	;Prepare sector(s) for write operation
	MOV	R2,#50		;Command code 50
	STR	R2,[R0]
	MOV	R2,#FSECTOR	;Start Sector Number
	STR	R2,[R0,#4]
	STR	R2,[R0,#8]	;End Sector Number
	LDR	R1,=IAPRES
	LDR	R2,=IAPADDR	;klic na IAP naslov
	MOV	LR,PC
	BX	R2
	LDR	R0,=IAPCMD	;Erase sector(s)
	MOV	R2,#52		;Command code 52
	STR	R2,[R0]
	MOV	R2,#FSECTOR	;Start Sector Number
	STR	R2,[R0,#4]
	STR	R2,[R0,#8]	;End Sector Number
	LDR	R2,=ARMTAKT	;system clock frequency (CCLK) in kHz
	LDR	R2,[R2]
	MOV	R2,R2,LSR#10
	STR	R2,[R0,#12]
	LDR	R1,=IAPRES
	LDR	R2,=IAPADDR	;klic na IAP naslov
	MOV	LR,PC
	BX	R2
	LDR	R0,=IAPCMD	;Prepare sector(s) for write operation
	MOV	R2,#50		;Command code 50
	STR	R2,[R0]
	MOV	R2,#FSECTOR	;Start Sector Number
	STR	R2,[R0,#4]
	STR	R2,[R0,#8]	;End Sector Number
	LDR	R1,=IAPRES
	LDR	R2,=IAPADDR	;klic na IAP naslov
	MOV	LR,PC
	BX	R2
	LDR	R0,=IAPCMD	;Copy RAM to Flash
	MOV	R2,#51		;Command code 51
	STR	R2,[R0]
	LDR	R2,=FNASLOV	;Destination Flash address
	STR	R2,[R0,#4]
	LDR	R2,=RAMFPAR	;Source RAM address
	STR	R2,[R0,#8]
	MOV	R2,#1024	;Number of bytes to be written (1024)
	STR	R2,[R0,#12]
	LDR	R2,=ARMTAKT	;System Clock Frequency (CCLK) in kHz
	LDR	R2,[R2]
	MOV	R2,R2,LSR#10
	STR	R2,[R0,#16]
	LDR	R1,=IAPRES
	LDR	R2,=IAPADDR	;klic na IAP naslov
	MOV	LR,PC
	BX	R2
	B	RIZVEDI		;ARM reset
	LTORG

;*** Izberi PIN in GPIO v SATNC *** (R0-1)
PINGPIO	LDR	R0,=0xE002C000	;PINSEL0
	LDR	R1,=0x405AA05A	;P0.0-1=UART3,P0.2-3=UART0,P0.6-9=SPI1,P0.10-11=UART2,P0.15=UART1
	STR	R1,[R0]
	LDR	R0,=0xE002C004	;PINSEL1
	LDR	R1,=0x00154001	;P0.16=UART1,P0.23-26=AD
	STR	R1,[R0]
	LDR	R0,=0xE002C044	;PINMODE1
	LDR	R1,=0x002A8000	;P0.23-26 brez pullup/pulldown
	STR	R1,[R0]
	LDR	R0,=0xE002C008	;PINSEL2
	LDR	R1,=0x50150105	;P1.0,1,4,8,9,10,14,15=EthernetRMII
	STR	R1,[R0]
	LDR	R0,=0xE01FC1A0	;SCS
	LDR	R1,=0x00000021	;main oscillator enable, high-speed GPIO 0&1
	STR	R1,[R0]
	LDR	R0,=0x3FFFC000	;FIO0DIR
	LDR	R1,=0x00000020	;izhod P0.5
	STR	R1,[R0]
	LDR	R0,=0x3FFFC014	;FIO0PIN0 izklopi CSN
	MOV	R1,#0xFF
	STRB	R1,[R0]
	LDR	R0,=0x3FFFC020	;FIO1DIR izhodi P1.16, P1.30(AD0.4) in P1.31
	LDR	R1,=0xC0010000
	STR	R1,[R0]
	LDR	R0,=0x3FFFC036	;FIO1PIN2 izklopi oddajnik
	MOV	R1,#-1
	STRB	R1,[R0]
	LDR	R0,=0x3FFFC037	;FIO1PIN3 nicla AD0.4 in izklopi telekomando
	MOV	R1,#0
	STRB	R1,[R0]
	LDR	R0,=0x3FFFC080	;FIO4DIR izhoda P4.28-29
	LDR	R1,=0x30000000
	STR	R1,[R0]
	MOV	PC,LR
	LTORG

;*** Inicializiraj PLL in MAM v LPC23xx *** (R0-2)
PLLMAM	LDR	R0,=0xE01FC1A0	;SCS
	LDR	R1,=0x00000021	;main oscillator enable, high-speed GPIO 0&1
	STR	R1,[R0]
	LDR	R1,[R0]		;main oscillator ready?
	TST	R1,#0x00000040
	BEQ	PLLMAM
	LDR	R0,=0xE01FC104	;CCLKCFG
	MOV	R1,#5		;deljenje /6
	STRB	R1,[R0]
	LDR	R0,=0xE01FC108	;USBCLKCFG
	MOV	R1,#5		;deljenje /6
	STRB	R1,[R0]
	LDR	R0,=0xE01FC10C	;CLKSRCSEL
	MOV	R1,#1		;main oscillator
	STRB	R1,[R0]
	LDR	R0,=QUARTZ	;izracunaj M*3-1, N-1=0
	LDR	R1,[R0]
	MOV	R1,R1,LSR#28
	ADD	R1,R1,R1,LSL#1
	SUB	R1,R1,#1
	LDR	R0,=0xE01FC084	;PLLCFG
	STR	R1,[R0]
	LDR	R0,=0xE01FC080	;PLLCON=0x01 PLL-enable
	MOV	R1,#0x01
	STRB	R1,[R0]
	LDR	R0,=0xE01FC08C	;PLLFEED
	MOV	R1,#0xAA
	MOV	R2,#0x55
	STRB	R1,[R0]
	STRB	R2,[R0]
PLLMAM1	LDR	R0,=0xE01FC088	;PLLSTAT cakanje uklenitve
	LDR	R1,[R0]
	TST	R1,#0x04000000
	BEQ	PLLMAM1
	LDR	R0,=0xE01FC080	;PLLCON=0x03 PLL-enable&connect
	MOV	R1,#0x03
	STRB	R1,[R0]
	LDR	R0,=0xE01FC08C	;PLLFEED
	MOV	R1,#0xAA
	MOV	R2,#0x55
	STRB	R1,[R0]
	STRB	R2,[R0]
	LDR	R0,=0xE01FC000	;MAMCR=0
	MOV	R1,#0
	STRB	R1,[R0]
	LDR	R0,=0xE01FC004	;MAMTIM=3
	MOV	R1,#3
	STRB	R1,[R0]
	LDR	R0,=0xE01FC000	;MAMCR=2
	MOV	R1,#2
	STRB	R1,[R0]
	MOV	PC,LR
	LTORG

;*** Vklop in takt I/O enot SATNC *** (R0-1)
PWRCLK	LDR	R0,=0xE01FC0C4	;PCONP
	LDR	R1,=0x4300141A 	;ETH,UART3&2,AD,SSP1,UART1&0,TIM0
	STR	R1,[R0]
	LDR	R0,=0xE01FC1A8	;PCLKSEL0
	LDR	R1,=0x01100144	;AD,SSP1,UART1&0,TIM0
	STR	R1,[R0]
	LDR	R0,=0xE01FC1AC	;PCLKSEL1
	LDR	R1,=0x10050015	;SYSCON,UART3&2,PCB,GPIO,BATRAM
	STR	R1,[R0]
	MOV	PC,LR
	LTORG

;*** Inicializiraj Timer0 *** (R0-1)
TIMER0	STR	LR,[SP,#-4]!
	LDR	R0,=1000000	;miljon mikrosekund
	LDR	R3,=ARMTAKT	;izracunaj modulo iz PCLK
	LDR	R3,[R3]
	ADD	R3,R0,LSR#1	;zaokrozi na najblizji modulo
	BL	NDELI
	LDR	R0,=0xE000400C	;T0PR
	STR	R3,[R0]
	LDR	R0,=0xE0004004	;T0TCR
	MOV	R1,#0x01	;CounterEnable
	STRB	R1,[R0]
	LDR	R0,=SEKUNDA	;resetiraj sekundni stevec
	MOV	R1,#0
	STR	R1,[R0]
	LDR	PC,[SP],#4
	LTORG

;*** Razrezi RAM v bloke dolzine 1632=0x660 v LPC23xx *** (R0-3)
INIBLOK	LDR	R1,=INIBLO2	;45 blokov za LPC2387/88
	LDR	R0,=0xE002C028	;PINSEL10 preveri bit 3 (/RTCK)?
	LDR	R0,[R0]
	TST	R0,#8
	LDRNE	R1,=INIBLO3	;20 blokov za LPC2368/78
	LDR	R0,=RAMBLOK
	MOV	R2,#MAXBLOK
INIBLO1	LDR	R3,[R1],#4
	STR	R3,[R0],#4
	SUBS	R2,R2,#1
	BPL	INIBLO1
	LDR	R0,=NICBLOK	;brisi zastavico nic blokov
	MOV	R1,#-1
	STRB	R1,[R0]
	MOV	PC,LR
	LTORG
INIBLO2	DCD	0x40002000,0x40002660,0x40002CC0,0x40003320	;4 - LPC2387/88
	DCD	0x40003980,0x40003FE0,0x40004640,0x40004CA0	;8
	DCD	0x40005300,0x40005960,0x40005FC0,0x40006620	;12
	DCD	0x40006C80,0x400072E0,0x40007940,0x7FD00000	;16
	DCD	0x7FD00660,0x7FD00CC0,0x7FD01320,0x7FD01980	;20
	DCD	0x40008000,0x40008660,0x40008CC0,0x40009320	;24
	DCD	0x40009980,0x40009FE0,0x4000A640,0x4000ACA0	;28
	DCD	0x4000B300,0x4000B960,0x4000BFC0,0x4000C620	;32
	DCD	0x4000CC80,0x4000D2E0,0x4000D940,0x4000DFA0	;36
	DCD	0x4000E600,0x4000EC60,0x4000F2C0,0x4000F920	;40
	DCD	0x7FD02000,0x7FD02660,0x7FD02CC0,0x7FD03320	;44
	DCD	0x7FD03980,0x00000000,0x00000000,0x00000000	;48
INIBLO3	DCD	0x40002000,0x40002660,0x40002CC0,0x40003320	;4 - LPC2368/78
	DCD	0x40003980,0x40003FE0,0x40004640,0x40004CA0	;8
	DCD	0x40005300,0x40005960,0x40005FC0,0x40006620	;12
	DCD	0x40006C80,0x400072E0,0x40007940,0x7FD00000	;16
	DCD	0x7FD00660,0x7FD00CC0,0x7FD01320,0x7FD01980	;20
	DCD	0x00000000,0x00000000,0x00000000,0x00000000	;24
	DCD	0x00000000,0x00000000,0x00000000,0x00000000	;28
	DCD	0x00000000,0x00000000,0x00000000,0x00000000	;32
	DCD	0x00000000,0x00000000,0x00000000,0x00000000	;36
	DCD	0x00000000,0x00000000,0x00000000,0x00000000	;40
	DCD	0x00000000,0x00000000,0x00000000,0x00000000	;44
	DCD	0x00000000,0x00000000,0x00000000,0x00000000	;48

;*** Prestej proste bloke pomnilnika v R0 *** (R0-3)
	ALIGN	16
PROSTO	MOV	R0,#0
	LDR	R1,=RAMBLOK
	MOV	R2,#MAXBLOK
PROSTO1	LDR	R3,[R1],#4
	MOVS	R3,R3
	ADDNE	R0,R0,#1
	SUBS	R2,R2,#1
	BPL	PROSTO1
	MOV	PC,LR
	LTORG

;*** Dodeli blok iz RAMBLOK v R8 v LPC23xx *** (R0,R8)
	MACRO			;makro za iskanje bloka
	MDODELI
	SWP	R8,R8,[R0]	;atomarni ukaz
	MOVS	R8,R8
	MOVNE	PC,LR
	ADD	R0,R0,#4
	MEND
	ALIGN	16
DODELI	MOV	R8,#0
	LDR	R0,=NICBLOK	;zastavica nic blokov?
	LDRB	R0,[R0]
	MOVS	R0,R0
	MOVEQ	PC,LR
	LDR	R0,=RAMBLOK	;isci blok
	MDODELI	;blok 1
	MDODELI	;blok 2
	MDODELI	;blok 3
	MDODELI	;blok 4
	MDODELI	;blok 5
	MDODELI	;blok 6
	MDODELI	;blok 7
	MDODELI	;blok 8
	MDODELI	;blok 9
	MDODELI	;blok 10
	MDODELI	;blok 11
	MDODELI	;blok 12
	MDODELI	;blok 13
	MDODELI	;blok 14
	MDODELI	;blok 15
	MDODELI	;blok 16
	MDODELI	;blok 17
	MDODELI	;blok 18
	MDODELI	;blok 19
	MDODELI	;blok 20
	MDODELI	;blok 21
	MDODELI	;blok 22
	MDODELI	;blok 23
	MDODELI	;blok 24
	MDODELI	;blok 25
	MDODELI	;blok 26
	MDODELI	;blok 27
	MDODELI	;blok 28
	MDODELI	;blok 29
	MDODELI	;blok 30
	MDODELI	;blok 31
	MDODELI	;blok 32
	MDODELI	;blok 33
	MDODELI	;blok 34
	MDODELI	;blok 35
	MDODELI	;blok 36
	MDODELI	;blok 37
	MDODELI	;blok 38
	MDODELI	;blok 39
	MDODELI	;blok 40
	MDODELI	;blok 41
	MDODELI	;blok 42
	MDODELI	;blok 43
	MDODELI	;blok 44
	MDODELI	;blok 45
	LDR	R0,=NICBLOK	;postavi zastavico nic blokov!
	STRB	R8,[R0]
	MOV	PC,LR
	LTORG

;*** Sprosti blok iz R8 v RAMBLOK *** (R0-1,R8)
	ALIGN	16
SPROSTI	MOVS	R8,R8		;neobstojec blok?
	MOVEQ	PC,LR
	LDR	R0,=RAMBLOK	;vstavi sprosceni blok v seznam
	MOV	R1,#MAXBLOK
SPROST1	SWP	R8,R8,[R0]	;atomarni ukaz
	MOVS	R8,R8
	BEQ	SPROST3
	ADD	R0,R0,#4
	SUBS	R1,R1,#1
	BPL	SPROST1
SPROST3	LDR	R0,=NICBLOK	;brisi zastavico nic blokov
	MOV	R1,#-1
	STRB	R1,[R0]
	MOV	PC,LR
	LTORG

;*** Inicializiraj vse vmesnike v SATNC *** (R0-1)
INIVMES	MOV	R1,#0
	LDR	R0,=KTXVMES
	STR	R1,[R0]
	LDR	R0,=HTPVMES
	STR	R1,[R0]
	LDR	R0,=HTXVMES
	STR	R1,[R0]
	LDR	R0,=HTZVMES
	STR	R1,[R0]
	LDR	R0,=SRXVMES
	STR	R1,[R0]
	LDR	R0,=STXVMES
	STR	R1,[R0]
	LDR	R0,=ETXVMES
	STR	R1,[R0]
	MOV	PC,LR
	LTORG

;*** Blok R8 v vmesnik R0 brez nadzora *** (R0-1,R8)
	ALIGN	16
VVMES	MOVS	R8,R8
	MOVEQ	PC,LR
	LDRH	R1,[R0]
	ADD	R1,R1,#1
	STR	R8,[R0,R1,LSL#2]
	CMP	R1,#MAXVMES
	MOVCS	R1,#0
	STRH	R1,[R0]
	MOV	PC,LR
	LTORG

;*** Blok R8 iz vmesnika R0 *** (R0-2,R8)
	ALIGN	16
IZVMES	MOV	R8,#0
	LDRH	R1,[R0]
	LDRH	R2,[R0,#2]
	CMP	R1,R2
	MOVEQ	PC,LR
	ADD	R2,R2,#1
	LDR	R8,[R0,R2,LSL#2]
	CMP	R2,#MAXVMES
	MOVCS	R2,#0
	STRH	R2,[R0,#2]
	MOV	PC,LR
	LTORG

;*** Polinomski izvor (2^32-1) v R0 *** (R0-2)
	ALIGN	16
PSEVNAK	LDR	R1,=POLINOM
	LDR	R0,[R1]
	MOVS	R0,R0		;izloci stanje samih nicel?
	MOVEQ	R0,#-1
	LDR	R2,=0x80000062	;polinom 1+X^25+X^26+X^30+X^32
	MOVS	R0,R0,LSR#1
	EORCS	R0,R0,R2
	STR	R0,[R1]
	LDR	R1,=MNASLOV	;EXOR z MNASLOV
	LDR	R2,[R1]
	EOR	R0,R0,R2
	MVNS	R0,R0		;izloci same enice
	BEQ	PSEVNAK
	MVNS	R0,R0		;izloci same nicle
	BEQ	PSEVNAK
	MOV	PC,LR
	LTORG

;*** Inicializiraj UART0 v LPC23xx *** (R0-3)
U0BAUD	EQU	9600		;baudrate UART0
UART0	STR	LR,[SP,#-4]!
	LDR	R0,=0xE000C00C	;U0LCR=0x83 DLAB=1,no-parity,1stop,8bit
	MOV	R1,#0x83
	STRB	R1,[R0]
	LDR	R0,=U0BAUD
	LDR	R3,=ARMTAKT	;izracunaj modulo iz PCLK/16
	LDR	R3,[R3]
	MOV	R3,R3,LSR#4
	ADD	R3,R0,LSR#1	;zaokrozi na najblizji modulo
	BL	NDELI
	LDR	R0,=0xE000C000	;U0DLL
	STRB	R3,[R0]
	LDR	R0,=0xE000C004	;U0DLM
	MOV	R3,R3,LSR#8
	STRB	R3,[R0]
	LDR	R0,=0xE000C008	;U0FCR=0xC1 RXtrig=14bit,FIFOenable
	MOV	R1,#0xC1
	STRB	R1,[R0]
	LDR	R0,=0xE000C00C	;U0LCR=0x03 DLAB=0,no-parity,1stop,8bit
	MOV	R1,#0x03
	STRB	R1,[R0]
	LDR	R0,=TIPVMES	;resetiraj tipke
	MOV	R1,#0
	STRH	R1,[R0]
	LDR	R0,=ZASKAZ	;resetiraj zaslon
	MOV	R1,#0
	STR	R1,[R0]
	LDR	R0,=ZASTRI
	MOV	R1,#0
	STR	R1,[R0]
	LDR	R0,=ZNACIN	;ukazni nacin
	MOV	R1,#0
	STRB	R1,[R0]
	LDR	R0,=NODZIV	;izkljuci odziv
	MOV	R1,#0
	STRH	R1,[R0]
	LDR	PC,[SP],#4
	LTORG

;*** Zaslon na UART0 v LPC23xx *** (R0-3)
	ALIGN	16
ZASLON	LDR	R0,=0xE000C014	;U0LSR bit5=TxHoldingRegisterEmpty?
	LDRB	R0,[R0]
	TST	R0,#0x20
	MOVEQ	PC,LR
	LDR	R0,=ZASKAZ	;4 znaki v vmesniku BATRAM?
	LDRH	R1,[R0]
	LDRH	R2,[R0,#2]
	CMP	R1,R2
	BEQ	ZASLON1
	ADD	R2,R2,#4	;povecaj kazalec
	CMP	R2,#MAXZAS
	MOVCS	R2,#0
	STRH	R2,[R0,#2]
	LDR	R0,=BATRAM	;4 znaki (32bit) iz BATRAM
	LDR	R1,[R0,R2]
	LDR	R0,=0xE000C000	;U0THR
	MOV	R1,R1,ROR#24	;izpisi 1.znak
	STRB	R1,[R0]
	MOV	R1,R1,ROR#24	;izpisi 2.znak
	STRB	R1,[R0]
	MOV	R1,R1,ROR#24	;izpisi 3.znak
	STRB	R1,[R0]
	MOV	R1,R1,ROR#24	;izpisi 4.znak
	STRB	R1,[R0]
	MOV	PC,LR
ZASLON1	LDR	R0,=ZASTRI	;1-3 znaki v ZASTRI?
	LDR	R1,[R0]
	LDR	R0,=0xE000C000	;U0THR
	MOV	R2,R1,LSR#16	;izpisi 1.znak?
	TST	R2,#255
	STRBNE	R2,[R0]
	MOV	R2,R1,LSR#8	;izpisi 2.znak?
	TST	R2,#255
	STRBNE	R2,[R0]
	TST	R1,#255		;izpisi 3.znak?
	STRBNE	R1,[R0]
	LDR	R0,=ZASTRI	;brisi ZASTRI
	MOV	R1,#0
	STR	R1,[R0]
	MOV	PC,LR
	LTORG

;*** Znak R0 na zaslon/v (R7)+ brez nadzora *** (R0-2,R7,brezSP!!!)
	ALIGN	16
ZNAK	LDR	R1,=ZNACIN
	LDRB	R1,[R1]
	TST	R1,#8
	BEQ	ZNAK1
	STRB	R0,[R7],#1	;znak v (R7)+
	MOV	PC,LR
ZNAK1	LDR	R1,=ZASTRI	;dodaj znak v ZASTRI
	LDR	R2,[R1]
	ORR	R0,R0,R2,LSL#8
	STR	R0,[R1]
	TST	R0,#0xFF000000	;ZASTRI vsebuje 4 znake?
	MOVEQ	PC,LR
	LDR	R1,=ZASKAZ	;povecaj kazalec
	LDRH	R2,[R1]
	ADD	R2,R2,#4
	CMP	R2,#MAXZAS
	MOVCS	R2,#0
	STRH	R2,[R1]
	LDR	R1,=BATRAM	;4 znaki (32bit) v BATRAM
	STR	R0,[R1,R2]
	LDR	R0,=ZASTRI	;brisi ZASTRI
	MOV	R1,#0
	STR	R1,[R0]
	MOV	PC,LR
	LTORG

;*** Besedilo (R3) zakljuceno z 0 na zaslon *** (R0-3)
TXTIZP	STR	LR,[SP,#-4]!	;resi link
TXTIZP1	LDRB	R0,[R3],#1
	MOVS	R0,R0
	LDREQ	PC,[SP],#4
	BL	ZNAK
	B	TXTIZP1

;*** Desetiski izpis R3 na zaslon *** (R0-4)
DECIZP	STR	LR,[SP,#-4]!	;resi link
	MOV	R4,#-1
DECIZP1	MOV	R0,R3		;ostanek v R0
	MOV	R1,#0xA0000000
	MOV	R2,#0x10000000
	MOV	R3,#0		;kvocient v R3
DECIZP2	CMP	R0,R1
	SUBCS	R0,R0,R1
	ADDCS	R3,R3,R2
	MOV	R1,R1,LSR#1
	MOVS	R2,R2,LSR#1
	BNE	DECIZP2
	STRB	R0,[SP,#-1]!	;stevilke v sklad
	ADD	R4,R4,#1
	MOVS	R3,R3
	BNE	DECIZP1
DECIZP3	LDRB	R0,[SP],#1	;stevilke iz sklada
	ADD	R0,R0,#48
	BL	ZNAK		;brez SP!!!
	SUBS	R4,R4,#1
	BPL	DECIZP3
	LDR	PC,[SP],#4

;*** Nepredznaceno deljenje R3/R0=R3, ostanek v R0 *** (R0-3)
NDELI	MOVS	R1,R0		;prepreci deljenje z 0?
	MOVEQ	PC,LR
	MOV	R2,#1
	BMI	NDELI2
NDELI1	MOV	R2,R2,LSL#1	;enica delitelja v MSB
	MOVS	R1,R1,LSL#1
	BPL	NDELI1
NDELI2	MOV	R0,R3		;ostanek v R0
	MOV	R3,#0		;kvocient v R3
NDELI3	CMP	R0,R1
	SUBCS	R0,R0,R1
	ADDCS	R3,R3,R2
	MOV	R1,R1,LSR#1
	MOVS	R2,R2,LSR#1
	BNE	NDELI3
	MOV	PC,LR

;*** Desetiski izpis R3 na eno decimalko *** (R0-4)
DEC1IZP	STR	LR,[SP,#-4]!	;resi link
	MOV	R0,#10
	BL	NDELI
	STR	R0,[SP,#-4]!
	BL	DECIZP		;celi del
	MOV	R0,#"."
	BL	ZNAK
	LDR	R3,[SP],#4	;decimalka
	BL	DECIZP
	LDR	PC,[SP],#4

;*** Naslov36 izpis R3 na zaslon *** (R0-4)
N36IZP	LDR	R0,=ZNACIN	;vkljucen HEX?
	LDRB	R0,[R0]
	TST	R0,#32
	BNE	HEXIZP
	STR	LR,[SP,#-4]!	;resi link
	MVNS	R0,R3		;naslov VSEM=0xFFFFFFFF?
	BEQ	N36IZP2
N36IZP1	MOV	R0,#36		;R3/36=R0, ostanek R0
	BL	NDELI
	ADD	R0,#48		;ostanek v znak(36)
	CMP	R0,#58
	ADDCS	R0,#7
	BL	ZNAK
	MOVS	R3,R3		;konec izpisa?
	BNE	N36IZP1
	LDR	PC,[SP],#4
N36IZP2	MOV	R0,#"*"		;izpisi naslov VSEM="*"
	BL	ZNAK
	LDR	PC,[SP],#4
	LTORG

;*** Hexadecimalni izpis R3 na zaslon *** (R0-4)
HEXIZP	STR	LR,[SP,#-4]!	;resi link
	MOV	R4,#7
HEXIZP1	MOV	R3,R3,ROR#28
	AND	R0,R3,#15	;izracunaj hexacecimalni znak
	CMP	R0,#10
	ADDCS	R0,R0,#7
	ADD	R0,R0,#48
	BL	ZNAK
	SUBS	R4,R4,#1
	BPL	HEXIZP1
	LDR	PC,[SP],#4

;*** Izpis naslova IP4 iz [R5],#4 na zaslon *** (R0-5)
IP4IZP	STR	LR,[SP,#-4]!
	LDRB	R3,[R5],#1
	BL	DECIZP
	MOV	R0,#"."
	BL	ZNAK
	LDRB	R3,[R5],#1
	BL	DECIZP
	MOV	R0,#"."
	BL	ZNAK
	LDRB	R3,[R5],#1
	BL	DECIZP
	MOV	R0,#"."
	BL	ZNAK
	LDRB	R3,[R5],#1
	BL	DECIZP
	LDR	PC,[SP],#4

;*** Izpisi MAC iz [R5],#6 *** (R0-5)
MACIZP	STR	LR,[SP,#-4]!
	MOV	R4,#6
MACIZP1	LDRB	R3,[R5],#1
	MOV	R0,R3,LSR#4	;prvi HEX znak
	ADD	R0,R0,#48
	CMP	R0,#58
	ADDCS	R0,R0,#7
	BL	ZNAK
	AND	R0,R3,#15	;drugi HEX znak
	ADD	R0,R0,#48
	CMP	R0,#58
	ADDCS	R0,R0,#7
	BL	ZNAK
	SUBS	R4,R4,#1	;konec zanke?
	LDREQ	PC,[SP],#4
	MOV	R0,#"-"		;minus med pari HEX
	BL	ZNAK
	B	MACIZP1

;*** Izpis vsebine okvirja R8 *** (R0-8)
	ALIGN	16
LIZPOKV	LDR	R0,=ZNACIN	;prikazi vse lokalno sprejete okvirje?
	LDRB	R0,[R0]
	TST	R0,#2
	MOVEQ	PC,LR
	MOV	R7,#"L"
	B	IZPOKV
	LTORG
	ALIGN	16
RIZPOKV	LDR	R0,=ZNACIN	;prikazi vse sprejete okvirje?
	LDRB	R0,[R0]
	TST	R0,#2
	MOVEQ	PC,LR
	MOV	R7,#"R"
	B	IZPOKV
	LTORG
	ALIGN	16
SIZPOKV	LDR	R0,=ZNACIN	;prikazi vse lokalno oddane okvirje?
	LDRB	R0,[R0]
	TST	R0,#2
	MOVEQ	PC,LR
	MOV	R7,#"S"
	B	IZPOKV
	LTORG
	ALIGN	16
TIZPOKV	LDR	R0,=ZNACIN	;prikazi vse oddane okvirje?
	LDRB	R0,[R0]
	TST	R0,#2
	MOVEQ	PC,LR
	MOV	R7,#"T"
	B	IZPOKV
	LTORG
	ALIGN	16
UIZPOKV	LDR	R0,=ZNACIN	;prikazi uporabne okvirje?
	LDRB	R0,[R0]
	TST	R0,#2
	MOVNE	PC,LR
	MOV	R7,#0
	B	IZPOKV
	LTORG
IZPOKV	STR	LR,[SP,#-4]!
	BL	UBRISI		;brisi natipkan ukaz
	MOVS	R0,R7		;izpisi R,T ali nicesar
	BLNE	ZNAK
	LDRH	R5,[R8]		;okvir: stevec R5, kazalec R6
	LDRH	R6,[R8,#2]
	ADD	R6,R6,R8
	LDR	R3,[R6],#4	;stevilka okvirja
	SUB	R5,R5,#4
	MOVS	R3,R3		;potrdilni okvir?
	BEQ	IZPOKV9
	MOVS	R7,R7		;zastavica za izpis strevilke?
	BEQ	IZPOKV1
	MOV	R0,#"("
	BL	ZNAK
	BL	HEXIZP
	MOV	R0,#")"
	BL	ZNAK
IZPOKV1	MOV	R7,#1		;izpisi naslove do dveh locil (0)
	MOV	R1,#0
IZPOKV2	LDR	R3,[R6],#4
	SUBS	R5,R5,#4
	BMI	IZPOKV6
	MOVS	R3,R3
	BEQ	IZPOKV3
	MOV	R0,#","		;vejica samo med veljavnimi naslovi
	MOVS	R1,R1
	BLNE	ZNAK
	BL	N36IZP
	MOV	R1,#-1
	B	IZPOKV2
IZPOKV3	MOV	R0,#">"		;znak za locilo ">"
	BL	ZNAK
	MOV	R1,#0
	SUBS	R7,R7,#1
	BPL	IZPOKV2
	BL	MERI		;meritev odziva?
	MOV	R7,#0		;izpisi besedilo okvirja
	B	IZPOKV5
IZPOKV4	LDRB	R0,[R6],#1
	CMP	R0,#13		;nova vrstica <CR>+<LF>?
	BNE	IZPOKV8
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	MOV	R7,#-1
	B	IZPOKV5
IZPOKV8	CMP	R0,#32		;izpisi natisljiv znak?
	BCC	IZPOKV5
	CMP	R0,#127
	BCS	IZPOKV5
	BL	ZNAK
	MOV	R7,#0
IZPOKV5	SUBS	R5,R5,#1
	BPL	IZPOKV4
IZPOKV6	MOVS	R7,R7		;nova vrstica na koncu?
	BMI	IZPOKV7
	MOV	R0,#13
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
IZPOKV7	BL	UPPISI		;ponovno izpisi natipkan ukaz
	LDR	PC,[SP],#4
IZPOKV9	MOV	R0,#"#"		;izpisi stevilk potrdilnega okvirja
	BL	ZNAK
	MOV	R7,#0
	SUBS	R5,R5,#4
	BMI	IZPOKV6		;nova vrstica na koncu?
	LDR	R3,[R6],#4
	BL	HEXIZP
	B	IZPOKV9
	LTORG

;*** Brisi natipkan ukaz *** (R0-3)
	ALIGN	16
UBRISI	STR	LR,[SP,#-4]!
	LDR	R0,=TIPVMES
	LDRH	R3,[R0]
UBRISI1	SUBS	R3,R3,#1
	LDRMI	PC,[SP],#4
	MOV	R0,#8
	BL	ZNAK
	MOV	R0,#32
	BL	ZNAK
	MOV	R0,#8
	BL	ZNAK
	B	UBRISI1
	LTORG

;*** Ponovno izpisi natipkan ukaz *** (R0-4)
	ALIGN	16
UPPISI	STR	LR,[SP,#-4]!
	LDR	R0,=TIPVMES
	LDRH	R3,[R0]
	ADD	R4,R0,#2
UPPISI1	SUBS	R3,R3,#1
	LDRMI	PC,[SP],#4
	LDRB	R0,[R4],#1
	BL	ZNAK
	B	UPPISI1
	LTORG

;*** Inicializiraj tabelo dvojnikov *** (R0-2)
INIDVOJ	LDR	R0,=DVOJTAB
	MOV	R1,#-1
	MOV	R2,#1024
INIDVO1	STR	R1,[R0],#4
	SUBS	R2,R2,#1
	BNE	INIDVO1
	MOV	PC,LR
	LTORG

;*** Oznaci okvir R8 dvojnik z zastavico R0 *** (R0-3,R8)
	MACRO
	DVOJNI
	LDR	R3,[R2],#-4
	CMP	R3,R1
	BEQ	DVOJNI3
	ORR	R2,R2,#0x1000
	MEND
	ALIGN	16
DVOJNIK	LDRH	R0,[R8,#2]	;preveri stevilko VSEM=0xFFFFFFFF v R1?
	LDR	R1,[R8,R0]
	MVNS	R0,R1		;R0=0 za naslov VSEM?
	MOVEQ	PC,LR
	LDR	R0,=DVOJKAZ	;naslov tabele + preverjeni kazalec v R2
	LDRH	R2,[R0]
	LDR	R0,=0x0FFC
	AND	R2,R2,R0
	LDR	R0,=DVOJTAB
	ADD	R2,R2,R0
	MOV	R0,#32
DVOJNI1	;;;;;;;;;;;;;;;;;;;;;;;;;zanka iskanja 32macro*32R0=1024
	DVOJNI	;1
	DVOJNI	;2
	DVOJNI	;3
	DVOJNI	;4
	DVOJNI	;5
	DVOJNI	;6
	DVOJNI	;7
	DVOJNI	;8
	DVOJNI	;9
	DVOJNI	;10
	DVOJNI	;11
	DVOJNI	;12
	DVOJNI	;13
	DVOJNI	;14
	DVOJNI	;15
	DVOJNI	;16
	DVOJNI	;17
	DVOJNI	;18
	DVOJNI	;19
	DVOJNI	;20
	DVOJNI	;21
	DVOJNI	;22
	DVOJNI	;23
	DVOJNI	;24
	DVOJNI	;25
	DVOJNI	;26
	DVOJNI	;27
	DVOJNI	;28
	DVOJNI	;29
	DVOJNI	;30
	DVOJNI	;31
	DVOJNI	;32
	SUBS	R0,R0,#1	;R0=0 za neuspesno iskanje
	BNE	DVOJNI1
DVOJNI2	LDR	R3,=DVOJKAZ	;povecaj in shrani kazalec
	LDRH	R2,[R3]
	ADD	R2,R2,#4
	STRH	R2,[R3]
	LDR	R3,=0x0FFC	;preveri kazalec
	AND	R2,R2,R3
	LDR	R3,=DVOJTAB	;nova stevilka R1 v tabelo
	STR	R1,[R3,R2]
	MOV	PC,LR
DVOJNI3	MOV	R0,#-1		;zastavica R0=-1 za dvojnik
	B	DVOJNI2
	LTORG

;*** Obdelava okvirja (R8) iz RMII sprejemnika *** (R0-9,SP+4)+OBDELA,PRESKOK,KISSRX
	ALIGN	16
HOBDELA	LDR	R0,=SMETER	;jakost signala v R3
	LDR	R3,[R0]
	BL	SVNOS		;vnos slisane postaje iz (R8), jakost iz R3
	LDR	R0,=MNASLOV	;brez lastnega naslova >>> KISS sprejemnik?
	LDR	R0,[R0]
	MOVS	R0,R0
	BEQ	KISSRX
	LDRH	R1,[R8,#2]	;znacka nicla za potrdilni okvir?
	LDR	R1,[R8,R1]
	MOVS	R1,R1
	BEQ	POTRDI
	LDRH	R1,[R8,#2]	;izlusci naslov prejemnika v R1
	ADD	R1,R1,#4
	LDR	R1,[R8,R1]
	CMP	R0,R1		;prejemnik je moj naslov?
	BEQ	HOBDEL1
	MVNS	R0,R1		;naslov VSEM=0xFFFFFFFF?
	BNE	PRESKOK		;preskok vozlja v ASVju?
HOBDEL1	LDRH	R0,[R8]		;sprosti prekratek okvir?
	CMP	R0,#20
	BCC	HOBDEL3
	BL	RIZPOKV		;prikazi vse podatke?
	MOV	R3,R8		;stevilka okvirja v potrditev
	LDRH	R0,[R3,#2]	
	LDR	R8,[R3,R0]
	MVNS	R1,R8		;veljavna stevilka?
	BEQ	HOBDEL2
	LDR	R0,=HTPVMES
	BL	VVMES
	LDR	R0,=MPONOVI	;brisi minimum ponavljanja
	MOV	R1,#0
	STRB	R1,[R0]
HOBDEL2	MOV	R8,R3		;preveri in sprosti dvojnik?
	BL	DVOJNIK
	MOVS	R0,R0
	BEQ	OBDELA		;obdelava sprejetih okvirjev
HOBDEL3	BL	SPROSTI		;sprosti prekratek okvir, dvojnik
	LDR	PC,[SP],#4
	LTORG

;*** Vnos slisane postaje iz (R8), jakost iz R3 *** (R0-8)
	ALIGN	16
SVNOS	LDRH	R0,[R8,#2]	;zacetek okvirja v R0
	ADD	R0,R0,R8
	LDR	R1,[R0],#4	;izloci potrdilni okvir ali neveljaven #okvirja?
	MOVS	R1,R1
	MOVEQ	PC,LR
	LDR	R1,[R0],#4	;izloci prejemnika, neveljaven>>>izhod?
	MOVS	R1,R1
	MOVEQ	PC,LR
	LDRH	R5,[R8]		;stevilo bajtov v R5
	SUB	R1,R5,#12	;-#okvirja,-prejemnik,-tiptovora
SVNOS1	SUBS	R1,R1,#4	;konec vsebine okvirja?
	MOVMI	PC,LR
	LDR	R2,[R0],#4	;nasel prvo locilo?
	MOVS	R2,R2		
	BNE	SVNOS1
	SUB	R1,R1,#4	;najdeni vir v R7
	LDR	R7,[R0],#4
SVNOS2	SUBS	R1,R1,#4	;konec vsebine okvirja?
	MOVMI	PC,LR
	LDR	R2,[R0],#4	;nasel drugo locilo?
	MOVS	R2,R2
	BNE	SVNOS2
	MOV	R2,R7		;najdeni vir dodatno v R2
	MOV	R4,#1		;dodaj en okvir v R4
	LDR	R0,=SEKUNDA	;cas okvirja v R6
	LDR	R6,[R0]
	LDR	R0,=SLISAL	;seznam slisanih postaj
	MOV	R1,#MAXSLIS
SVNOS3	SUBS	R1,R1,#1	;konec seznama?
	MOVMI	PC,LR
	MOVS	R2,R2		;neveljaven naslov?
	MOVEQ	PC,LR
	SWP	R2,R2,[R0]	;zamenjaj naslov v seznamu
	ADD	R0,R0,#4
	SWP	R3,R3,[R0]	;zamenjaj jakost v seznamu
	ADD	R0,R0,#4
	SWP	R4,R4,[R0]	;zamenjaj stevilo okvirjev v seznamu
	ADD	R0,R0,#4
	SWP	R5,R5,[R0]	;zamenjaj stevilo bajtov v seznamu
	ADD	R0,R0,#4
	SWP	R6,R6,[R0]	;zamenjaj cas v seznamu
	ADD	R0,R0,#4
	CMP	R2,R7		;enaka naslova vira?
	BNE	SVNOS3
	LDR	R0,=SLISAL
	LDR	R1,[R0,#8]	;pristej stevilo okvirjev
	ADD	R1,R1,R4
	STR	R1,[R0,#8]
	LDR	R1,[R0,#12]	;pristej stevilo bajtov
	ADD	R1,R1,R5
	STR	R1,[R0,#12]
	MOV	PC,LR
	LTORG

;*** Obdelava sprejetih okvirjev *** (R0-8,SP+4)
DATAMIN	EQU	8	;min dolzina podatkov za SLIP okvir
	ALIGN	16
OBDELA	LDRH	R0,[R8,#2]	;izlusci naslednji naslov, prvo locilo?
	ADD	R0,R0,R8
	ADD	R0,R0,#8
	LDR	R1,[R0],#4
	MOVS	R1,R1
	BNE	USMERI		;usmeri okvir R8 po dodatnih naslovih
	LDRH	R1,[R8]		;poisci zacetek vsebine (drugo locilo)
	SUBS	R1,R1,#12
OBDELA1	SUBS	R1,R1,#4
	BMI	OBDELA3		;sprosti okvir brez dveh locil
	LDR	R2,[R0],#4
	MOVS	R2,R2
	BNE	OBDELA1
	LDR	R2,[R0]		;nicla za besedilo?
	MOVS	R2,R2
	BEQ	OBDELA2
	BL	ANAJDI		;najdi vpis v tabelo ARP
	SUB	R0,R0,R8	;zacetek vsebine
	STRH	R0,[R8,#2]
	STRH	R1,[R8]		;dolzina vsebine, sprosti prekratko?
	CMP	R1,#DATAMIN
	BCC	OBDELA3
	LDR	R2,=LNASLOV	;lokalno vkljuceno?
	LDR	R2,[R2]
	MOVS	R2,R2
	LDR	R0,=STXVMES	;v SLIP oddajni vmesnik
	LDRNE	R0,=ETXVMES	;v ETH oddajni vmesnik
	BL	VVMES
	LDR	PC,[SP],#4
OBDELA2	BL	UIZPOKV		;prikazi uporabne okvirje
	B	JAVISE		;javi se na ukaz "////"
OBDELA3	BL	SPROSTI		;brez dveh locil, lokalno, prekratka vsebina
	LDR	PC,[SP],#4
	LTORG

;*** Preskok vozlja v ASVju *** (R0-9,SP+4)
	ALIGN	16
PRESKOK	LDR	R2,=LNASLOV	;primerjava R1 z lokalnimi naslovi
	MOV	R3,#MAXLOK
	MOV	R9,#0		;izracunaj oddaljenost cilja v R9
PRESKO1	MOV	R9,R9,LSL#4
	ORR	R9,R9,#10
	LDR	R0,[R2],#4	;konec vpisov v seznamu lokalnih?
	MOVS	R0,R0
	BEQ	PRESKO2
	CMP	R0,R1		;nasel lokalni naslov?
	BEQ	PRESKO3
	SUBS	R3,R3,#1	;konec seznama lokalnih?
	BNE	PRESKO1
PRESKO2	BL	SPROSTI		;ni lokalni, prekratek, dvojnik, ni preskok, brez locila
	LDR	PC,[SP],#4
PRESKO3	LDRH	R0,[R8]		;sprosti prekratek okvir?
	CMP	R0,#20
	BCC	PRESKO2
	BL	RIZPOKV		;prikazi vse podatke?
	MOV	R3,R8		;stevilka okvirja v potrditev
	LDRH	R0,[R3,#2]	
	LDR	R8,[R3,R0]
	LDR	R0,=HTPVMES
	BL	VVMES
	LDR	R0,=MPONOVI	;brisi minimum ponavljanja
	MOV	R1,#0
	STRB	R1,[R0]
	MOV	R8,R3		;preveri in sprosti dvojnik?
	BL	DVOJNIK
	MOVS	R0,R0
	BNE	PRESKO2
	LDRH	R0,[R8,#2]	;zavrti naslove navzdol do locila
	ADD	R0,R0,R8
	MOV	R2,#MAXNAS
PRESKO4	LDR	R1,[R0,#4]
	STR	R1,[R0],#4
	MOVS	R1,R1		;nasel locilo?
	BEQ	PRESKO5
	SUBS	R2,R2,#1	;sprosti brez locila?
	BMI	PRESKO2
	B	PRESKO4
	LTORG
PRESKO5	LDR	R1,=MNASLOV	;dodaj moj naslov za locilo
	LDR	R1,[R1]
	STR	R1,[R0]
	LDRH	R0,[R8,#2]	;preskok brez nicle na zacetku naslovov?
	ADD	R0,R0,R8
	LDR	R1,[R0,#4]
	MOVS	R1,R1
	BEQ	PRESKO6
	LDRH	R1,[R8,#2]	;oddaljenost cilja iz R9 v okvir
	STR	R9,[R8,R1]
	LDR	R0,=STXVMES	;okvir v SLIP oddajni vmesnik
	BL	VVMES
	LDR	PC,[SP],#4
PRESKO6	MOV	R1,#-1		;domet do cilja v zanki
	STR	R1,[R0,#-4]
	LDRH	R0,[R8]		;dolzina okvirja +4
	ADD	R0,R0,#4
	STRH	R0,[R8]
	LDRH	R0,[R8,#2]	;zacetek okvirja -4
	SUB	R0,R0,#4
	STRH	R0,[R8,#2]
	LDR	R0,=STXVMES	;okvir v SLIP oddajni vmesnik
	BL	VVMES
	LDR	PC,[SP],#4
	LTORG

;*** KISS sprejemnik v SATNC *** (R0-2,R8,SP+4)
	ALIGN	16
KISSRX	LDR	R0,=LNASLOV	;KISS ali Ethernet<>SLIP?
	LDR	R0,[R0]
	MOVS	R0,R0
	BEQ	KISSRX1
	BL	SPROSTI		;sprosti za Ethernet<>SLIP pretvornik
	LDR	PC,[SP],#4
KISSRX1	BL	RIZPOKV		;prikazi vse okvirje?
	LDRH	R0,[R8]		;podaljsaj vsebino za 1 byte
	ADD	R0,R0,#1
	STRH	R0,[R8]
	LDRH	R1,[R8,#2]	;vsebina 1byte navzgor, zacetek #0 
	ADD	R1,R1,R8
	MOV	R2,#0
KISSRX2	SWPB	R2,R2,[R1]
	ADD	R1,R1,#1
	SUBS	R0,R0,#1
	BNE	KISSRX2
	LDR	R0,=STXVMES	;okvir v SLIP oddajni vmesnik
	BL	VVMES
	LDR	PC,[SP],#4
	LTORG

;*** Javi se na ukaz "////" *** (R0-8,SP+4)+ODDAJA
JAVISE	LDRH	R0,[R8,#2]	;poisci ukaz (tri locila)
	ADD	R0,R0,R8
	LDRH	R4,[R8]
	MOV	R2,#2
JAVISE1	LDR	R3,[R0],#4
	SUBS	R4,R4,#4
	BMI	JAVISE9
	MOVS	R3,R3
	SUBEQ	R2,R2,#1
	MOVS	R2,R2
	BPL	JAVISE1
	LDR	R2,=0x2F2F2F2F	;ukaz "////"?
	LDR	R3,[R0]
	SUBS	R4,R4,#4
	BMI	JAVISE9
	CMP	R3,R2
	BNE	JAVISE9
	LDR	R0,=JAVITXT	;javljanje vklopljeno?
	LDRB	R0,[R0]
	CMP	R0,#32
	BCC	JAVISE9
	LDRH	R1,[R8,#2]	;preskoci stevilko okvirja
	ADD	R1,R1,R8
	ADD	R1,R1,#4
	ADD	R2,R1,#8	;naslovi dve mesti navzdol
	MOV	R3,#MAXNAS
JAVISE2	LDR	R0,[R2],#4
	MOVS	R0,R0
	BEQ	JAVISE3
	STR	R0,[R1],#4
	SUBS	R3,R3,#1
	BPL	JAVISE2
JAVISE3	MOV	R0,#0		;prvo locilo znakov
	STR	R0,[R1],#4
	LDR	R0,=MNASLOV	;vstavi moj naslov
	LDR	R0,[R0]
	STR	R0,[R1],#4
	MOV	R0,#0		;drugo locilo znakov
	STR	R0,[R1],#4
	MOV	R0,#0		;oznaka besedila
	STR	R0,[R1],#4
	LDR	R0,=0x2A2A2A2A	;vstavi "****" za preprecitev zankanja
	STR	R0,[R1],#4
	LDRB	R0,[R1]
	CMP	R0,#"!"		;javljanje NBP-ARP?
	BEQ	JNBPARP
	CMP	R0,#"#"		;javljanje PHY+ARP?
	BEQ	JPHYSTS
	CMP	R0,#"*"		;javljanje nastavitev?
	BEQ	JPARAM
	CMP	R0,#"+"		;javljanje slisano?
	BEQ	JSLISI
	CMP	R0,#"-"		;telekomanda in javljanje ADC?
	BEQ	KJADC
	CMP	R0,#"="		;vstavi/javi NBP pot?
	BEQ	KJNBP
	CMP	R0,#"/"		;javljanje stanja?
	BEQ	JSTANJE
	CMP	R0,#">"		;merilni odziv?
	BEQ	JODZIV
	CMP	R0,#"?"		;javljanje besedila?
	BEQ	JBESED
	LDR	R2,=JAVISE6	;vstavi neznano javljanje
JAVISE4	LDRB	R0,[R2],#1
	CMP	R0,#32
	BCC	JAVISE5
	STRB	R0,[R1],#1
	B	JAVISE4
JAVISE5	LDRH	R0,[R8,#2]	;izracunaj dolzino sestavljenega okvirja
	SUB	R1,R1,R8
	SUB	R1,R1,R0
	STRH	R1,[R8]
	BL	UBRISI		;brisi natipkan ukaz
	LDR	R3,=JAVISE7	;javi neznano javljanje
	BL	TXTIZP
	BL	UPPISI		;ponovni izpisi natipkan ukaz
	B	ODDAJA		;okvir R8 v oddajo RMII/SLIP
JAVISE6 DCB	" Nepoznan daljinski ukaz (!,#,*,+,-,/,=,>,?) ****",0
JAVISE7	DCB	"!!! Javljam nepoznan ukaz !!!",13,10,0
	ALIGN	4
	LTORG
JAVISE9	BL	SPROSTI
	LDR	PC,[SP],#4

;*** Javi besedilo *** (R0-8,SP+4)+ODDAJA
JBESED	LDR	R2,=JAVITXT	;vstavi besedilo javljanja
	MOV	R3,#MAXJAVI
JBESED1	LDRB	R0,[R2],#1
	CMP	R0,#32
	BCC	JBESED2
	STRB	R0,[R1],#1
	SUBS	R3,R3,#1
	BNE	JBESED1
JBESED2	LDRH	R0,[R8,#2]	;izracunaj dolzino sestavljenega okvirja
	SUB	R1,R1,R8
	SUB	R1,R1,R0
	STRH	R1,[R8]
	BL	UBRISI		;brisi natipkan ukaz
	LDR	R3,=JBESED3	;javi javljanje besedila
	BL	TXTIZP
	BL	UPPISI		;ponovni izpisi natipkan ukaz
	B	ODDAJA		;okvir R8 v oddajo RMII/SLIP
JBESED3	DCB	"!!! Javljam besedilo !!!",13,10,0
	ALIGN	4
	LTORG

;*** Merilni odziv *** (R0-8,SP+4)+ODDAJA
JODZIV	BL	UBRISI		;brisi natipkan ukaz
	LDR	R3,=JODZIV1	;javi merilni odziv
	BL	TXTIZP
	BL	UPPISI		;ponovni izpisi natipkan ukaz
	B	ODDAJA		;okvir R8 v oddajo RMII/SLIP
JODZIV1	DCB	"!!! Merilni odziv !!!",13,10,0
	ALIGN	4
	LTORG

;*** Javi slisano *** (R0-8,SP+4)+ODDAJA
JSLISI	MOV	R7,R1		;vstavi stanje v (R7)+
	LDR	R1,=ZNACIN
	LDRB	R0,[R1]
	ORR	R0,R0,#8
	STRB	R0,[R1]
	MOV	R0,#32		;presledek
	BL	ZNAK
	BL	SSLIZP		;izpis seznama slisanih postaj
	LDR	R1,=ZNACIN	;preklop na zaslon
	LDRB	R0,[R1]
	BIC	R0,R0,#8
	STRB	R0,[R1]
	LDRH	R0,[R8,#2]	;izracunaj dolzino sestavljenega okvirja
	SUB	R1,R7,R8
	SUB	R1,R1,R0
	STRH	R1,[R8]
	BL	UBRISI		;brisi natipkan ukaz
	LDR	R3,=JSLISI1	;javi javljanje stanja
	BL	TXTIZP
	BL	UPPISI		;ponovni izpisi natipkan ukaz
	B	ODDAJA		;okvir R8 v oddajo RMII/SLIP
JSLISI1	DCB	"!!! Javljam slisano !!!",13,10,0
	ALIGN	4
	LTORG

;*** Javi stanje *** (R0-8,SP+4)+ODDAJA
JSTANJE	MOV	R7,R1		;vstavi stanje v (R7)+
	LDR	R1,=ZNACIN
	LDRB	R0,[R1]
	ORR	R0,R0,#8
	STRB	R0,[R1]
	MOV	R0,#32		;presledek
	BL	ZNAK
	BL	STANJE
	LDR	R1,=ZNACIN	;preklop na zaslon
	LDRB	R0,[R1]
	BIC	R0,R0,#8
	STRB	R0,[R1]
	LDRH	R0,[R8,#2]	;izracunaj dolzino sestavljenega okvirja
	SUB	R1,R7,R8
	SUB	R1,R1,R0
	STRH	R1,[R8]
	BL	UBRISI		;brisi natipkan ukaz
	LDR	R3,=JSTANJ1	;javi javljanje stanja
	BL	TXTIZP
	BL	UPPISI		;ponovni izpisi natipkan ukaz
	B	ODDAJA		;okvir R8 v oddajo RMII/SLIP
JSTANJ1	DCB	"!!! Javljam stanje !!!",13,10,0
	ALIGN	4
	LTORG

;*** Javi PHY+ARP *** (R0-8,SP+4)+ODDAJA
JPHYSTS	MOV	R7,R1		;vstavi stanje v (R7)+
	LDR	R1,=ZNACIN
	LDRB	R0,[R1]
	ORR	R0,R0,#8
	STRB	R0,[R1]
	MOV	R0,#32		;presledek
	BL	ZNAK
	BL	PHYMAC		;izpisi PHY status in sogovornikov MAC
	BL	ARPIZP		;izpis ETH-ARP cacheja
	LDR	R1,=ZNACIN	;preklop na zaslon
	LDRB	R0,[R1]
	BIC	R0,R0,#8
	STRB	R0,[R1]
	LDRH	R0,[R8,#2]	;izracunaj dolzino sestavljenega okvirja
	SUB	R1,R7,R8
	SUB	R1,R1,R0
	STRH	R1,[R8]
	BL	UBRISI		;brisi natipkan ukaz
	LDR	R3,=JPHYST1	;javi javljanje stanja
	BL	TXTIZP
	BL	UPPISI		;ponovni izpisi natipkan ukaz
	B	ODDAJA		;okvir R8 v oddajo HDLC/SLIP
JPHYST1	DCB	"!!! Javljam PHY+ARP !!!",13,10,0
	ALIGN	4
	LTORG

;*** Javi nastavitve *** (R0-8,SP+4)+ODDAJA
JPARAM	MOV	R7,R1		;vstavi stanje v (R7)+
	LDR	R1,=ZNACIN
	LDRB	R0,[R1]
	ORR	R0,R0,#8
	STRB	R0,[R1]
	MOV	R0,#13		;nova vrsta
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	BL	PARAM		;pregled nastavitev SATNC
	LDR	R1,=ZNACIN	;preklop na zaslon
	LDRB	R0,[R1]
	BIC	R0,R0,#8
	STRB	R0,[R1]
	LDRH	R0,[R8,#2]	;izracunaj dolzino sestavljenega okvirja
	SUB	R1,R7,R8
	SUB	R1,R1,R0
	STRH	R1,[R8]
	BL	UBRISI		;brisi natipkan ukaz
	LDR	R3,=JPARAM1	;javi javljanje nastavitev
	BL	TXTIZP
	BL	UPPISI		;ponovni izpisi natipkan ukaz
	B	ODDAJA		;okvir R8 v oddajo RMII/SLIP
JPARAM1	DCB	"!!! Javljam nastavitve !!!",13,10,0
	ALIGN	4
	LTORG

;*** Javi NBP-ARP *** (R0-8,SP+4)+ODDAJA
JNBPARP	MOV	R7,R1		;vstavi stanje v (R7)+
	LDR	R1,=ZNACIN
	LDRB	R0,[R1]
	ORR	R0,R0,#8
	STRB	R0,[R1]
	LDR	R3,=JNBPAR1	;izpis NBP nacina usmerjanja
	BL	TXTIZP
	LDR	R0,=ANACIN
	LDRB	R3,[R0]
	AND	R3,R3,#7
	BL	DECIZP
	MOV	R0,#13		;nova vrsta
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	BL	NBPIZP		;izpis NBP-ARP tabele
	LDR	R1,=ZNACIN	;preklop na zaslon
	LDRB	R0,[R1]
	BIC	R0,R0,#8
	STRB	R0,[R1]
	LDRH	R0,[R8,#2]	;izracunaj dolzino sestavljenega okvirja
	SUB	R1,R7,R8
	SUB	R1,R1,R0
	STRH	R1,[R8]
	BL	UBRISI		;brisi natipkan ukaz
	LDR	R3,=JNBPAR2	;javi javljanje NBP usmerjanje
	BL	TXTIZP
	BL	UPPISI		;ponovni izpisi natipkan ukaz
	B	ODDAJA		;okvir R8 v oddajo RMII/SLIP
JNBPAR1	DCB	" NBP usmerjanje: ",0
JNBPAR2	DCB	"!!! Javljam NBP usmerjanje !!!",13,10,0
	ALIGN	4
	LTORG

;*** Telekomanda in javi ADC v LPC23xx *** (R0-8,SP+4)+ODDAJA,RIZVEDI
KJADC	MOV	R7,R1		;vstavi stanje v (R7)+
	MOV	R2,R1		;zakljuci z 0 ukaz iz (R2)+
	MOV	R0,#0
	STRB	R0,[R2,R4]
	BL	HVHOD		;telekomanda?
	MOVS	R1,R1
	BMI	KJADC1
	MOV	R4,R0
	BL	HVHOD		;geslo?
	LDR	R1,=GESLO
	LDR	R1,[R1]
	CMP	R1,#-1		;VSEM?
	MOVEQ	R0,R1
	CMP	R0,R1
	BNE	KJADC1
	CMP	R4,R1		;RESET (ukaz enak geslu)?
	BEQ	RIZVEDI
	TST	R4,#0x00000001	;izvedi telekomando
	LDR	R0,=0x3FFFC03B	;FIO1SET3
	LDREQ	R0,=0x3FFFC03F  ;FIO1CLR3
	MOV	R1,#0x80
	STRB	R1,[R0]
	LDR	R1,=ZNACIN	;vklop/izklop TX?
	LDRB	R0,[R1]
	BIC	R0,R0,#128
	TST	R4,#0x00000002
	ORRNE	R0,R0,#128
	STRB	R0,[R1]
KJADC1	LDR	R1,=ZNACIN	;javi ADC in telekomando
	LDRB	R0,[R1]
	ORR	R0,R0,#8
	STRB	R0,[R1]
	MOV	R0,#32		;presledek
	BL	ZNAK
	BL	ZSLOT		;zakasnitev TSLOT
	BL	ADCK		;izpis ADC telemetrije in telekomande
	LDR	R1,=ZNACIN	;preklop na zaslon
	LDRB	R0,[R1]
	BIC	R0,R0,#8
	STRB	R0,[R1]
	LDRH	R0,[R8,#2]	;izracunaj dolzino sestavljenega okvirja
	SUB	R1,R7,R8
	SUB	R1,R1,R0
	STRH	R1,[R8]
	BL	UBRISI		;brisi natipkan ukaz
	LDR	R3,=KJADC2	;javi javljanje stanja
	BL	TXTIZP
	BL	UPPISI		;ponovni izpisi natipkan ukaz
	B	ODDAJA		;okvir R8 v oddajo RMII/SLIP
KJADC2	DCB	"!!! Telekomanda in javljanje ADC !!!",13,10,0
	ALIGN	4
	LTORG

;*** Vstavi/javi NBP pot *** (R0-8,SP+4)+ODDAJA,RIZVEDI
KJNBP	MOV	R7,R1		;vstavi NBP pot v (R7)+
	MOV	R2,R1		;zakljuci z 0 ukaz iz (R2)+
	MOV	R0,#0
	STRB	R0,[R2,R4]
	BL	HVHOD		;preberi geslo?
	MOVS	R1,R1
	BMI	KJNBP2
	LDR	R1,=GESLO
	LDR	R1,[R1]
	CMP	R1,#-1		;geslo VSEM?
	MOVEQ	R0,R1
	CMP	R0,R1		;geslo veljavno?
	BNE	KJNBP2
	BL	DVHOD		;vnos nastavitve A
	MOVS	R1,R1
	BMI	KJNBP2
	LDR	R3,=ANACIN
	STRB	R0,[R3]
	LDR	R6,=INASLOV	;vnos naslova
	MOV	R5,#MAXNAS
KJNBP1	SUBS	R5,R5,#1
	BMI	KJNBP2
	BL	NVHOD
	MOVS	R1,R1
	MOVMI	R0,#0
	STR	R0,[R6],#4
	BPL	KJNBP1
KJNBP2	LDR	R1,=ZNACIN	;preklop v (R7)+
	LDRB	R0,[R1]
	ORR	R0,R0,#8
	STRB	R0,[R1]
	MOV	R0,#32		;izpis nastavitve A
	BL	ZNAK
	MOV	R0,#"("
	BL	ZNAK
	LDR	R0,=ANACIN
	LDRB	R3,[R0]
	AND	R3,R3,#7
	BL	DECIZP
	MOV	R0,#")"
	BL	ZNAK
	LDR	R6,=INASLOV	;izpis NBP poti [R6]+
	BL	POTIZP
	LDR	R1,=ZNACIN	;preklop na zaslon
	LDRB	R0,[R1]
	BIC	R0,R0,#8
	STRB	R0,[R1]
	LDRH	R0,[R8,#2]	;izracunaj dolzino sestavljenega okvirja
	SUB	R1,R7,R8
	SUB	R1,R1,R0
	STRH	R1,[R8]
	BL	UBRISI		;brisi natipkan ukaz
	LDR	R3,=KJNBP3	;javi javljanje I naslova
	BL	TXTIZP
	BL	UPPISI		;ponovni izpisi natipkan ukaz
	B	ODDAJA		;okvir R8 v oddajo RMII/SLIP
KJNBP3	DCB	"!!! Vstavi/javi NBP pot !!!",13,10,0
	ALIGN	4
	LTORG

;*** Usmeri okvir R8 po dodatnih naslovih *** (R0-8,SP+4)+ODDAJA
	ALIGN	16
USMERI	LDRH	R0,[R8,#2]	;zavrti naslove navzdol
	ADD	R0,R0,R8
	ADD	R0,R0,#4
	MOV	R2,#MAXNAS
USMERI1	LDR	R1,[R0,#4]
	STR	R1,[R0],#4
	MOVS	R1,R1
	BEQ	USMERI2
	SUBS	R2,R2,#1
	BNE	USMERI1
	BL	SPROSTI		;sprosti okvir brez locila
	LDR	PC,[SP],#4
USMERI2	LDR	R1,=MNASLOV	;dodaj moj naslov na konec
	LDR	R1,[R1]
	STR	R1,[R0]
	B	ODDAJA		;okvir R8 v oddajo RMII/SLIP
	LTORG

;*** Okvir R8 v oddajo RMII/SLIP *** (R0-3,R8,SP+4)
	ALIGN	16
ODDAJA	LDRH	R0,[R8,#2]	;poisci naslov med lokalnimi?
	ADD	R0,R0,R8
	LDR	R3,[R0,#4]
	LDR	R2,=LNASLOV
	MOV	R1,#MAXLOK
ODDAJA1	LDR	R0,[R2],#4
	MVNS	R0,R0		;lokalno VSEM?
	BEQ	ODDAJA4
	MVNS	R0,R0		;konec vpisov v seznamu lokalnih?
	BEQ	ODDAJA2
	CMP	R0,R3		;nasel lokalni naslov?
	BEQ	ODDAJA4
	SUBS	R1,R1,#1	;konec seznama lokalnih?
	BNE	ODDAJA1
ODDAJA2	BL	PSEVNAK		;nova stevilka okvirja za RMII
	LDRH	R1,[R8,#2]
	STR	R0,[R8,R1]
	LDR	R0,=MPONOVI	;brisi minimum ponavljanja
	MOV	R1,#0
	STRB	R1,[R0]
	LDR	R0,=NPONOVI	;novo stevilo ponavljanj
	LDRB	R1,[R0]
	STRB	R1,[R8,#4]
	MOV	R0,#0		;zakasnitev ponavljanja 0
	STRB	R0,[R8,#5]
	LDRH	R0,[R8,#2]	;naslov VSEM=0xFFFFFFFF?
	ADD	R0,R0,#4
	LDR	R1,[R8,R0]
	MVNS	R2,R1
	BNE	ODDAJA3
	LDRH	R0,[R8,#2]	;stevilka VSEM in nic ponavljanja
	STR	R1,[R8,R0]
	MOV	R0,#0
	STRB	R0,[R8,#4]
ODDAJA3	LDR	R0,=HTXVMES	;okvir v RMII oddajni vmesnik
	BL	VVMES
	LDR	PC,[SP],#4
ODDAJA4	MOV	R0,#-1		;stevilka VSEM za SLIP
	LDRH	R1,[R8,#2]
	STR	R0,[R8,R1]
	LDR	R0,=STXVMES	;okvir v SLIP oddajni vmesnik
	BL	VVMES
	LDR	PC,[SP],#4
	LTORG

;*** Sprosti potrditev R8 in potrjene podatke *** (R0-8,SP+4)
	ALIGN	16
POTRDI	BL	RIZPOKV		;prikazi vse potrditve?
	MOV	R7,R8		;resi R8 v R7
	LDRH	R6,[R7,#2]	;kazalec R6, preskoci znacko
	ADD	R6,R6,R7
	ADD	R6,R6,#4
	LDRH	R5,[R7]		;stevec R5, odsteje znacko
	SUB	R5,R5,#5
POTRDI5	LDR	R3,[R6],#4	;stevilka potrditve v R3
	BL	POTRDI6
	SUBS	R5,R5,#4
	BPL	POTRDI5
	MOV	R8,R7
	BL	SPROSTI		;sprosti potrdilni okvir
	LDR	PC,[SP],#4
	ALIGN	16
POTRDI6	STR	LR,[SP,#-4]!	;resi link
POTRDI1	LDR	R0,=HTXVMES	;podatkovni okvirji
	BL	IZVMES
	MOVS	R8,R8
	BEQ	POTRDI3
	LDRH	R0,[R8,#2]	;stevilka okvirja
	LDR	R4,[R8,R0]
	CMP	R4,R3
	BEQ	POTRDI2
	LDR	R0,=HTZVMES	;nepotrjeni v zacasni vmesnik
	BL	VVMES
	B	POTRDI1
POTRDI2	BL	SPROSTI		;sprosti potrjeni okvir
	LDR	R1,=MPONOVI	;minimum ponavljanja MAX=255
	MOV	R0,#-1
	STRB	R0,[R1]
	B	POTRDI1
POTRDI3	LDR	R0,=HTZVMES	;iz zacasnega vmesnika
	BL	IZVMES
	MOVS	R8,R8
	BEQ	POTRDI4
	LDRB	R0,[R8,#5]	;poisci minimum ponavljanja
	LDR	R1,=MPONOVI
	LDRB	R2,[R1]
	CMP	R2,R0
	STRBCS	R0,[R1]
	LDR	R0,=HTXVMES	;v oddajni vmesnik
	BL	VVMES
	B	POTRDI3
POTRDI4	LDR	R0,=HTPVMES	;cakajoce stevilke potrditev na oddajo?
	LDRH	R1,[R0]
	LDRH	R2,[R0,#2]
	CMP	R1,R2
	LDREQ	PC,[SP],#4
	LDR	R1,=MPONOVI	;minimum ponavljanja MIN=0
	MOV	R0,#0
	STRB	R0,[R1]
	LDR	PC,[SP],#4
	LTORG

;*** Najdi vpis v tabelo ARP (R0,R1,R8) *** (R2-6)
MINIP4	EQU	21		;najmanjsa dolzina IP4 okvirja
IP4MIN	EQU	0x45		;prvi znak IP4 okvirja (MIN in MAX)
IP4MAX	EQU	0x50
IPIZVOR	EQU	12		;odmik IP4 izvora
MINAX25	EQU	16		;najmanjsa dolzina AX25 okvirja (KISS)
AXKISS	EQU	0		;zacetni znak AX25 KISS okvirja
AXIZVOR	EQU	13		;odmik AX.25 izvora
	ALIGN	16
ANAJDI	LDR	R2,=ANACIN	;vpis vklopljen (nacin 2,3,6 ali 7)?
	LDRB	R2,[R2]
	TST	R2,#2
	MOVEQ	PC,LR
	TST	R2,#4		;najdi izvor IP4 ali AX25?
	BNE	ANAJDI5
	CMP	R1,#MINIP4	;najmanjsa dolzina IP4 okvirja?
	MOVCC	PC,LR
	LDRB	R2,[R0]		;prvi znak IP4 okvirja?
	CMP	R2,#IP4MIN
	MOVCC	PC,LR
	CMP	R2,#IP4MAX
	MOVCS	PC,LR
	AND	R2,R2,#15	;preveri checksum IP4 glave?
	MOV	R2,R2,LSL#1
	MOV	R3,#0
	MOV	R4,R0
ANAJDI8	LDRH	R5,[R4],#2
	ADD	R3,R3,R5
	SUBS	R2,R2,#1
	BNE	ANAJDI8
	MOV	R4,R3,ROR#16
	ADD	R3,R3,R4
	CMP	R3,#-1
	MOVNE	PC,LR
	LDR	R2,=ZNASLOV	;prepisi IP posiljatelja
	LDR	R3,[R0,#IPIZVOR]
	STR	R3,[R2],#4
	B	ANAJDI6
ANAJDI5	CMP	R1,#MINAX25	;najmanjsa dolzina AX25 okvirja?
	MOVCC	PC,LR
	LDRB	R2,[R0]		;prvi znak AX25 KISS okvirja?
	CMP	R2,#AXKISS
	MOVNE	PC,LR
	MOV	R2,#AXIZVOR	;prepisi AX25 izvor modulo 36 v R3
	ADD	R2,R2,R0
	MOV	R3,#0
	MOV	R4,#6
ANAJDI7	LDRB	R5,[R2],#-1	;preveri nov byte sodo stevilo?
	TST	R5,#1
	MOVNE	PC,LR
	MOV	R5,R5,LSR#1	;pretvori nov byte v R5
	CMP	R5,#64
	ANDCC	R5,R5,#15	;nova stevilka
	ANDCS	R5,R5,#31	;nova crka
	ADDCS	R5,R5,#9
	ADD	R3,R3,R3,LSL#3	;pomnozi R3 s 36 in pristej R5
	ADD	R3,R5,R3,LSL#2
	SUBS	R4,R4,#1
	BNE	ANAJDI7
	LDR	R2,=ZNASLOV
	STR	R3,[R2],#4
ANAJDI6	LDRH	R3,[R8,#2]	;prepisi naslov
	ADD	R3,R3,R8
	ADD	R3,R3,#12
	MOV	R4,#MAXNAS
ANAJDI1	LDR	R5,[R3],#4
	STR	R5,[R2],#4
	MOVS	R5,R5
	BEQ	ANAJDI2
	SUBS	R4,R4,#1
	BNE	ANAJDI1
ANAJDI2	LDR	R2,=ANACIN	;nacin 3/7=VPIS?
	LDRB	R2,[R2]
	TST	R2,#1
	BNE	AVNOS		;vnos v tabelo ARP - 3/7=vedno!
	LDR	R2,=INASLOV	;primerjaj INASLOV<>ZNASLOV?
	LDR	R3,=ZNASLOV
	ADD	R3,R3,#4
	MOV	R4,#MAXNAS
ANAJDI3 LDR	R5,[R2],#4
	LDR	R6,[R3],#4
	CMP	R5,R6
	BNE	AVNOS		;vnos v tabelo ARP - 2/6=razlicno!
	MOVS	R5,R5
	BEQ	ANAJDI4
	SUBS	R4,R4,#1
	BNE	ANAJDI3
ANAJDI4	MOV	R2,#0		;brisanje privzete poti
	LDR	R3,=ZNASLOV
	STR	R2,[R3,#4]
	B	AVNOS		;brisanje iz tabele ARP - 2/6=enako!
	LTORG

;*** Vnos v tabelo ARP *** (R2-6)
	ALIGN	16
AVNOS	LDR	R5,=ZNASLOV	;poisci enak IP v tabeli ARP
	LDR	R6,=ANASLOV
	MOV	R2,#MAXARP
	LDR	R3,[R5]
AVNOS1	LDR	R4,[R6]
	CMP	R3,R4
	BEQ	AVNOS3
	SUBS	R2,R2,#1
	BEQ	AVNOS2
	ADD	R6,R6,#AKORAK
	B	AVNOS1
AVNOS2	LDR	R3,[R5,#4]	;izloci brisanje neobstojecega IP?
	MOVS	R3,R3
	MOVEQ	PC,LR
AVNOS3	LDR	R3,[R5,#4]	;brisanje obstojecega IP?
	MOVS	R3,R3
	BEQ	AVNOS7
	ADD	R2,R6,#AKORAK	;prepisi staro vsebino navzgor
	LDR	R4,=ANASLOV
AVNOS4	CMP	R4,R6
	BEQ	AVNOS5
	LDR	R3,[R6,#-4]!
	STR	R3,[R2,#-4]!
	B	AVNOS4
AVNOS5	MOV	R2,#AKORAK	;prepisi novi veljavni vnos na zacetek
AVNOS6	LDR	R3,[R5],#4
	STR	R3,[R4],#4
	SUBS	R2,R2,#4
	BNE	AVNOS6
	MOV	PC,LR
AVNOS7	SUBS	R2,R2,#1	;prepisi staro vsebino navzdol
	BEQ	AVNOS9
	ADD	R5,R6,#AKORAK
	MOV	R4,#AKORAK
AVNOS8	LDR	R3,[R5],#4
	STR	R3,[R6],#4
	SUBS	R4,R4,#4
	BNE	AVNOS8
	B	AVNOS7
AVNOS9	MOV	R2,#0		;neveljaven vnos na konec
	STR	R2,[R6,#4]
	MOV	PC,LR
	LTORG

;*** Cilj okvirja [R8] iz ARP tabele v R0 *** (R0-2,R4-5)
IPCILJ	EQU	16		;odmik IP4 cilja
AXCILJ	EQU	6		;odmik AX25 cilja
	ALIGN	16
ACILJ	LDR	R1,=ANACIN	;ARP izklopljen?
	LDRB	R1,[R1]
	TST	R1,#3
	MOVEQ	PC,LR
	TST	R1,#4
	BNE	ACILJ3
	LDRH	R1,[R8]		;najmanjsa dolzina IP4 okvirja?
	CMP	R1,#MINIP4
	MOVCC	PC,LR
	LDRH	R1,[R8,#2]	;prvi znak IP4 okvirja?
	ADD	R1,R1,R8
	LDRB	R2,[R1]
	CMP	R2,#IP4MIN
	MOVCC	PC,LR
	CMP	R2,#IP4MAX
	MOVCS	PC,LR
	LDR	R2,[R1,#IPCILJ]	;prepisi ciljni IP naslovnika v R2
	B	ACILJ4
ACILJ3	LDRH	R1,[R8]		;najmanjsa dolzina AX25 okvirja?
	CMP	R1,#MINAX25
	MOVCC	PC,LR
	LDRH	R1,[R8,#2]	;prvi znak AX25 KISS okvirja?
	ADD	R1,R1,R8
	LDRB	R2,[R1]
	CMP	R2,#AXKISS
	MOVNE	PC,LR
	ADD	R1,R1,#AXCILJ	;cilnji AX25 naslovnika modulo 36 v R2
	MOV	R2,#0
	MOV	R4,#6
ACILJ5	LDRB	R5,[R1],#-1	;pretvori nov byte v R5
	MOV	R5,R5,LSR#1
	CMP	R5,#64
	ANDCC	R5,R5,#15	;nova stevilka
	ANDCS	R5,R5,#31	;nova crka
	ADDCS	R5,R5,#9
	ADD	R2,R2,R2,LSL#3	;pomnozi R2 s 36 in pristej R5
	ADD	R2,R5,R2,LSL#2
	SUBS	R4,R4,#1
	BNE	ACILJ5
ACILJ4	LDR	R1,=ANASLOV	;poisci ciljni IP naslovnika v R2
	MOV	R4,#MAXARP
ACILJ1	LDR	R5,[R1]
	CMP	R5,R2
	BEQ	ACILJ2
	ADD	R1,R1,#AKORAK
	SUBS	R4,R4,#1
	BNE	ACILJ1
	MOV	PC,LR
ACILJ2	LDR	R5,[R1,#4]	;preveri naslov cilja
	MOVS	R5,R5
	ADDNE	R0,R1,#4	;veljaven naslov v R0
	MOV	PC,LR
	LTORG

;*** Obdelava SLIP okvirjev *** (R0-8)
	ALIGN	16
SOBDELA	STR	LR,[SP,#-4]!
	LDR	R0,=SRXVMES	;iz sprejemnega vmesnika
	BL	IZVMES
	MOVS	R8,R8
	LDREQ	PC,[SP],#4
	LDR	R0,=MNASLOV	;brez naslova >>> KISS oddajnik?
	LDR	R0,[R0]
	MOVS	R0,R0
	BEQ	KISSTX
	LDR	R1,=LNASLOV	;lokalno vkljuceno?
	LDR	R1,[R1]
	MOVS	R1,R1
	BEQ	IPODDAJ		;oddaja IP okvirjev
	MOV	R3,#-1
	BL	SVNOS		;vnos slisane postaje iz (R8), jakost iz R3
	BL	LIZPOKV		;prikazi vse lokalno sprejete okvirje
	LDRH	R2,[R8,#2]	;kazalec R2 in zivljenjska doba R1
	ADD	R2,R2,R8
	LDR	R1,[R2]
	CMP	R1,#10		;preskok okvirja v oddajo?
	BEQ	ODDAJA
	AND	R0,R1,#15	;okvir brez preskoka?
	CMP	R0,#15
	BNE	SOBDEL1
	LDR	R3,[R2,#4]	;naslov R3
	LDR	R0,=MNASLOV	;moj naslov?
	LDR	R0,[R0]
	CMP	R0,R3
	BEQ	OBDELA		;obdelava sprejetih okvirjev
	MVNS	R0,R3		;naslov VSEM?
	BEQ	OBDELA		;obdelava sprejetih okvirjev
SOBDEL1	MOVS	R1,R1,LSR#4	;zmanjsaj zivljenjsko dobo
	STR	R1,[R2]
	BNE	SOBDEL2		;preveri zivljenjsko dobo?
	BL	SPROSTI
	LDR	PC,[SP],#4
SOBDEL2	LDR	R0,=STXVMES	;okvir v SLIP oddajni vmesnik
	BL	VVMES
	LDR	PC,[SP],#4
	LTORG

;*** Oddaja IP okvirjev *** (R0-5,R8,SP+4)+ODDAJA
NMINZAC	EQU	96		;minimalni zacetek okvirja
	ALIGN	16
IPODDAJ	LDRH	R3,[R8,#2]	;preverjeni kazalec R3?
	CMP	R3,#NMINZAC
	BCC	IPODDA2
	ADD	R3,R3,R8
	MOV	R0,#0		;vstavi locilo
	STR	R0,[R3,#-4]!
	LDR	R0,=MNASLOV	;vstavi MNASLOV
	LDR	R0,[R0]
	STR	R0,[R3,#-4]!
	MOV	R0,#0		;vstavi locilo
	STR	R0,[R3,#-4]!
	LDR	R0,=INASLOV	;prepisi INASLOV
	BL	ACILJ		;cilj iz ARP tabele?
	LDR	R1,[R0]		;neobstojec naslov?
	MOVS	R1,R1
	BEQ	IPODDA2
	MOV	R1,#MAXNAS
	ADD	R0,R0,R1,LSL#2
	MOV	R4,R3
IPODDA1	LDR	R2,[R0,#-4]!
	STR	R2,[R3,#-4]!
	MOVS	R2,R2
	MOVEQ	R3,R4
	SUBS	R1,R1,#1
	BNE	IPODDA1
	SUB	R3,R3,#4	;prostor za stevilko okvirja
	SUB	R3,R3,R8	;izracunaj nov zacetek
	LDRH	R0,[R8,#2]
	STRH	R3,[R8,#2]
	LDRH	R1,[R8]		;izracunaj novo dolzino
	ADD	R1,R1,R0
	SUB	R1,R1,R3
	STRH	R1,[R8]
	B	ODDAJA		;okvir R8 v oddajo RMII/SLIP
IPODDA2	BL	SPROSTI		;sprosti napacni zacetek
	LDR	PC,[SP],#4
	LTORG

;*** KISS oddajnik SATNC *** (R0-2,R8,SP+4)
KISSTX	LDR	R0,=LNASLOV	;lokalno vkljuceno SLIP >>> Ethernet?
	LDR	R0,[R0]
	MOVS	R0,R0
	BEQ	KISSTX0
	LDR	R0,=ETXVMES	;v ETH oddajni vmesnik
	BL	VVMES
	LDR	PC,[SP],#4
KISSTX0	LDRH	R0,[R8]		;skrajsa 1 byte
	SUB	R0,R0,#1
	STRH	R0,[R8]
	LDRH	R1,[R8,#2]	;prepisi ostanek 1 byte navzdol
	ADD	R1,R1,R8
KISSTX1	LDRB	R2,[R1,#1]	
	STRB	R2,[R1],#1
	SUBS	R0,R0,#1
	BNE	KISSTX1
	LDR	R0,=MPONOVI	;brisi minimum ponavljanja
	MOV	R1,#0
	STRB	R1,[R0]
	MOV	R0,#0		;brez ponavljanj
	STRB	R0,[R8,#4]
	MOV	R0,#0		;zakasnitev ponavljanja 0
	STRB	R0,[R8,#5]
	LDR	R0,=HTXVMES	;v RMII oddajni vmesnik
	BL	VVMES
	LDR	PC,[SP],#4
	LTORG

;*** Chip ID mikrokrmilnika & KSZ8851SNL *** (R0-3)+IAP
CHIPID	STR	LR,[SP,#-4]!
	LDR	R0,=IAPCMD	;Read part identification number
	MOV	R2,#54		;Command code 54
	STR	R2,[R0]
	LDR	R1,=IAPRES
	LDR	R2,=IAPADDR	;klic na IAP naslov
	MOV	LR,PC
	BX	R2
	LDR	R0,=IAPRES	;Result0 preko R3 v PARTID
	LDR	R3,[R0,#4]
	LDR	R0,=PARTID
	STR	R3,[R0]
	MOV	R0,#0xC0	;(CIDER) ID KSZ8851SNL
	BL	KSZ16R
	MOV	R0,R3		;neveljaven ID?
	LDR	R1,=0xFFF0
	AND	R0,R0,R1
	LDR	R1,=0x8870
	CMP	R0,R1
	ORRNE	R3,R3,#0xFF000000
	LDR	R0,=KSZID	;shrani ID in zastavico
	STR	R3,[R0]
	MOV	R0,#13		;nova vrsta ob zagonu 
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	LDR	PC,[SP],#4
	LTORG

;*** Inacica programa in ID mikrokrmilnika & SPI-ETH *** (R0-4)
INACICA	STR	LR,[SP,#-4]!
	LDR	R3,=INACIC1	;izpisi inacico programa
	BL	TXTIZP
	LDR	R0,=PARTID	;ID mikrokrmilnika
	LDR	R3,[R0]
	BL	HEXIZP
	MOV	R0,#":"
	BL	ZNAK
	LDR	R0,=KSZID	;ID KSZ8851SNL
	LDR	R3,[R0]
	BL	HEXIZP
	LDR	R3,=INACIC2
	BL	TXTIZP
	LDR	PC,[SP],#4
INACIC1	DCB	"*** SATNC - S53MV 27.03.2024 ID=",0
INACIC2	DCB	" ***",13,10,0
	ALIGN	4
	LTORG

;*** Inicializiraj seznam slisanih *** (R0-2)
INISLIS	LDR	R0,=SMETER	;brisi SMETER
	MOV	R2,#0
	STR	R2,[R0]
	LDR	R0,=SLISAL	;brisi seznam
	LDR	R1,=MAXSLIS
INISLI1	SUBS	R1,R1,#1
	MOVMI	PC,LR
	STR	R2,[R0],#4	;brisi 20byte
	STR	R2,[R0],#4
	STR	R2,[R0],#4
	STR	R2,[R0],#4
	STR	R2,[R0],#4
	B	INISLI1
	LTORG

;*** Inicializiraj ADC *** (R0-1)
INIADC	LDR	R0,=0xE0034000	;AD0CR
	LDR	R1,=0x0021181F	;START=0,PDN=1,BURST=1,CLKDIV=0x18,SEL=0x1F
	STR	R1,[R0]
	MOV	PC,LR
	LTORG

;*** Povprecje AD0.0,1,2 v R3 *** (R0-3)
VAD012	LDR	R0,=0xE0034010	;AD0DR0,1,2
	LDR	R1,[R0],#4
	LDR	R2,[R0],#4
	LDR	R3,[R0],#4
	ADD	R1,R1,R2	;R3=5*R1+5*R2+6*R3
	ADD	R1,R1,R3
	ADD	R1,R1,R1,LSL#2
	ADD	R3,R3,R1
	MOV	R3,R3,LSL#12	;ocisceno povprecje/16 v R3
	MOV	R3,R3,LSR#22
	MOV	PC,LR
	LTORG

;*** Vrednost AD0.3 v R3 *** (R0-1,R3)
VAD3	LDR	R0,=0xE003401C	;AD0DR3
	LDR	R1,[R0]
	MOV	R1,R1,LSL#16	;ociscen rezultat v R3
	MOV	R3,R1,LSR#22
	MOV	PC,LR
	LTORG

;*** Zakasnitev TSLOT *** (R0-2)
ZSLOT	LDR	R0,=0xE0004008	;T0TC - Timer0
	LDR	R1,[R0]
ZSLOT1	LDR	R0,=0xE0004008	;T0TC - Timer0
	LDR	R2,[R0]
	SUB	R2,R2,R1
	LDR	R0,=TSLOT	;pocakaj TSLOT
	LDR	R0,[R0]
	CMP	R2,R0
	BCC	ZSLOT1
	MOV	PC,LR
	LTORG

;*** Zacetek programa po resetu SATNC ***
ZACNI	LDR	SP,=SKLAD	;inicializiraj SP
	BL	INIPAR		;inicializiraj parametre iz FLASH
	BL	INITAKT		;preveri takt ARM procesorja
	BL	PESJAN		;vkljuci psa cuvaja v LPC23xx
	BL	INIBLOK		;razrezi RAM v bloke dolzine 1632=0x660 v LPC23xx
	BL	INIVMES		;inicializiraj vse vmesnike v SATNC
	BL	INIDVOJ		;inicializiraj tabelo dvojnikov
	BL	INISLIS		;inicializiraj seznam slisanih
	BL	PINGPIO		;izberi PIN in GPIO v SATNC
	BL	PLLMAM		;inicializiraj PLL in MAM v LPC23xx
	BL	PWRCLK		;vklop in takt I/O enot SATNC
	BL	TIMER0		;inicializiraj Timer0
	BL	UART0		;inicializiraj UART0
	BL	UART1		;inicializiraj UART1
	BL	INIADC		;inicializiraj ADC
	BL	INISSP1		;inicializiraj SSP1
	BL	INIKSZ		;inicializiraj KSZ8851SNL
	BL	INIRMII		;inicializiraj RMII v LPC23xx
	BL	VIC		;nastavi prekinitve
	BL	CHIPID		;Chip ID mikrokrmilnika
	BL	INACICA		;inacica programa in ID mikrokrmilnika
	BL	IRQON		;vkljuci prekinitve IRQ in FIQ v ARM
	B	ZANKA		;glavna zanka programa
	LTORG

;*** Glavna hitra zanka programa v SATNC ***
	ALIGN	16
ZANKA	BL	SLIPTX		;SLIP oddaja UART1
	BL	PREKLOP		;preklop sprejem/oddaja v SATNC
	BL	PSEVNAK		;polinomski izvor (2^32-1) v R0
	BL	SOBDELA		;obdelava SLIP okvirjev 
	BL	RMIIRX		;sprejem RMII okvirjev (+HOBDELA)
	BL	KETHTX		;oddaja ETH okvirja iz KTXVMES na KSZ8851SNL
	LDR	R0,=SZANK	;stevec zank
	LDR	R1,[R0]
	ADD	R1,R1,#1
	STR	R1,[R0]
	LDR	R0,=0xE0004008	;T0TC - Timer0
	LDR	R1,[R0]
	LDR	R0,=MIKRSEK	;mikrosekundni stevec
	LDRH	R2,[R0]
	SUB	R1,R1,R2
	MOVS	R1,R1,LSL#16
	BMI	ZANKA
;*** Milisekundna zanka programa ***
	ADD	R2,R2,#1000
	STRH	R2,[R0]
	LDR	R0,=0x3FFFC036	;FIO1PIN2 P1.17=/DCD?
	LDRB	R0,[R0]
	TST	R0,#0x02
	LDR	R2,=SDCD	;stevec DCD
	LDRHEQ	R1,[R2]
	ADDEQ	R1,R1,#1
	STRHEQ	R1,[R2]
	LDR	R0,=0x3FFFC036	;FIO1PIN2 P1.16 sprejem/oddaja?
	LDRB	R0,[R0]
	TST	R0,#0x01
	LDR	R2,=SPTT	;stevec PTT
	LDRHEQ	R1,[R2]
	ADDEQ	R1,R1,#1
	STRHEQ	R1,[R2]
	BL	KETHRX		;sprejem ETH okvirjev iz KSZ8851SNL
	BL	EODDAJ		;Ethernet oddaja IP okvirjev z ARP
	BL	UKAZ		;ukaz iz UART0 v LPC23xx
	BL	ZASLON		;zaslon na UART0 v LPC23xx
	LDR	R0,=MILISEK	;milisekundni stevec
	LDRH	R1,[R0]
	ADD	R1,R1,#1
	STRH	R1,[R0]
	CMP	R1,#1000
	BCC	ZANKA
;*** Sekundna zanka programa ***
	MOV	R1,#0
	STRH	R1,[R0]
	LDR	R0,=SDCD	;prepisi SDCD v NDCD in reset SDCD
	LDRH	R2,[R0]
	LDR	R1,=NDCD
	STRH	R2,[R1]
	MOV	R2,#0
	STRH	R2,[R0]
	LDR	R0,=SPTT	;prepisi SPTT v NPTT in reset SPTT
	LDRH	R2,[R0]
	LDR	R1,=NPTT
	STRH	R2,[R1]
	MOV	R2,#0
	STRH	R2,[R0]
	LDR	R0,=SZANK	;prepisi SZANK v NZANK in reset SZANK
	LDR	R2,[R0]
	LDR	R1,=NZANK
	STR	R2,[R1]
	MOV	R2,#0
	STR	R2,[R0]
	LDR	R0,=SEKUNDA	;sekundni stevec
	LDR	R1,[R0]
	ADD	R1,R1,#1
	STR	R1,[R0]
	BL	PESCUVA	;pes cuva sprejem v LPC23xx
	BL	SSTANJE	;izpis stanja vsako sekundo?
	BL	SADCK	;izpis ADC vsako sekundo?
	BL	SPHYMAC	;izpis PHY in MAC vsako sekundo?
	BL	ODZIV	;oddaja meritve odziva?
	BL	SCACHE	;staranje EthernetARP cacheja
	BL	KOKODAK	;kokodakanje istega sporocila?
	B	ZANKA
	LTORG

;*** Izpis stanja vsako sekundo *** (R0-4)
SSTANJE	LDR	R0,=ZNACIN
	LDRB	R0,[R0]
	TST	R0,#4
	MOVEQ	PC,LR
	STR	LR,[SP,#-4]!	;resi link
	BL	UBRISI		;brisi natipkan ukaz
	BL	STANJE
	BL	UPPISI		;ponovni izpisi natipkan ukaz
	LDR	PC,[SP],#4
	LTORG

;*** Izpis ADC vsako sekundo *** (R0-4)
SADCK	LDR	R0,=ZNACIN
	LDRB	R0,[R0]
	TST	R0,#64
	MOVEQ	PC,LR
	STR	LR,[SP,#-4]!	;resi link
	BL	UBRISI		;brisi natipkan ukaz
	BL	ADCK		;izpisi ADC telemetrijo in telekomando
	MOV	R0,#13		;nova vrsta
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	BL	UPPISI		;ponovni izpisi natipkan ukaz
	LDR	PC,[SP],#4
	LTORG

;*** Izpisi %DCD, %PTT, bloki, zanke in cas *** (R0-4)
STANJE	STR	LR,[SP,#-4]!	;resi link
	LDR	R3,=STANJE1	;odstotek DCD XX.X%
	BL	TXTIZP
	LDR	R0,=NDCD
	LDRH	R3,[R0]
	BL	DEC1IZP
	LDR	R3,=STANJE2	;odstotek PTT XX.X%
	BL	TXTIZP
	LDR	R0,=NPTT
	LDRH	R3,[R0]
	BL	DEC1IZP
	LDR	R3,=STANJE3	;stevilo prostih blokov
	BL	TXTIZP
	BL	PROSTO
	MOV	R3,R0
	BL	DECIZP
	LDR	R3,=STANJE4	;stevilo zank/s
	BL	TXTIZP
	LDR	R0,=NZANK
	LDR	R3,[R0]
	BL	DECIZP
	LDR	R3,=STANJE5	;cas delovanja
	BL	TXTIZP
	LDR	R0,=SEKUNDA
	LDR	R3,[R0]
	BL	HMSIZP		;izpisi R3 kot dneve/ure/minute/sekunde
	LDR	PC,[SP],#4
STANJE1 DCB	"DCD: ",0
STANJE2	DCB	"% PTT: ",0
STANJE3	DCB	"% ",0
STANJE4	DCB	"blok ",0
STANJE5	DCB	"zank/s ",0
	ALIGN	4
	LTORG

;*** Izpis seznama slisanih postaj *** (R0-6)+HMSIZP
SSLIZP	STR	LR,[SP,#-4]!	;resi link
	LDR	R1,=ZNACIN	;izpisi obliko N36/HEX
	LDRB	R0,[R1]
	LDR	R3,=SSLIZP5
	TST	R0,#32
	LDRNE	R3,=SSLIZP6
	BL	TXTIZP
	LDR	R5,=MAXSLIS	;seznam slisanih
	LDR	R6,=SLISAL
SSLIZP1	SUBS	R5,R5,#1	;konec seznama?
	LDRMI	PC,[SP],#4
	LDR	R3,[R6],#4	;veljaven vnos?
	MOVS	R3,R3
	LDREQ	PC,[SP],#4
	STR	R5,[SP,#-4]!
	BL	N36IZP		;izpis slisanega naslova
	LDR	R3,=SSLIZP2
	BL	TXTIZP
	LDR	R3,[R6]		;izpis jakosti signala?
	MOVS	R3,R3
	LDR	R5,=NENOTA	;Yskala/enota
	STR	R6,[SP,#-4]!
	BLPL	ADCIZP
	LDR	R6,[SP],#4
	LDR	R3,[R6],#4	;izpis "SLIP"?
	MOVS	R3,R3
	LDR	R3,=SSLIZP7
	BLMI	TXTIZP
	MOV	R0,#32
	BL	ZNAK
	LDR	R3,[R6],#4	;izpis stevila okvirjev
	BL	DECIZP
	LDR	R3,=SSLIZP3
	BL	TXTIZP
	LDR	R3,[R6],#4	;izpis stevila bajtov
	BL	DECIZP
	LDR	R3,=SSLIZP4
	BL	TXTIZP
	LDR	R0,=SEKUNDA	;izpis starosti vpisa v sekundah
	LDR	R3,[R0]
	LDR	R1,[R6],#4
	SUB	R3,R3,R1
	BL	HMSIZP		;izpisi R3 kot dneve/ure/minute/sekunde
	LDR	R5,[SP],#4
	B	SSLIZP1
SSLIZP2	DCB	" S:",0
SSLIZP3	DCB	"frame/",0
SSLIZP4	DCB	"byte -",0
SSLIZP5	DCB	"Slisano 0=N36(crke+stevilke):",13,10,0
SSLIZP6	DCB	"Slisano 1=HEX(8znakov):",13,10,0
SSLIZP7	DCB	"SLIP",0
	ALIGN	4
	LTORG

;*** Izpisi R3 kot dneve/ure/minute/sekunde *** (R0-4)
HMSIZP	STR	LR,[SP,#-4]!	;resi link
	MOV	R0,#60		;preracunaj v dneve/ure/minute/sekunde
	BL	NDELI
	STR	R0,[SP,#-4]!
	MOV	R0,#60
	BL	NDELI
	STR	R0,[SP,#-4]!
	MOV	R0,#24
	BL	NDELI
	STR	R0,[SP,#-4]!
	BL	DECIZP		;izpisi dneve
	LDR	R3,=HMSIZP1
	BL	TXTIZP
	LDR	R3,[SP],#4
	BL	DECIZP		;izpisi ure
	LDR	R3,=HMSIZP2
	BL	TXTIZP
	LDR	R3,[SP],#4
	BL	DECIZP		;izpisi minute
	LDR	R3,=HMSIZP3
	BL	TXTIZP
	LDR	R3,[SP],#4
	BL	DECIZP		;izpisi sekunde
	LDR	R3,=HMSIZP4
	BL	TXTIZP
	LDR	PC,[SP],#4
HMSIZP1	DCB	"d/",0
HMSIZP2	DCB	"h/",0
HMSIZP3	DCB	"min/",0
HMSIZP4	DCB	"s",13,10,0
	ALIGN	4
	LTORG

;*** Izpis PHY in MAC vsako sekundo *** (R0-5)
SPHYMAC	LDR	R0,=ZNACIN
	LDRB	R1,[R0]
	TST	R1,#16
	MOVEQ	PC,LR
	STR	LR,[SP,#-4]!	;resi link
	BL	UBRISI		;brisi natipkan ukaz
	BL	PHYMAC
	BL	UPPISI		;ponovni izpisi natipkan ukaz
	LDR	PC,[SP],#4
	LTORG

;*** Izpisi PHY status in sogovornikov MAC *** (R0-5)
PHYMAC	STR	LR,[SP,#-4]!	;resi link
	LDR	R0,=PHYSTAT
	LDRH	R5,[R0]
	LDR	R3,=PHYMAC1	;izpis MDI
	TST	R5,#0x0080
	LDREQ	R3,=PHYMAC2	;izpis MDIX?
	BL	TXTIZP
	LDR	R3,=PHYMAC3	;izpis POL?
	TST	R5,#0x2000
	BLNE	TXTIZP
	LDR	R3,=PHYMAC4	;izpis ANC?
	TST	R5,#0x0040
	BLNE	TXTIZP
	LDR	R3,=PHYMAC5	;izpis DUP?
	TST	R5,#0x0200
	BLNE	TXTIZP
	LDR	R3,=PHYMAC6	;izpis 100M?
	TST	R5,#0x0400
	BLNE	TXTIZP
	LDR	R3,=PHYMAC7	;izpis FLOW?
	TST	R5,#0x0010
	BLNE	TXTIZP
	LDR	R3,=PHYMAC8	;izpis LINK?
	TST	R5,#0x0020
	BLNE	TXTIZP
	LDR	R3,=PHYMAC9	;izpisi sogovornikov MAC/izracunani MAC
	BL	TXTIZP
	LDR	R5,=MACMAC
	BL	MACIZP
	MOV	R0,#13
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	LDR	PC,[SP],#4
PHYMAC1	DCB	"MDI",0
PHYMAC2	DCB	"MDIX",0
PHYMAC3	DCB	" POL",0
PHYMAC4	DCB	" ANC",0
PHYMAC5	DCB	" DUP",0
PHYMAC6	DCB	" 100M",0
PHYMAC7	DCB	" FLOW",0
PHYMAC8	DCB	" LINK",0
PHYMAC9	DCB	" MAC: ",0
	ALIGN	4
	LTORG

;*** Izpisi ADC telemetrijo in telekomando v LPC23xx *** (R0-6)
ADCK	STR	LR,[SP,#-4]!	;resi link
	LDR	R5,=NENOTA	;Yskala/enota
	LDR	R3,=ADCK1	;#S
	BL	TXTIZP
	LDR	R0,=SMETER	;S-METER sprejetega okvirja
	LDR	R3,[R0]
	LDR	R5,=NENOTA	;Yskala/enota
	BL	ADCIZP		;izpisi ADC TLM R3, parametri [R5],#12
	LDR	R3,=ADCK2	;#1
	BL	TXTIZP
	BL	VAD012		;povprecje AD0.0,1,2 v R3
	LDR	R5,=NENOTA	;Yskala/enota
	BL	ADCIZP		;izpisi ADC TLM R3, parametri [R5],#12
	LDR	R3,=ADCK3	;#2
	BL	TXTIZP
	BL	VAD3		;vrednost AD0.3 v R3
	BL	ADCIZP		;izpisi ADC TLM R3, parametri [R5],#12
	LDR	R3,=ADCK4	;telekomanda
	BL	TXTIZP
	LDR	R0,=0x3FFFC037	;FIO1PIN3 P1.31 telekomanda?
	LDRB	R3,[R0]
	MOV	R3,R3,LSR#7
	LDR	R0,=ZNACIN	;vklop/izklop TX?
	LDR	R1,[R0]
	TST	R1,#128
	ORRNE	R3,R3,#2
	BL	DECIZP
	LDR	PC,[SP],#4
ADCK1	DCB	"S:",0
ADCK2	DCB	" 1:",0
ADCK3	DCB	" 2:",0
ADCK4	DCB	" K:",0
	ALIGN	4
	LTORG

;*** Izpisi ADC TLM R3, parametri [R5],#12 *** (R0-6)
ADCIZP	STR	LR,[SP,#-4]!
	MOV	R6,R3
	BL	DECIZP		;surovi izpis ADC
	MOV	R0,#"="
	BL	ZNAK
	LDR	R0,[R5],#4	;mnozenje s konstanto
	MUL	R3,R6,R0
	LDR	R0,[R5],#4	;odmik od vrednosti/1024
	ADDS	R3,R0,R3,ASR#10
	RSBMI	R3,R3,#0	;obrni predznak?
	MOV	R0,#"-"
	BLMI	ZNAK
	BL	DEC1IZP		;izpis na eno decimalko
	LDR	R3,[R5],#4	;izpis 4 znakov merske enote
	BL	ADCIZP1
	BL	ADCIZP1
	BL	ADCIZP1
	BL	ADCIZP1
	LDR	PC,[SP],#4
ADCIZP1	MOV	R3,R3,ROR#24	;izpis znaka brez nicle
	ANDS	R0,R3,#255
	BNE	ZNAK
	MOV	PC,LR
	LTORG

;*** Staranje EthernetARP cacheja *** (R0-2)
SCACHE	LDR	R0,=ECACHE
	MOV	R1,#MCACHE
SCACHE1	SUBS	R1,R1,#1	;konec seznama?
	MOVMI	PC,LR
	LDRH	R2,[R0,#10]	;veljaven vnos?
	MOVS	R2,R2		
	MOVEQ	PC,LR
	SUB	R2,R2,#1	;znizaj zivljenjsko dobo
	STRH	R2,[R0,#10]
	ADD	R0,R0,#16	;naslednji vnos
	B	SCACHE1
	LTORG

;*** Sestavi odziv N byte *** (R0-4,R8)+ODDAJA
ODZZAC	EQU	8
ODZIV	STR	LR,[SP,#-4]!	;resi link
	LDR	R0,=NODZIV	;odziv vkljucen?
	LDRH	R0,[R0]
	MOVS	R0,R0
	LDREQ	PC,[SP],#4
	LDR	R0,=MNASLOV	;KISS RX/TX?
	LDR	R0,[R0]
	MOVS	R0,R0
	LDREQ	PC,[SP],#4	
	LDR	R0,=NASLOV	;NASLOV obstaja?
	LDR	R0,[R0]
	MOVS	R0,R0
	LDREQ	PC,[SP],#4
	BL	DODELI		;dodeli blok za odziv, obstaja?
	MOVS	R8,R8
	LDREQ	PC,[SP],#4
	MOV	R0,#ODZZAC	;zacetek sestavljanja odziva
	STRH	R0,[R8,#2]
	ADD	R4,R0,R8	;kazalec sestavljanja R4
	ADD	R4,R4,#4	;prostor za stevilko
	LDR	R2,=NASLOV	;naslov v okvir
	MOV	R3,#MAXNAS
ODZIV1	LDR	R0,[R2],#4
	MOVS	R0,R0
	BEQ	ODZIV2
	STR	R0,[R4],#4
	SUBS	R3,R3,#1
	BNE	ODZIV1
ODZIV2	MOV	R0,#0		;locilo na koncu naslova v okvir
	STR	R0,[R4],#4
	LDR	R0,=MNASLOV	;moj naslov v okvir
	LDR	R0,[R0]
	STR	R0,[R4],#4
	MOV	R0,#0		;locilo po mojem naslovu v okvir
	STR	R0,[R4],#4
	MOV	R0,#0		;oznaka besedila v okvir
	STR	R0,[R4],#4
	LDR	R0,=0x2F2F2F2F	;oznaka javljanja "////"
	STR	R0,[R4],#4
	LDR	R0,=0x3E3E3E3E	;oznaka meritve odziva ">>>>"
	STR	R0,[R4],#4
	LDR	R0,=0xE0004008	;vstavi T0TC - Timer0 v R3
	LDR	R3,[R0]
	STR	R3,[R4],#4
	LDR	R0,=NODZIV	;vstavi preizkusno zaporedje N byte
	LDRH	R2,[R0]
ODZIV3	MOV	R0,#8
ODZIV4	ADDS	R3,R3,R3	;polinom 1+X^28+X^31
	EORMI	R3,R3,#1
	TST	R3,#0x10000000
	EORNE	R3,R3,#1
	SUBS	R0,R0,#1
	BNE	ODZIV4
	STRB	R3,[R4],#1	;nov byte v odziv
	SUBS	R2,R2,#1
	BNE	ODZIV3
	SUB	R4,R4,R8	;izracunaj dolzino sestavljenega okvirja
	LDRH	R1,[R8,#2]
	SUB	R4,R4,R1
	STRH	R4,[R8]
	BL	UBRISI		;brisi natipkan ukaz
	MOV	R0,#"!"		;napisi ! za oddani odziv
	BL	ZNAK
	BL	UPPISI		;ponovni izpisi natipkan ukaz
	B	ODDAJA		;okvir R8 v oddajo RMII/SLIP
	LTORG

;*** Meritev odziva *** (R0-7)
MERI	STR	LR,[SP,#-4]!	;resi link
	CMP	R5,#16		;dolzina podatkov?
	LDRCC	PC,[SP],#4
	LDR	R0,[R6]		;nicla za besedilo?
	MOVS	R0,R0
	LDRNE	PC,[SP],#4
	LDR	R0,[R6,#4]	;"////" za javise?
	LDR	R1,=0x2F2F2F2F
	CMP	R0,R1
	BEQ	MERI1
	LDR	R0,[R6,#4]	;"****" za javljanje?
	LDR	R1,=0x2A2A2A2A
	CMP	R0,R1
	LDRNE	PC,[SP],#4
MERI1	LDR	R0,[R6,#8]	;">>>>" za meritev odziva?
	LDR	R1,=0x3E3E3E3E
	CMP	R0,R1
	LDRNE	PC,[SP],#4
	MOV	R0,#32		;izpisi odziv
	BL	ZNAK
	LDR	R0,=0xE0004008	;T0TC - Timer0 v R3
	LDR	R3,[R0]
	LDR	R7,[R6,#12]	;odstej cas odziva R7
	SUBS	R3,R3,R7
	RSBMI	R3,R3,#0	;obrni predznak?
	MOV	R0,#"-"
	BLMI	ZNAK
	MOV	R0,#100		;deli s 100 za korak 0.1ms
	BL	NDELI
	BL	DEC1IZP
	LDR	R3,=MERI5
	BL	TXTIZP
	SUB	R5,R5,#16	;preskoci glavo odziva
	ADD	R6,R6,#16
	MOV	R3,R5		;izpisi stevilo byte
	BL	DECIZP
	LDR	R3,=MERI6
	BL	TXTIZP
	MOV	R3,#0		;resetiraj stevec napak R3
	BL	MERI4
MERI2	MOV	R0,#8
MERI3	ADDS	R7,R7,R7	;polinom 1+X^28+X^31
	EORMI	R7,R7,#1
	TST	R7,#0x10000000
	EORNE	R7,R7,#1
	SUBS	R0,R0,#1
	BNE	MERI3
	LDRB	R1,[R6],#1	;primerjaj izracunano in sprejeto
	EOR	R1,R1,R7
	TST	R1,#255
	ADDNE	R3,R3,#1
MERI4	SUBS	R5,R5,#1
	BPL	MERI2
	BL	DECIZP		;izpisi stevilo napak
	LDR	R3,=MERI7
	BL	TXTIZP
	LDR	PC,[SP],#4
MERI5	DCB	"ms ",0
MERI6	DCB	"byte ",0
MERI7	DCB	"napak",0
	ALIGN	4
	LTORG

;*** Ukaz iz UART0 v LPC23xx *** (R0-2)+IZVEDI
	ALIGN	16
UKAZ	LDR	R0,=0xE000C014	;U0LSR bit0=ReceiveDataReady?
	LDRB	R1,[R0]
	TST	R1,#0x01
	MOVEQ	PC,LR
	STR	LR,[SP,#-4]!	;resi link
	LDR	R0,=0xE000C000	;U0RBR sprejeti znak 7bit v R0
	LDRB	R0,[R0]
	AND	R0,R0,#0x7F
	LDR	R2,=TIPVMES	;vmesnik R2 in kazalec R1
	LDRH	R1,[R2]
	CMP	R0,#127		;brisanje DEL?
	BEQ	UKAZ1
	CMP	R0,#8		;brisanje BS?
	BEQ	UKAZ1
	CMP	R0,#32		;CR ali kontrolni znak za konec ukaza?
	BCC	UKAZ4
	CMP	R1,#MAXUKAZ	;znak v vmesnik za tipke
	LDRCS	PC,[SP],#4
	ADD	R1,R1,#1
	STRH	R1,[R2]
	ADD	R1,R1,#1
	STRB	R0,[R2,R1]
	BL	ZNAK
	LDR	PC,[SP],#4
UKAZ1	SUBS	R1,R1,#1	;popravi kazalec nazaj
	LDRMI	PC,[SP],#4
	STRH	R1,[R2]
	BNE	UKAZ3		;brisi veljavno vrstico?
	MOV	R0,#32
	MOV	R1,#MAXUKAZ
	ADD	R2,R2,#2
UKAZ2	STRB	R0,[R2],#1
	SUBS	R1,R1,#1
	BPL	UKAZ2
UKAZ3	MOV	R0,#8		;brisi znak na zaslonu
	BL	ZNAK
	MOV	R0,#32
	BL	ZNAK
	MOV	R0,#8
	BL	ZNAK
	LDR	PC,[SP],#4
UKAZ4	ADD	R1,R1,#1	;povecaj kazalec
	STRH	R1,[R2]
	ADD	R1,R1,#1
	MOV	R0,#13		;CR v vmesnik, CR+LF na zaslon
	STRB	R0,[R2,R1]
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	BL	IZVEDI		;izvedi natipkan ukaz
	LDR	R0,=TIPVMES	;resetiraj tipke
	MOV	R1,#0
	STRH	R1,[R0]
	LDR	PC,[SP],#4
	LTORG

;*** Kokodakanje istega sporocila *** (R0-3)+SESTAVI
KOKODAK	STR	LR,[SP,#-4]!	;resi link
	LDR	R0,=ZNACIN	;cvekalni nacin?
	LDRB	R1,[R0]
	TST	R1,#1
	LDREQ	PC,[SP],#4
	LDR	R0,=TIPVMES	;prazen ukaz?
	LDRH	R1,[R0]
	MOVS	R1,R1
	LDRNE	PC,[SP],#4
	LDR	R0,=KSTEVEC	;odstevaj uro kokodakanja
	LDRB	R1,[R0]
	SUBS	R1,R1,#1
	STRB	R1,[R0]
	LDRNE	PC,[SP],#4
	LDRB	R1,[R0,#1]	;nova casovna meja
	STRB	R1,[R0]
	MOVS	R1,R1		;cvekanje brez kokodakanja?
	LDREQ	PC,[SP],#4
	LDR	R1,=TIPVMES	;preveri odtipkano vrstico?
	ADD	R1,R1,#2
	MOV	R2,#MAXUKAZ
KOKODA1	LDRB	R0,[R1],#1
	CMP	R0,#32
	BCC	KOKODA2
	SUBS	R2,R2,#1
	BPL	KOKODA1
	LDR	PC,[SP],#4
KOKODA2	LDR	R3,=TIPVMES	;kokodakanje odtipkane vrstice
	ADD	R3,R3,#2
KOKODA3	LDRB	R0,[R3],#1
	CMP	R0,#32
	BCC	KOKODA4
	BL	ZNAK
	B	KOKODA3
KOKODA4	MOV	R0,#13		;povratek v novo vrsto
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	B	SESTAVI
	LTORG

;*** Sestavi sporocilo *** (R0-4,R8,SP+4)+ODDAJA
SESTZAC	EQU	8
SESTAVI	LDR	R0,=TIPVMES	;samo CR v ukazu?
	LDRB	R0,[R0,#2]
	CMP	R0,#32
	BCS	SESTAV2
	LDR	R0,=ZNACIN	;preklop v ukazni nacin
	LDRB	R1,[R0]
	BIC	R1,R1,#1
	STRB	R1,[R0]
	LDR	R3,=SESTAV1
	BL	TXTIZP
	LDR	PC,[SP],#4
SESTAV1	DCB	"*** Ukazni nacin ***",13,10,0
	ALIGN	4
SESTAV2	LDR	R0,=MNASLOV	;KISS RX/TX?
	LDR	R0,[R0]
	MOVS	R0,R0
	BEQ	SESTAV3	
	LDR	R0,=NASLOV	;NASLOV obstaja?
	LDR	R0,[R0]
	MOVS	R0,R0
	BEQ	SESTAV3	
	BL	DODELI		;dodeli blok za sporocilo
	MOVS	R8,R8
	BNE	SESTAV5
SESTAV3	LDR	R3,=SESTAV4
	BL	TXTIZP
	LDR	PC,[SP],#4
SESTAV4	DCB	"*** Sporocilo izgubljeno ***",13,10,0
	ALIGN	4
SESTAV5	MOV	R0,#SESTZAC	;zacetek sestavljanja
	STRH	R0,[R8,#2]
	ADD	R4,R0,R8	;kazalec sestavljanja R4
	ADD	R4,R4,#4	;prostor za stevilko
	LDR	R2,=NASLOV	;naslov v okvir
	MOV	R3,#MAXNAS
SESTAV6	LDR	R0,[R2],#4
	MOVS	R0,R0
	BEQ	SESTAV7
	STR	R0,[R4],#4
	SUBS	R3,R3,#1
	BNE	SESTAV6
SESTAV7	MOV	R0,#0		;locilo na koncu naslova v okvir
	STR	R0,[R4],#4
	LDR	R0,=MNASLOV	;moj naslov v okvir
	LDR	R0,[R0]
	STR	R0,[R4],#4
	MOV	R0,#0		;locilo po mojem naslovu v okvir
	STR	R0,[R4],#4
	MOV	R0,#0		;oznaka besedila v okvir
	STR	R0,[R4],#4
	LDR	R2,=TIPVMES	;besedilo ukaza v okvir
	ADD	R2,R2,#2
SESTAV8	LDRB	R0,[R2],#1
	STRB	R0,[R4],#1
	CMP	R0,#32
	BCS	SESTAV8
	SUB	R4,R4,R8	;izracunaj dolzino sestavljenega okvirja
	LDRH	R1,[R8,#2]
	SUB	R4,R4,R1
	STRH	R4,[R8]
	B	ODDAJA		;okvir R8 v oddajo RMII/SLIP
	LTORG

;*** Desetisko stevilo (R2) v R0 zastavica R1 *** (R0-2)
DVHOD	MOV	R1,#-1		;neveljavna zastavica -1
	LDRB	R0,[R2]
	CMP	R0,#32		;konec ukaza?
	MOVCC	PC,LR
	ADD	R2,R2,#1
	SUBS	R0,R0,#48
	BCC	DVHOD
	CMP	R0,#10
	BCS	DVHOD
DVHOD1	LDRB	R1,[R2]		;veljavna stevilka?
	SUBS	R1,R1,#48
	BCC	DVHOD2
	CMP	R1,#10
	BCS	DVHOD2
	ADD	R0,R0,R0,LSL#2	;pomnozi z 10 in pristej stevilko
	ADD	R0,R1,R0,LSL#1
	ADD	R2,R2,#1
	B	DVHOD1
DVHOD2	MOV	R1,#0		;veljavna zastavica 0
	MOV	PC,LR

;*** IP4 naslov v R3 zastavica R1 *** (R0-3)
IVHOD	STR	LR,[SP,#-4]!
	BL	DVHOD		;vnos prve IP stevilke?
	MOVS	R1,R1
	LDRMI	PC,[SP],#4
	AND	R3,R0,#255
	BL	DVHOD		;vnos druge IP stevilke?
	MOVS	R1,R1
	LDRMI	PC,[SP],#4
	AND	R0,R0,#255
	ORR	R3,R0,R3,ROR#8
	BL	DVHOD		;vnos tretje IP stevilke?
	MOVS	R1,R1
	LDRMI	PC,[SP],#4
	AND	R0,R0,#255
	ORR	R3,R0,R3,ROR#8
	BL	DVHOD		;vnos cetrte IP stevilke?
	AND	R0,R0,#255
	ORR	R3,R0,R3,ROR#8
	MOV	R3,R3,ROR#8
	LDR	PC,[SP],#4

;*** Hexadecimalno stevilo (R2) v R0 zastavica R1 *** (R0-2)
HVHOD	MOV	R1,#-1		;neveljavna zastavica -1
	LDRB	R0,[R2]
	CMP	R0,#32		;konec ukaza?
	MOVCC	PC,LR
	ADD	R2,R2,#1
	CMP	R0,#48		;nepravi znak <0?
	BCC	HVHOD
	CMP	R0,#96		;male crke v velike
	SUBCS	R0,R0,#32
	CMP	R0,#71		;neprava crka >F?
	BCS	HVHOD
	CMP	R0,#58
	SUBCS	R0,R0,#23	;crka v hex
	CMP	R0,#42		;nepravi znaki 58...64?
	BCC	HVHOD
	AND	R0,R0,#15
HVHOD1	LDRB	R1,[R2]
	CMP	R1,#48		;nepravi znak <0 ali konec ukaza?
	BCC	HVHOD2
	CMP	R1,#96		;male crke v velike
	SUBCS	R1,R1,#32
	CMP	R1,#71		;neprava crka >F?
	BCS	HVHOD2
	CMP	R1,#58
	SUBCS	R1,R1,#23	;crka v hex
	CMP	R1,#42		;nepravi znaki 58...64?
	BCC	HVHOD2
	AND	R1,R1,#15
	ADD	R0,R1,R0,LSL#4	;pomnozi s 16 in pristej stevilko
	ADD	R2,R2,#1
	B	HVHOD1
HVHOD2	MOV	R1,#0		;veljavna zastavica 0
	MOV	PC,LR

;*** Naslov36 (R2) v R0 zastavica R1 *** (R0-2)
NVHOD	LDR	R0,=ZNACIN	;vkljucen HEX?
	LDRB	R0,[R0]
	TST	R0,#32
	BNE	HVHOD
	MOV	R1,#-1		;neveljavna zastavica -1
	LDRB	R0,[R2]
	CMP	R0,#32		;konec ukaza?
	MOVCC	PC,LR
	ADD	R2,R2,#1	;povecaj kazalec
	CMP	R0,#"*"		;naslov VSEM?
	BEQ	NVHOD3	
	CMP	R0,#48		;nepravi znak <0?
	BCC	NVHOD
	CMP	R0,#96		;male crke v velike
	SUBCS	R0,R0,#32
	CMP	R0,#91		;neprava crka >Z?
	BCS	NVHOD
	CMP	R0,#58		;veljavna stevilka+71?
	ADDCC	R0,#71
	CMP	R0,#65		;nepravi znaki 58...64?
	BCC	NVHOD
	ADD	R0,R0,#9
	AND	R0,R0,#63
	STR	R3,[SP,#-4]!	;resi R3
	MOV	R3,#1
NVHOD1	ADD	R3,R3,R3,LSL#3	;pomnozi R3 s 36
	MOV	R3,R3,LSL#2
	LDRB	R1,[R2]
	CMP	R1,#48		;konec ukaza ali nepravi znak <0?
	BCC	NVHOD2
	CMP	R1,#96		;male crke v velike
	SUBCS	R1,R1,#32
	CMP	R1,#91		;neprava crka >Z?
	BCS	NVHOD2
	CMP	R1,#58		;veljavna stevilka+71?
	ADDCC	R1,#71
	CMP	R1,#65		;nepravi znaki 58...64?
	BCC	NVHOD2
	ADD	R1,R1,#9
	AND	R1,R1,#63
	MLA	R0,R1,R3,R0	;R0=R1*R3+R0
	ADD	R2,R2,#1	;povecaj kazalec
	B	NVHOD1
NVHOD2	LDR	R3,[SP],#4	;vrni R3
	MOV	R1,#0		;veljavna zastavica 0
	MOV	PC,LR
NVHOD3	MOV	R0,#-1		;naslov VSEM
	MOV	R1,#0		;veljavna zastavica 0
	MOV	PC,LR
	LTORG

;*** Izvedi natipkan ukaz *** (R0-6)+SESTAVI
IZVEDI	STR	LR,[SP,#-4]!
	LDR	R0,=ZNACIN	;ukazni/pogovorni nacin?
	LDRB	R0,[R0]
	TST	R0,#1
	BNE	SESTAVI
	LDR	R2,=TIPVMES	;ukaz v (R2)
	ADD	R2,R2,#2
	LDRB	R0,[R2],#1	;prvi znak ukaza
	CMP	R0,#96		;mala crka v veliko?
	SUBCS	R0,R0,#32
	CMP	R0,#"A"		;ukaz A?
	BEQ	AIZVEDI
	CMP	R0,#"B"		;ukaz B?
	BEQ	BIZVEDI
	CMP	R0,#"C"		;ukaz C?
	BEQ	CIZVEDI
	CMP	R0,#"D"		;ukaz D?
	BEQ	DIZVEDI	
	CMP	R0,#"E"		;ukaz E?
	BEQ	EIZVEDI
	CMP	R0,#"F"		;ukaz F?
	BEQ	FIZVEDI
	CMP	R0,#"G"		;ukaz G?
	BEQ	GIZVEDI
	CMP	R0,#"H"		;ukaz H?
	BEQ	HIZVEDI
	CMP	R0,#"I"		;ukaz I?
	BEQ	IIZVEDI
	CMP	R0,#"J"		;ukaz J?
	BEQ	JIZVEDI
	CMP	R0,#"K"		;ukaz K?
	BEQ	KIZVEDI
	CMP	R0,#"L"		;ukaz L?
	BEQ	LIZVEDI
	CMP	R0,#"M"		;ukaz M?
	BEQ	MIZVEDI
	CMP	R0,#"N"		;ukaz N?
	BEQ	NIZVEDI
	CMP	R0,#"O"		;ukaz O?
	BEQ	OIZVEDI
	CMP	R0,#"P"		;ukaz P?
	BEQ	PIZVEDI
	CMP	R0,#"Q"		;ukaz Q?
	BEQ	QIZVEDI
	CMP	R0,#"R"		;ukaz R?
	BEQ	RIZVEDI
	CMP	R0,#"S"		;ukaz S?
	BEQ	SIZVEDI
	CMP	R0,#"T"		;ukaz T?
	BEQ	TIZVEDI
	CMP	R0,#"U"		;ukaz U?
	BEQ	UIZVEDI
	CMP	R0,#"V"		;ukaz V?
	BEQ	VIZVEDI
	CMP	R0,#"W"		;ukaz W?
	BEQ	WIZVEDI
	CMP	R0,#"X"		;ukaz X?
	BEQ	XIZVEDI
	CMP	R0,#"Y"		;ukaz Y?
	BEQ	YIZVEDI
	CMP	R0,#"Z"		;ukaz Z?
	BEQ	ZIZVEDI
	CMP	R0,#"?"		;ukaz "?"
	LDR	R3,=IZVEDI1	;nepoznan ukaz
	LDREQ	R3,=IZVEDI2	;seznam ukazov
	BL	TXTIZP
	LDR	PC,[SP],#4
IZVEDI1	DCB	"!!! Nepoznan ukaz (A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R,S,T,U,V,W,X,Y,Z,?)",13,10,0
IZVEDI2	DCB	"*** Seznam veljavnih ukazov SPI-RMII ArmTNC ***",13,10
	DCB	"A <IP/AX> <naslov#1> <naslov#2> <naslov#3> - (A)RP usmerjanje",13,10
	DCB	"B <baudrate> - nastavi (B)audrate za UART1 (SLIP)",13,10
	DCB	"C <perioda> - (C)vekanje / kokodakanje (izhod: prazna vrstica)",13,10
	DCB	"D <IP> <sprejmi> <zavrni> - IP (D)domena",13,10
	DCB	"E <hitrost-X> <naslov> - (E)thernet hitrost, MDIX in MAC naslov",13,10
	DCB	"F <kljuc> - vpis trenutnih parametrov v (F)LASH in RESET SATNC",13,10
	DCB	"G <prehod> <maska> <trajanje> - (G)ateway in EthernetARP cache",13,10
	DCB	"H <preklop> - slisani naslovi in preklop N36/(H)EX",13,10
	DCB	"I <naslov#1> <naslov#2> <naslov#3> - (I)P/AX privzeti naslov",13,10
	DCB	"J:<besedilo> - besedilo (J)avljanja (izklop: prazna vrstica)",13,10
	DCB	"K <preklop> <ukaz> <geslo> - (K)omanda in geslo z izpisom ADC",13,10
	DCB	"L <naslov#1> <naslov#2> <naslov#3> - naslovi (L)okalne zanke",13,10
	DCB	"M <naslov> - (M)oj naslov (0=KISS/SLIP<>Ethernet)",13,10
	DCB	"N <naslov#1> <naslov#2> <naslov#3> - (N)aslov za pogovor/meritev",13,10
	DCB	"O <stevilo> - meritev (O)dziva stevila byte, 0=izklop",13,10
	DCB	"P <stevilo> <zakasnitev> <minblok> - (P)onavljanja",13,10
	DCB	"Q <quartz> <mnogokratnik> <takt> - (Q)uartz: takt ARM procesorja",13,10
	DCB	"R - (R)ESET SATNCja",13,10
	DCB	"S <slot> <glava> <rep> - zakasnitve (S)lot, glava in rep",13,10
	DCB	"T <z DCD> <brez DCD> - (T)ecnoba preklopa na oddajo /65536",13,10
	DCB	"U <preklop> - izpis (U)porabnih/vseh okvirjev",13,10
	DCB	"V - izpis (V)seh nastavitev",13,10
	DCB	"W <zakasnitev> - EthernetARP (W)AIT",13,10
	DCB	"X <vklop/izklop> - izpis MDI(X), POL, ANC, DUP, 100M, FLOW, LINK, MAC",13,10
	DCB	"Y <m1> <o1> <e1> <m2> <o2> <e2> - (Y)skala/enota TLM 1/2",13,10
	DCB	"Z <vklop/izklop> - izpis %DCD, %PTT, prosto, (Z)ank/s, cas",13,10
	DCB	"? - izpisi ta seznam",13,10,0
	ALIGN	4
	LTORG

;*** Pregled nastavitev SATNC *** (R0-6)
PARAM	STR	LR,[SP,#-4]!	;resi link
	BL	INACICA		;inacica programa in ID mikrokrmilnika
	MOV	R0,#"A"		;nastavitev A
	BL	ZNAK
	MOV	R0,#32
	BL	ZNAK
	LDR	R0,=ANACIN
	LDRB	R3,[R0]
	AND	R3,R3,#7
	BL	DECIZP
	MOV	R0,#13
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	MOV	R0,#"B"		;nastavitev B
	BL	ZNAK
	MOV	R0,#32
	BL	ZNAK
	LDR	R3,=ARMTAKT
	LDR	R3,[R3]
	MOV	R3,R3,LSR#4
	LDR	R0,=BAUDMOD
	LDRH	R0,[R0]
	BL	NDELI
	BL	DECIZP
	MOV	R0,#13
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	MOV	R0,#"C"		;nastavitev C
	BL	ZNAK
	MOV	R0,#32
	BL	ZNAK
	LDR	R0,=TIPVMES
	LDRH	R3,[R0]
	BL	DECIZP
	LDR	R0,=ZNACIN
	LDRB	R1,[R0]
	TST	R1,#1
	BEQ	PARAMC1
	MOV	R0,#32
	BL	ZNAK
	LDR	R0,=KSTEVEC
	LDRB	R3,[R0,#1]
	BL	DECIZP
PARAMC1	MOV	R0,#13
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	MOV	R0,#"D"		;nastavitev D
	BL	ZNAK
	MOV	R0,#32
	BL	ZNAK
	LDR	R5,=DOMENA
	BL	IP4IZP
	MOV	R0,#32
	BL	ZNAK
	BL	IP4IZP
	MOV	R0,#32
	BL	ZNAK
	BL	IP4IZP
	MOV	R0,#13
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	MOV	R0,#"E"		;nastavitev E
	BL	ZNAK
	MOV	R0,#32
	BL	ZNAK
	LDR	R0,=ESPEED
	LDRB	R3,[R0]
	BL	DECIZP
	MOV	R0,#32
	BL	ZNAK
	LDR	R5,=ETHMAC
	BL	MACIZP
	MOV	R0,#13
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	MOV	R0,#"G"		;nastavitev G
	BL	ZNAK
	MOV	R0,#32
	BL	ZNAK
	LDR	R5,=GATEWAY
	BL	IP4IZP
	MOV	R0,#32
	BL	ZNAK
	BL	IP4IZP
	MOV	R0,#32
	BL	ZNAK
	LDR	R0,=TCACHE
	LDRH	R3,[R0]
	BL	DECIZP
	MOV	R0,#32
	BL	ZNAK
	LDR	R0,=ARPMODE
	LDRB	R3,[R0]
	AND	R3,R3,#3
	BL	DECIZP
	MOV	R0,#13
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	MOV	R0,#"H"		;nastavitev H
	BL	ZNAK
	MOV	R0,#32
	BL	ZNAK
	LDR	R0,=ZNACIN
	LDRB	R1,[R0]
	MOV	R0,#"0"
	TST	R1,#32
	MOVNE	R0,#"1"
	BL	ZNAK
	MOV	R0,#13
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	MOV	R0,#"I"		;nastavitev I
	BL	ZNAK
	LDR	R6,=INASLOV
	BL	POTIZP
	MOV	R0,#"J"		;nastavitev J
	BL	ZNAK
	LDR	R3,=JAVITXT
	LDRB	R0,[R3]
	CMP	R0,#32
	BCC	PARAMJ2
	MOV	R0,#":"
	BL	ZNAK
	MOV	R4,#MAXJAVI
PARAMJ1	LDRB	R0,[R3],#1
	CMP	R0,#32
	BCC	PARAMJ2
	BL	ZNAK
	SUBS	R4,R4,#1
	BNE	PARAMJ1
PARAMJ2	MOV	R0,#13
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	MOV	R0,#"K"		;nastavitev K
	BL	ZNAK
	MOV	R0,#32
	BL	ZNAK
	LDR	R0,=ZNACIN	;vklop/izklop izpisa ADC?
	LDRB	R1,[R0]
	MOV	R0,#"0"
	TST	R1,#64
	MOVNE	R0,#"1"
	BL	ZNAK
	MOV	R0,#32
	BL	ZNAK
	LDR	R0,=0x3FFFC037	;FIO1PIN3 P1.31
	LDRB	R3,[R0]
	MOV	R3,R3,LSR#7
	LDR	R0,=ZNACIN	;vklop/izklop TX?
	LDRB	R1,[R0]
	TST	R1,#128
	ORRNE	R3,R3,#2
	BL	DECIZP
	MOV	R0,#13
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	MOV	R0,#"L"		;nastavitev L
	BL	ZNAK
	LDR	R6,=LNASLOV
	BL	LOKIZP
	MOV	R0,#"M"		;nastavitev M
	BL	ZNAK
	MOV	R0,#32
	BL	ZNAK
	LDR	R0,=MNASLOV
	LDR	R3,[R0]
	MOVS	R3,R3
	BLNE	N36IZP
	MOV	R0,#13
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	MOV	R0,#"N"		;nastavitev N
	BL	ZNAK
	LDR	R6,=NASLOV
	BL	POTIZP
	MOV	R0,#"O"		;nastavitev O
	BL	ZNAK
	MOV	R0,#32
	BL	ZNAK
	LDR	R0,=NODZIV
	LDRH	R3,[R0]
	BL	DECIZP
	MOV	R0,#13
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	MOV	R0,#"P"		;nastavitev P
	BL	ZNAK
	MOV	R0,#32
	BL	ZNAK
	LDR	R0,=NPONOVI
	LDRB	R3,[R0]
	BL	DECIZP
	MOV	R0,#32
	BL	ZNAK
	LDR	R0,=TPONOVI
	LDR	R3,[R0]
	BL	DECIZP
	MOV	R0,#32
	BL	ZNAK
	LDR	R0,=MINBLOK
	LDRB	R3,[R0]
	BL	DECIZP
	MOV	R0,#13
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	MOV	R0,#"Q"		;nastavitev Q
	BL	ZNAK
	MOV	R0,#32
	BL	ZNAK
	LDR	R0,=QUARTZ
	LDR	R3,[R0]
	BIC	R3,R3,#0xF0000000
	BL	DECIZP
	MOV	R0,#32
	BL	ZNAK
	LDR	R0,=QUARTZ
	LDR	R3,[R0]
	MOV	R3,R3,LSR#28
	BL	DECIZP
	MOV	R0,#32
	BL	ZNAK
	LDR	R0,=QUARTZ
	LDR	R3,[R0]
	BIC	R4,R3,#0xF0000000
	MOV	R5,R3,LSR#28
	MUL	R3,R4,R5
	BL	DECIZP
	MOV	R0,#13
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	MOV	R0,#"S"		;nastavitev S
	BL	ZNAK
	MOV	R0,#32
	BL	ZNAK
	LDR	R0,=TSLOT
	LDR	R3,[R0]
	BL	DECIZP
	MOV	R0,#32
	BL	ZNAK
	LDR	R0,=TGLAVA
	LDR	R3,[R0]
	BL	DECIZP
	MOV	R0,#32
	BL	ZNAK
	LDR	R0,=TREP
	LDR	R3,[R0]
	BL	DECIZP
	MOV	R0,#13
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	MOV	R0,#"T"		;nastavitev T
	BL	ZNAK
	MOV	R0,#32
	BL	ZNAK
	LDR	R0,=TECEN1
	LDRH	R3,[R0]
	BL	DECIZP
	MOV	R0,#32
	BL	ZNAK
	LDR	R0,=TECEN2
	LDRH	R3,[R0]
	BL	DECIZP
	MOV	R0,#13
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	MOV	R0,#"U"		;nastavitev U
	BL	ZNAK
	MOV	R0,#32
	BL	ZNAK
	LDR	R0,=ZNACIN
	LDRB	R1,[R0]
	MOV	R0,#"0"
	TST	R1,#2
	MOVNE	R0,#"1"
	BL	ZNAK
	MOV	R0,#13
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	MOV	R0,#"W"		;nastavitev W
	BL	ZNAK
	MOV	R0,#32
	BL	ZNAK
	LDR	R0,=TIMEOUT
	LDR	R3,[R0]
	BL	DECIZP
	MOV	R0,#13
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	MOV	R0,#"X"		;nastavitev X
	BL	ZNAK
	MOV	R0,#32
	BL	ZNAK
	LDR	R0,=ZNACIN
	LDRB	R1,[R0]
	MOV	R0,#"0"
	TST	R1,#16
	MOVNE	R0,#"1"
	BL	ZNAK
	MOV	R0,#13
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	MOV	R0,#"Y"		;nastavitev Y
	BL	ZNAK
	LDR	R6,=NENOTA
	MOV	R5,#MENOTA
PARAMY1	MOV	R0,#32
	BL	ZNAK
	LDR	R3,[R6],#4
	BL	HEXIZP
	SUBS	R5,R5,#1
	BNE	PARAMY1
	MOV	R0,#13
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	MOV	R0,#"Z"		;nastavitev Z
	BL	ZNAK
	MOV	R0,#32
	BL	ZNAK
	LDR	R0,=ZNACIN
	LDRB	R1,[R0]
	MOV	R0,#"0"
	TST	R1,#4
	MOVNE	R0,#"1"
	BL	ZNAK
	MOV	R0,#13
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	LDR	R3,=PARAM1	;mosticka za RAM in Bootloader
	BL	TXTIZP
	LDR	R0,=0xE002C028	;PINSEL10.3 (/RTCK)?
	LDR	R1,[R0]
	MOV	R0,#"0"
	TST	R1,#8
	MOVEQ	R0,#"1"
	BL	ZNAK
	LDR	R3,=PARAM2
	BL	TXTIZP
	LDR	R0,=0x3FFFC055	;FIO2PIN1 P2.10 (/Bootloader)
	LDR	R1,[R0]
	MOV	R0,#"0"
	TST	R1,#4
	MOVNE	R0,#"1"
	BL	ZNAK
	LDR	R3,=PARAM3
	BL	TXTIZP
	LDR	R0,=ARMTAKT	;trenutni ARMTAKT
	LDR	R3,[R0]
	BL	DECIZP
	LDR	R3,=PARAM4
	BL	TXTIZP
	LDR	PC,[SP],#4
PARAM1	DCB	"*** RAM(RTCK)=",0
PARAM2	DCB	" /Bootloader(P2.10)=",0
PARAM3	DCB	" CLK=",0
PARAM4	DCB	"Hz ***",13,10,0
	ALIGN	4
	LTORG

;*** Izpis NBP poti [R6]+ *** (R0-6)
POTIZP	STR	LR,[SP,#-4]!
	MOV	R5,#MAXNAS
POTIZP1 LDR	R3,[R6],#4
	MOVS	R3,R3
	BEQ	POTIZP2
	CMP	R5,#MAXNAS
	MOVEQ	R0,#32
	MOVNE	R0,#","
	BL	ZNAK
	BL	N36IZP
	SUBS	R5,R5,#1
	BNE	POTIZP1
POTIZP2	MOV	R0,#13
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	LDR	PC,[SP],#4

;*** Izpis lokalnih naslovov [R6]+ *** (R0-6)
LOKIZP	STR	LR,[SP,#-4]!
	MOV	R5,#MAXLOK
LOKIZP1 LDR	R3,[R6],#4
	MOVS	R3,R3
	BEQ	LOKIZP2
	CMP	R5,#MAXLOK
	MOVEQ	R0,#32
	MOVNE	R0,#"+"
	BL	ZNAK
	BL	N36IZP
	SUBS	R5,R5,#1
	BNE	LOKIZP1
LOKIZP2	MOV	R0,#13
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	LDR	PC,[SP],#4

;*** Izpis NBP-ARP tabele *** (R0-6)
NBPIZP	STR	LR,[SP,#-4]!	;resi link
	LDR	R5,=ANASLOV
	MOV	R6,#MAXARP
NBPIZP1	LDR	R0,[R5,#4]	;zanka po vpisih
	MOVS	R0,R0
	BEQ	NBPIZP4
	STR	R5,[SP,#-4]!
	STR	R6,[SP,#-4]!
	LDR	R0,=ANACIN	;izpis naslova IP4 iz [R5],#4 ?
	LDRB	R3,[R0]
	TST	R3,#4
	BLEQ	IP4IZP
	LDR	R0,=ANACIN	;izpis naslova AX25 iz [R5],#4 ?
	LDRB	R3,[R0]
	TST	R3,#4
	LDRNE	R3,[R5],#4
	BLNE	N36IZP
	MOV	R6,#MAXNAS	;izpis NBP poti
NBPIZP2	LDR	R3,[R5],#4
	MOVS	R3,R3
	BEQ	NBPIZP3
	CMP	R6,#MAXNAS
	MOVEQ	R0,#">"
	MOVNE	R0,#","
	BL	ZNAK
	BL	N36IZP
	SUBS	R6,R6,#1
	BNE	NBPIZP2
NBPIZP3	MOV	R0,#13		;nova vrsta
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	LDR	R6,[SP],#4
	LDR	R5,[SP],#4
NBPIZP4	ADD	R5,R5,#AKORAK	;nov vpis v tabeli
	SUBS	R6,R6,#1
	BNE	NBPIZP1
	LDR	PC,[SP],#4
	LTORG

;*** Izpis ETH-ARP cacheja *** (R0-6)
ARPIZP	STR	LR,[SP,#-4]!	;resi link
	LDR	R5,=ECACHE
	MOV	R6,#MCACHE
ARPIZP1	SUBS	R6,R6,#1	;konec seznama?
	LDRMI	PC,[SP],#4
	LDRH	R0,[R5,#10]	;preveri veljaven vnos?
	MOVS	R0,R0
	LDREQ	PC,[SP],#4
	BL	IP4IZP		;izpis naslova IP4 iz [R5],#4
	MOV	R0,#">"		;puscica
	BL	ZNAK
	BL	MACIZP		;izpisi MAC iz (R5),#6
	MOV	R0,#32		;izpisi cas ARP odgovora
	BL	ZNAK
	LDR	R3,[R5,#2]
	BL	DECIZP
	MOV	R0,#"u"
	BL	ZNAK
	MOV	R0,#"s"
	BL	ZNAK
	MOV	R0,#"/"
	BL	ZNAK
	LDRH	R3,[R5]		;izpisi trajanje
	BL	DECIZP
	MOV	R0,#"s"
	BL	ZNAK
	MOV	R0,#13		;nova vrstica
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	ADD	R5,R5,#6
	B	ARPIZP1
	LTORG

;*** Ukaz A - IP4/AX25 usmerjanje *** (R0-6,SP+4)
AIZVEDI	LDRB	R0,[R2]
	CMP	R0,#32		;konec ukaza (R2)?
	BCC	AIZVED6
	ADDEQ	R2,R2,#1	;preskoci presledke (R2)?
	BEQ	AIZVEDI
	CMP	R0,#"!"		;! za vnos nacina usmerjanja?
	BNE	AIZVED1
	ADD	R2,R2,#1
	BL	DVHOD
	MOVS	R1,R1
	BMI	AIZVED6
	LDR	R3,=ANACIN
	STRB	R0,[R3]
	B	AIZVED6
AIZVED1	LDR	R0,=ANACIN	;vnos naslova IP4 ali AX25 (R2)?
	LDRB	R4,[R0]
	TST	R4,#4
	BNE	AIZVED2
	BL	DVHOD		;vnos prve IP stevilke (R2)?
	MOVS	R1,R1
	BMI	AIZVED6
	AND	R3,R0,#255
	BL	DVHOD		;druga IP stevilka (R2)?
	MOVS	R1,R1
	BMI	AIZVED6
	AND	R0,R0,#255
	ORR	R3,R0,R3,ROR#8
	BL	DVHOD		;tretja IP stevilka (R2)?
	MOVS	R1,R1
	BMI	AIZVED6
	AND	R0,R0,#255
	ORR	R3,R0,R3,ROR#8
	BL	DVHOD		;cetrta IP stevilka (R2)?
	MOVS	R1,R1
	BMI	AIZVED6
	AND	R0,R0,#255
	ORR	R3,R0,R3,ROR#8
	MOV	R3,R3,ROR#8
	B	AIZVED3
AIZVED2	BL	NVHOD		;AX25 naslov modulo 36 (R2)?
	MOVS	R1,R1
	BMI	AIZVED6
	MOV	R3,R0
AIZVED3	LDR	R4,=ZNASLOV
	STR	R3,[R4],#4
	MOV	R3,#MAXNAS	;vnos naslova (R2)
AIZVED4	MOV	R0,#0
	STR	R0,[R4]
	BL	NVHOD
	MOVS	R1,R1
	BMI	AIZVED5
	STR	R0,[R4],#4
	SUBS	R3,R3,#1
	BNE	AIZVED4
AIZVED5	BL	AVNOS		;vnos v tabelo ARP
AIZVED6	LDR	R0,=ANACIN	;izpis nacina IP4 ali AX25
	LDRB	R4,[R0]
	TST	R4,#4
	LDREQ	R3,=AIZVED7
	LDRNE	R3,=AIZVED8
	BL	TXTIZP
	AND	R0,R4,#7	;izpis stevilke nacina
	ADD	R0,R0,#48
	BL	ZNAK
	AND	R4,R4,#3	;izpis nacina uporabe tabele
	LDR	R3,=AIZVED9
	CMP	R4,#1
	LDREQ	R3,=AIZVE10
	CMP	R4,#2
	LDREQ	R3,=AIZVE11
	CMP	R4,#3
	LDREQ	R3,=AIZVE12
	BL	TXTIZP
	BL	NBPIZP		;izpisi NBP-ARP tabelo
	LDR	PC,[SP],#4
AIZVED7	DCB	"*** IP4 usmerjanje: ",0
AIZVED8	DCB	"*** AX25 usmerjanje: ",0
AIZVED9	DCB	"=IZKLOP",13,10,0
AIZVE10	DCB	"=BRANJE",13,10,0
AIZVE11	DCB	"=VPIS-PRIVZETO",13,10,0
AIZVE12	DCB	"=VPIS",13,10,0
	ALIGN	4
	LTORG

;*** Ukaz B - nastavi baudrate modulo UART1 *** (R0-4,SP+4)
BIZVEDI	BL	DVHOD		;vnos baudrate?
	MOVS	R1,R1
	BMI	BIZVED1
	LDR	R3,=ARMTAKT	;izracunaj modulo iz PCLK/16
	LDR	R3,[R3]
	MOV	R3,R3,LSR#4
	ADD	R3,R0,LSR#1	;zaokrozi na najblizji modulo
	BL	NDELI
	MOVS	R3,R3		;nenicelni kvocient
	MOVEQ	R3,#1
	LDR	R0,=BAUDMOD
	STRH	R3,[R0]
BIZVED1	LDR	R3,=BIZVED2	;izracunaj baudrate iz modula
	BL	TXTIZP
	LDR	R3,=ARMTAKT	;PCLK/16
	LDR	R3,[R3]
	MOV	R3,R3,LSR#4
	LDR	R0,=BAUDMOD
	LDRH	R0,[R0]
	BL	NDELI
	BL	DECIZP
	LDR	R3,=BIZVED3
	BL	TXTIZP
	LDR	PC,[SP],#4
BIZVED2	DCB	"*** Baudrate UART1: ",0
BIZVED3	DCB	"bit/s (FLASH+RESET)",13,10,0
	ALIGN	4
	LTORG

;*** Ukaz C - preklop v pogovorni nacin / kokodakanje *** (R0-3,SP+4)
CIZVEDI	BL	DVHOD		;perioda kokodakanja?
	MOVS	R1,R1
	MOVMI	R0,#0
	LDR	R1,=KSTEVEC
	STRB	R0,[R1]
	STRB	R0,[R1,#1]
	LDR	R1,=TIPVMES	;brisi veljavno vrstico
	ADD	R1,R1,#2
	MOV	R2,#MAXUKAZ
	MOV	R0,#32
CIZVED1	STRB	R0,[R1],#1
	SUBS	R2,R2,#1
	BPL	CIZVED1
	LDR	R0,=ZNACIN	;preklop v pogovorni nacin
	LDRB	R1,[R0]
	ORR	R1,R1,#1
	STRB	R1,[R0]
	LDR	R0,=KSTEVEC	;izpisi pogovorni nacin?
	LDRB	R1,[R0,#1]
	MOVS	R1,R1
	BNE	CIZVED2
	LDR	R3,=CIZVED3
	BL	TXTIZP	
	LDR	PC,[SP],#4
CIZVED2	LDR	R3,=CIZVED4	;izpisi periodo kokodakanja
	BL	TXTIZP
	LDR	R0,=KSTEVEC
	LDRB	R3,[R0,#1]
	BL	DECIZP
	LDR	R3,=CIZVED5
	BL	TXTIZP
	LDR	PC,[SP],#4
CIZVED3	DCB	"*** Pogovorni nacin ***",13,10,0
CIZVED4	DCB	"*** Kokodakanje s periodo ",0
CIZVED5	DCB	"s ***",13,10,0
	ALIGN	4
	LTORG

;*** Ukaz D - IP domena *** (R0-5,SP+4)
DIZVEDI	LDR	R5,=DOMENA
	BL	IVHOD		;vnos IP domene?
	MOVS	R1,R1
	BMI	DIZVED1
	STR	R3,[R5],#4
	BL	IVHOD		;vnos maske sprejmi?
	MOVS	R1,R1
	BMI	DIZVED1
	STR	R3,[R5],#4
	BL	IVHOD		;vnos maske zavrni?
	MOVS	R1,R1
	BMI	DIZVED1
	STR	R3,[R5],#4
DIZVED1	LDR	R3,=DIZVED2	;izpis stirih IP stevilk domene
	BL	TXTIZP
	LDR	R5,=DOMENA
	BL	IP4IZP		;izpis naslova IP4 iz [R5],#4
	LDR	R3,=DIZVED3	;izpis stirih stevilk maske sprejmi
	BL	TXTIZP
	BL	IP4IZP		;izpis naslova IP4 iz [R5],#4
	LDR	R3,=DIZVED4	;izpis stirih stevilk maske zavrni
	BL	TXTIZP
	BL	IP4IZP		;izpis naslova IP4 iz [R5],#4
	MOV	R0,#13
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	LDR	PC,[SP],#4
DIZVED2	DCB	"*** IP domena: ",0
DIZVED3	DCB	" Sprejmi: ",0
DIZVED4	DCB	" Zavrni: ",0
	ALIGN	4
	LTORG

;*** Ukaz E - Ethernet hitrost, MDIX in MAC naslov *** (R0-6,SP+4)
EIZVEDI	BL	DVHOD		;vnesi Ethernet hitrost in MDIX
	MOVS	R1,R1
	BMI	EIZVED2
	LDR	R1,=ESPEED
	STRB	R0,[R1]
	LDR	R5,=ETHMAC	;vnesi Ethernet MAC naslov
	MOV	R6,#6
EIZVED1	BL	HVHOD
	MOVS	R1,R1
	BMI	EIZVED2
	STRB	R0,[R5],#1
	SUBS	R6,R6,#1
	BNE	EIZVED1
EIZVED2	LDR	R3,=EIZVED3	;izpisi Ethernet hitrost in MDIX
	BL	TXTIZP
	LDR	R3,=EIZVED4
	LDR	R0,=ESPEED
	LDRB	R0,[R0]
	CMP	R0,#10
	LDREQ	R3,=EIZVED5
	CMP	R0,#11
	LDREQ	R3,=EIZVED6
	CMP	R0,#100
	LDREQ	R3,=EIZVED7
	CMP	R0,#101
	LDREQ	R3,=EIZVED8
	BL	TXTIZP
	LDR	R3,=EIZVED9	;izpisi Ethernet MAC naslov
	BL	TXTIZP
	LDR	R5,=ETHMAC
	BL	MACIZP
	LDR	R3,=EIZVED0
	BL	TXTIZP
	LDR	PC,[SP],#4
EIZVED3	DCB	"*** Ethernet: ",0
EIZVED4	DCB	"AutoNegotiation",0
EIZVED5	DCB	"10Mbit/s",0
EIZVED6	DCB	"10Mbit/s-X",0
EIZVED7	DCB	"100Mbit/s",0
EIZVED8	DCB	"101Mbit/s-X",0
EIZVED9	DCB	" MAC: ",0
EIZVED0	DCB	" (FLASH+RESET)",13,10,0
	ALIGN	4
	LTORG

;*** Ukaz F - vpis parametrov v FLASH *** (R0-3,SP+4/RESET)
FIZVEDI	BL	DVHOD
	LDR	R1,=123456789	;kljuc
	CMP	R0,R1
	BEQ	FLASH
	LDR	R3,=FIZVED1
	BL	TXTIZP
	LDR	PC,[SP],#4
FIZVED1	DCB	"*** Napacen kljuc ***",13,10,0
	ALIGN	4
	LTORG

;*** Ukaz G - Gateway/maska in EthernetARP cache *** (R0-7,SP+4)
GIZVEDI	LDR	R5,=GATEWAY
	BL	IVHOD		;vnos IP prizvetega prehoda?
	MOVS	R1,R1
	BMI	GIZVED1
	STR	R3,[R5],#4
	BL	IVHOD		;vnos maske privzetega prehoda?
	MOVS	R1,R1
	BMI	GIZVED1
	STR	R3,[R5],#4
	BL	DVHOD		;vstavi cache trajanje
	MOVS	R1,R1
	BMI	GIZVED1
	LDR	R1,=TCACHE
	STRH	R0,[R1]
	BL	DVHOD		;vstavi ARP mode
	MOVS	R1,R1
	BMI	GIZVED1
	LDR	R1,=ARPMODE
	STRB	R0,[R1]
GIZVED1	LDR	R3,=GIZVED2	;izpis IP privzetega prehoda
	BL	TXTIZP
	LDR	R5,=GATEWAY
	BL	IP4IZP		;izpis naslova IP4 iz [R5],#4
	LDR	R3,=GIZVED3	;izpis maske privzetega prehoda
	BL	TXTIZP
	BL	IP4IZP		;izpis naslova IP4 iz [R5],#4
	LDR	R3,=GIZVED4	;izpisi cache trajanje
	BL	TXTIZP
	LDR	R0,=TCACHE
	LDRH	R3,[R0]
	BL	DECIZP
	LDR	R3,=GIZVED5	;izpisi ARP mode
	BL	TXTIZP
	LDR	R0,=ARPMODE
	LDRB	R3,[R0]
	AND	R3,R3,#3
	BL	DECIZP
	MOV	R0,#13		;nova vrstica
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	BL	ARPIZP		;izpis ETH-ARP cacheja
	LDR	PC,[SP],#4
GIZVED2	DCB	"*** Gateway: ",0
GIZVED3	DCB	" / ",0
GIZVED4	DCB	" ARPcache: ",0
GIZVED5	DCB	"s ARPmode: ",0
	ALIGN	4
	LTORG

;*** Ukaz H - seznam slisanih postaj v obliki N36/HEX *** (R0-6,SP+4)+SSLIZP
HIZVEDI	BL	DVHOD		;izberi obliko N36/HEX?
	MOVS	R1,R1
	BMI	HIZVED1
	LDR	R2,=ZNACIN
	LDRB	R1,[R2]
	BIC	R1,R1,#32
	MOVS	R0,R0
	ORRNE	R1,R1,#32
	STRB	R1,[R2]
HIZVED1	LDR	R3,=HIZVED2
	BL	TXTIZP
	BL	SSLIZP		;izpis seznama slisanih postaj
	LDR	PC,[SP],#4
HIZVED2	DCB	"*** ",0
	ALIGN	4
	LTORG

;*** Ukaz I - IP naslov *** (R0-6,SP+4)
IIZVEDI	LDR	R6,=INASLOV	;vnos naslova
	MOV	R5,#MAXNAS
	BL	NVHOD
	MOVS	R1,R1
	BMI	IIZVED3
	STR	R0,[R6],#4
	SUB	R5,R5,#1
IIZVED1	BL	NVHOD
	MOVS	R1,R1
	BMI	IIZVED2
	STR	R0,[R6],#4
	SUBS	R5,R5,#1
	BNE	IIZVED1
	B	IIZVED3
IIZVED2	MOV	R0,#0		;nicla na koncu <MAXNAS
	STR	R0,[R6]
IIZVED3	LDR	R3,=IIZVED5	;izpis naslova
	BL	TXTIZP
	LDR	R6,=INASLOV	;INASLOV obstaja?
	LDR	R0,[R6]
	MOVS	R0,R0
	BEQ	IIZVED4
	BL	POTIZP		;izpis veljavne poti
	LDR	PC,[SP],#4
IIZVED4	LDR	R3,=IIZVED6	;izpis IZKLOP
	BL	TXTIZP
	LDR	PC,[SP],#4
IIZVED5	DCB	"*** IP/AX privzeto:",0
IIZVED6	DCB	" 0=IZKLOP",13,10,0
	ALIGN	4
	LTORG

;*** Ukaz J - besedilo javljanja na "////?" *** (R0-4,SP+4)
JIZVEDI	LDRB	R0,[R2],#1
	CMP	R0,#":"		;konec ukaza ali dvopicje za besedilo?
	BNE	JIZVED2
	LDR	R1,=JAVITXT	;vnos besedila javljanja
	MOV	R4,#MAXJAVI
JIZVED1	LDRB	R0,[R2],#1
	STRB	R0,[R1],#1
	CMP	R0,#32
	BCC	JIZVED2
	SUBS	R4,R4,#1
	BNE	JIZVED1
JIZVED2	LDR	R1,=JAVITXT	;javljanje vklopljeno?
	LDRB	R0,[R1]
	CMP	R0,#32
	BCC	JIZVED5
	LDR	R3,=JIZVED6	;izpis besedila javljanja
	BL	TXTIZP
	LDR	R3,=JAVITXT
	MOV	R4,#MAXJAVI
JIZVED3	LDRB	R0,[R3],#1
	CMP	R0,#32
	BCC	JIZVED4
	BL	ZNAK
	SUBS	R4,R4,#1
	BNE	JIZVED3
JIZVED4	MOV	R0,#13		;nova vrsta na koncu
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	LDR	PC,[SP],#4
JIZVED5	LDR	R3,=JIZVED7	;izpis javljanje izklopljeno
	BL	TXTIZP
	LDR	PC,[SP],#4
JIZVED6	DCB	"*** Javi:",0
JIZVED7	DCB	"*** Javljanje izklopljeno ***",13,10,0
	ALIGN	4
	LTORG

;*** Ukaz K - komanda in geslo z izpisom ADC v LPC23xx *** (R0-7)
KIZVEDI	MOV	R7,#0		;izpis gesla izkljucen
	BL	DVHOD
	MOVS	R1,R1
	BMI	KIZVED1
	LDR	R1,=ZNACIN	;vklop/izklop izpisa ADC
	LDRB	R3,[R1]
	BIC	R3,R3,#64
	MOVS	R0,R0
	ORRNE	R3,R3,#64
	STRB	R3,[R1]
	BL	HVHOD		;nova telekomanda?
	MOVS	R1,R1
	BMI	KIZVED1
	MOV	R7,#-1		;izpis gesla vkljucen
	TST	R0,#0x00000001
	LDR	R3,=0x3FFFC03B	;FIO1SET3
	LDREQ	R3,=0x3FFFC03F  ;FIO1CLR3
	MOV	R1,#0x80
	STRB	R1,[R3]
	LDR	R1,=ZNACIN	;vklop/izklop TX?
	LDRB	R3,[R1]
	BIC	R3,R3,#128
	TST	R0,#0x00000002
	ORRNE	R3,R3,#128
	STRB	R3,[R1]
	BL	HVHOD		;novo geslo?
	MOVS	R1,R1
	BMI	KIZVED1
	LDR	R1,=GESLO
	STR	R0,[R1]
KIZVED1	LDR	R0,=ZNACIN	;vklop/izklop sekundnega izpisa ADC?
	LDRB	R1,[R0]
	LDR	R3,=KIZVED2
	TST	R1,#64
	LDRNE	R3,=KIZVED3
	BL	TXTIZP
	BL	ADCK		;izpisi ADC telemetrijo in telekomando
	MOVS	R7,R7		;izpis gesla?
	LDR	R3,=KIZVED4
	BLMI	TXTIZP
	LDR	R0,=GESLO
	LDR	R3,[R0]
	MOVS	R7,R7
	BLMI	HEXIZP
	MOV	R0,#13		;nova vrsta
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	LDR	PC,[SP],#4
KIZVED2	DCB	"*** Izpis TLM: 0=IZKLOP ",0
KIZVED3	DCB	"*** Izpis TLM: 1=VKLOP ",0
KIZVED4	DCB	" Geslo: ",0
	ALIGN	4
	LTORG

;*** Ukaz L - naslovi lokalne zanke *** (R0-6,SP+4)
LIZVEDI	LDR	R6,=LNASLOV	;vnos naslova
	MOV	R5,#MAXLOK
	BL	NVHOD
	MOVS	R1,R1
	BMI	LIZVED3
	STR	R0,[R6],#4
	SUB	R5,R5,#1
LIZVED1	BL	NVHOD
	MOVS	R1,R1
	BMI	LIZVED2
	STR	R0,[R6],#4
	SUBS	R5,R5,#1
	BNE	LIZVED1
	B	LIZVED3
LIZVED2	MOV	R0,#0		;nicla na koncu <MAXLOK
	STR	R0,[R6]
LIZVED3	LDR	R3,=LIZVED5	;izpis lokalnih naslovov
	BL	TXTIZP
	LDR	R6,=LNASLOV
	LDR	R0,[R6]		;izklop?
	MOVS	R0,R0
	BEQ	LIZVED4
	BL	LOKIZP		;izpisi lokalne naslove
	LDR	PC,[SP],#4
LIZVED4	LDR	R3,=LIZVED6	;izpisi izklop
	BL	TXTIZP
	LDR	PC,[SP],#4
LIZVED5	DCB	"*** Lokalni naslovi:",0
LIZVED6	DCB	" 0=KISS/SLIP (izklopljen Ethernet)",13,10,0
	ALIGN	4
	LTORG

;*** Ukaz M - moj naslov *** (R0-4,SP+4)
MIZVEDI	BL	NVHOD
	MOVS	R1,R1
	BMI	MIZVED1
	LDR	R1,=MNASLOV
	STR	R0,[R1]
MIZVED1	LDR	R3,=MIZVED3
	BL	TXTIZP
	LDR	R0,=MNASLOV
	LDR	R3,[R0]
	MOVS	R3,R3
	BEQ	MIZVED2
	BL	N36IZP
	MOV	R0,#13
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	LDR	PC,[SP],#4
MIZVED2	LDR	R3,=MIZVED4
	BL	TXTIZP
	LDR	PC,[SP],#4
MIZVED3	DCB	"*** Moj naslov: ",0
MIZVED4	DCB	"0=KISS/SLIP<>Ethernet",13,10,0
	ALIGN	4
	LTORG

;*** Ukaz N - naslov *** (R0-6,SP+4)
NIZVEDI	LDR	R6,=NASLOV	;vnos naslova
	MOV	R5,#MAXNAS
	BL	NVHOD
	MOVS	R1,R1
	BMI	NIZVED3
	STR	R0,[R6],#4
	SUB	R5,R5,#1
NIZVED1	BL	NVHOD
	MOVS	R1,R1
	BMI	NIZVED2
	STR	R0,[R6],#4
	SUBS	R5,R5,#1
	BNE	NIZVED1
	B	NIZVED3
NIZVED2	MOV	R0,#0		;nicla na koncu <MAXNAS
	STR	R0,[R6]
NIZVED3	LDR	R3,=NIZVED5	;izpis naslova
	BL	TXTIZP
	LDR	R6,=NASLOV
	LDR	R0,[R6]		;NASLOV obstaja?
	MOVS	R0,R0
	BEQ	NIZVED4
	BL	POTIZP		;izpis veljavne poti
	LDR	PC,[SP],#4
NIZVED4	LDR	R3,=NIZVED6	;izpis IZKLOP
	BL	TXTIZP
	LDR	PC,[SP],#4
NIZVED5	DCB	"*** Naslov:",0
NIZVED6	DCB	" 0=IZKLOP",13,10,0
	ALIGN	4
	LTORG

;*** Ukaz O - meritev odziva stevila byte, 0=izklop *** (R0-4,SP+4)
MODZIV	EQU	1488		;max byte odziva
OIZVEDI	BL	DVHOD
	MOVS	R1,R1
	BMI	OIZVED1
	CMP	R0,#MODZIV
	MOVCS	R0,#MODZIV
	LDR	R1,=NODZIV
	STRH	R0,[R1]
OIZVED1	LDR	R3,=OIZVED2	;izpisi stevilo byte
	BL	TXTIZP
	LDR	R0,=NODZIV
	LDRH	R3,[R0]
        MOVS	R3,R3
	BEQ	OIZVED4
	BL	DECIZP
	LDR	R3,=OIZVED3
	BL	TXTIZP
	LDR	PC,[SP],#4
OIZVED4	LDR	R3,=OIZVED5	;izklop
	BL	TXTIZP
	LDR	PC,[SP],#4
OIZVED2	DCB	"*** Odziv: ",0
OIZVED3	DCB	"byte",13,10,0
OIZVED5	DCB	"0=IZKLOP",13,10,0
	ALIGN	4
	LTORG

;*** Ukaz P - stevilo ponavljanj, zakasnitev minblok *** (R0-4,SP+4)
PIZVEDI	BL	DVHOD		;stvilo ponavljanj
	MOVS	R1,R1
	BMI	PIZVED1
	LDR	R1,=NPONOVI
	STRB	R0,[R1]
	BL	DVHOD		;zakasnitev
	MOVS	R1,R1
	BMI	PIZVED1
	LDR	R1,=TPONOVI
	STR	R0,[R1]
	BL	DVHOD		;minblok
	MOVS	R1,R1
	BMI	PIZVED1
	LDR	R1,=MINBLOK
	STRB	R0,[R1]
PIZVED1	LDR	R3,=PIZVED2	;izpisi stevilo ponavljanj
	BL	TXTIZP
	LDR	R0,=NPONOVI
	LDRB	R3,[R0]
	BL	DECIZP
	LDR	R3,=PIZVED3	;izpisi zakasnitev
	BL	TXTIZP
	LDR	R0,=TPONOVI
	LDR	R3,[R0]
	BL	DECIZP
	LDR	R3,=PIZVED4	;izpisi minblok
	BL	TXTIZP
	LDR	R0,=MINBLOK
	LDRB	R3,[R0]
	BL	DECIZP
	LDR	R3,=PIZVED5
	BL	TXTIZP
	LDR	PC,[SP],#4
PIZVED2	DCB	"*** Ponovi: ",0
PIZVED3	DCB	"-krat Zakasnitev: ",0
PIZVED4	DCB	"us Min: ",0
PIZVED5	DCB	"blok",13,10,0
	ALIGN	4
	LTORG

;*** Ukaz Q - nastavi frekvenco in mnogokratnik kristala *** (R0-6,SP+4)
QIZVEDI	BL	DVHOD		;vnos frekvence?
	MOVS	R1,R1
	BMI	QIZVED1
	MOV	R4,R0
	BL	DVHOD		;vnos mnogokratnika?
	MOVS	R1,R1
	BMI	QIZVED1
	MOV	R5,R0
	BL	DVHOD		;vnos takta?
	MOVS	R1,R1
	BMI	QIZVED1
	MOV	R6,R0
	MUL	R0,R4,R5	;primerjaj frekvenca*mnogokratnik=takt?
	CMP	R0,R6
	BNE	QIZVED1
	MOV	R0,R4		;shrani novo frekvenco in mnogokratnik
	ADD	R0,R0,R5,LSL#28
	LDR	R1,=QUARTZ
	STR	R0,[R1]
QIZVED1	LDR	R3,=QIZVED2	;izpisi frekvenco
	BL	TXTIZP
	LDR	R0,=QUARTZ
	LDR	R3,[R0]
	BIC	R3,R3,#0xF0000000
	BL	DECIZP
	LDR	R3,=QIZVED3	;izpisi mnogokratnik
	BL	TXTIZP
	LDR	R0,=QUARTZ
	LDR	R3,[R0]
	MOV	R3,R3,LSR#28
	BL	DECIZP
	LDR	R3,=QIZVED4	;izpisi takt
	BL	TXTIZP
	LDR	R0,=QUARTZ
	LDR	R3,[R0]
	BIC	R4,R3,#0xF0000000
	MOV	R5,R3,LSR#28
	MUL	R3,R4,R5
	BL	DECIZP
	LDR	R3,=QIZVED5
	BL	TXTIZP
	LDR	PC,[SP],#4
QIZVED2	DCB	"*** Quartz: ",0
QIZVED3	DCB	"Hz X ",0
QIZVED4	DCB	" = ",0
QIZVED5	DCB	"Hz (FLASH+RESET)",13,10,0
	ALIGN	4
	LTORG

;*** Ukaz R - RESET *** (R0,SP+4)
RIZVEDI	LDR	R0,=0xE0000008	;WDFEED pes grize!!!
	MOV	R1,#0xAA
	STRB	R1,[R0]
	MOV	R1,#0
	STRB	R1,[R0]
	LDR	PC,[SP],#4	;naj se ne bi izvedel...
	LTORG

;*** Ukaz S - slot, glava in rep *** (R0-4,SP+4)
SIZVEDI	BL	DVHOD
	MOVS	R1,R1
	BMI	SIZVED1
	LDR	R1,=TSLOT
	STR	R0,[R1]
	BL	DVHOD
	MOVS	R1,R1
	BMI	SIZVED1
	LDR	R1,=TGLAVA
	STR	R0,[R1]
	BL	DVHOD
	MOVS	R1,R1
	BMI	SIZVED1
	LDR	R1,=TREP
	STR	R0,[R1]
SIZVED1	LDR	R3,=SIZVED2	;izpisi slot
	BL	TXTIZP
	LDR	R0,=TSLOT
	LDR	R3,[R0]
	BL	DECIZP
	LDR	R3,=SIZVED3	;izpisi glavo
	BL	TXTIZP
	LDR	R0,=TGLAVA
	LDR	R3,[R0]
	BL	DECIZP
	LDR	R3,=SIZVED4	;izpisi rep
	BL	TXTIZP
	LDR	R0,=TREP
	LDR	R3,[R0]
	BL	DECIZP
	LDR	R3,=SIZVED5
	BL	TXTIZP
	LDR	PC,[SP],#4
SIZVED2	DCB	"*** Slot: ",0
SIZVED3	DCB	"us Glava: ",0
SIZVED4	DCB	"us Rep: ",0
SIZVED5	DCB	"us",13,10,0
	ALIGN	4
	LTORG

;*** Ukaz T - tecnoba z DCD in brez DCD *** (R0-4,SP+4)
TIZVEDI	BL	DVHOD
	MOVS	R1,R1
	BMI	TIZVED1
	LDR	R1,=TECEN1
	STRH	R0,[R1]
	BL	DVHOD
	MOVS	R1,R1
	BMI	TIZVED1
	LDR	R1,=TECEN2
	STRH	R0,[R1]
TIZVED1	LDR	R3,=TIZVED2	;izpisi tecnobo z DCD
	BL	TXTIZP
	LDR	R0,=TECEN1
	LDRH	R3,[R0]
	BL	DECIZP
	LDR	R3,=TIZVED3	;izpisi tecnobo brez DCD
	BL	TXTIZP
	LDR	R0,=TECEN2
	LDRH	R3,[R0]
	BL	DECIZP
	LDR	R3,=TIZVED4
	BL	TXTIZP
	LDR	PC,[SP],#4
TIZVED2	DCB	"*** Tecnoba: ",0
TIZVED3	DCB	"/65536 brez DCD: ",0
TIZVED4	DCB	"/65536",13,10,0
	ALIGN	4
	LTORG

;*** Ukaz U - uporabni/vsi okvirji na zaslon *** (R0-4,SP+4)
UIZVEDI	BL	DVHOD		;izberi uporabni/vsi?
	MOVS	R1,R1
	BMI	UIZVED1
	LDR	R2,=ZNACIN
	LDRB	R1,[R2]
	BIC	R1,R1,#2
	MOVS	R0,R0
	ORRNE	R1,R1,#2
	STRB	R1,[R2]
UIZVED1	LDR	R3,=ZNACIN	;izpisi uporabni/vsi
	LDRB	R0,[R3]
	LDR	R3,=UIZVED2
	TST	R0,#2
	LDRNE	R3,=UIZVED3
	BL	TXTIZP
	LDR	PC,[SP],#4
UIZVED2	DCB	"*** Izpis okvirjev: 0=UPORABNI",13,10,0
UIZVED3	DCB	"*** Izpis okvirjev: 1=VSI",13,10,0
	ALIGN	4
	LTORG

;*** Ukaz V - izpis vseh nastavitev *** (R0-6,SP+4)
VIZVEDI	BL	PARAM	
	LDR	PC,[SP],#4

;*** Ukaz W - EthernetARP wait *** (R0-4,SP+4)
WIZVEDI	BL	DVHOD
	MOVS	R1,R1
	BMI	WIZVED1
	LDR	R1,=TIMEOUT
	STR	R0,[R1]
WIZVED1	LDR	R3,=WIZVED2
	BL	TXTIZP
	LDR	R0,=TIMEOUT
	LDR	R3,[R0]
	BL	DECIZP
	LDR	R3,=WIZVED3
	BL	TXTIZP
	LDR	PC,[SP],#4
WIZVED2	DCB	"*** EthernetARP WAIT: ",0
WIZVED3	DCB	"us",13,10,0
	ALIGN	4
	LTORG

;*** Ukaz X - PHY status in sogovornikov MAC *** (R0-6,SP+4)
XIZVEDI	BL	DVHOD
	MOVS	R1,R1
	BMI	XIZVED1
	LDR	R2,=ZNACIN	;vklop/izklop
	LDRB	R1,[R2]
	BIC	R1,R1,#16
	MOVS	R0,R0
	ORRNE	R1,R1,#16
	STRB	R1,[R2]
	LDR	R3,=XIZVED2
	LDRNE	R3,=XIZVED3
	BL	TXTIZP
	LDR	PC,[SP],#4
XIZVED1	BL	PHYMAC		;izpisi PHY status (R5) in sogovornikov MAC
	LDR	PC,[SP],#4
XIZVED2	DCB	"*** Izpis PHY: 0=IZKLOP",13,10,0
XIZVED3	DCB	"*** Izpis PHY: 1=VKLOP",13,10,0
	ALIGN	4
	LTORG

;*** Ukaz Y - skala in enota ADC TLM 1/2 *** (R0-6,SP+4)
YIZVEDI	LDR	R6,=NENOTA	;vnos konstant in enot (HEX)
	MOV	R5,#MENOTA
YIZVED1	BL	HVHOD
	MOVS	R1,R1
	BMI	YIZVED2
	STR	R0,[R6],#4
	SUBS	R5,R5,#1
	BNE	YIZVED1
YIZVED2	LDR	R3,=YIZVED4	;izpis konstant in enot (HEX)
	BL	TXTIZP
	LDR	R6,=NENOTA
	MOV	R5,#MENOTA
YIZVED3	MOV	R0,#32
	BL	ZNAK
	LDR	R3,[R6],#4
	BL	HEXIZP
	SUBS	R5,R5,#1
	BNE	YIZVED3
	MOV	R0,#13
	BL	ZNAK
	MOV	R0,#10
	BL	ZNAK
	LDR	PC,[SP],#4
YIZVED4	DCB	"*** Yskala/enota:",0
	ALIGN	4
	LTORG

;*** Ukaz Z - %DCD, %PTT, bloki, zanke in cas *** (R0-4,SP+4)
ZIZVEDI	BL	DVHOD
	MOVS	R1,R1
	BMI	ZIZVED1
	LDR	R2,=ZNACIN	;vklop/izklop
	LDRB	R1,[R2]
	BIC	R1,R1,#4
	MOVS	R0,R0
	ORRNE	R1,R1,#4
	STRB	R1,[R2]
	LDR	R3,=ZIZVED2
	LDRNE	R3,=ZIZVED3
	BL	TXTIZP
	LDR	PC,[SP],#4
ZIZVED1	BL	STANJE		;izpis stanja
	LDR	PC,[SP],#4
ZIZVED2	DCB	"*** Izpis stanja: 0=IZKLOP",13,10,0
ZIZVED3	DCB	"*** Izpis stanja: 1=VKLOP",13,10,0
	ALIGN	4
	LTORG

;*** Konec delovanja zbirnika ARM ***
	END
